/*
 * http.h
 *
 *  Created on: Jun 3, 2012
 *      Author: lucasschmidt
 */
#ifndef HTTP_H_
#define HTTP_H_

#include <utility/list.h>
#include <utility/path_parser.h>
#include <tcp.h>
#include <utility/string.h>
#include <utility/vector.h>

__BEGIN_SYS

/*
 // HTTP versions
 const char * HTTP_VERSION_1_0_S = "HTTP/1.0";
 const char * HTTP_VERSION_1_1_S = "HTTP/1.1";

 // methods

 const char * HTTP_METHOD_CONNECT_S = "CONNECT";
 const char * HTTP_METHOD_DELETE_S = "DELETE";
 const char * HTTP_METHOD_GET_S = "GET";
 const char * HTTP_METHOD_HEAD_S = "HEAD";
 const char * HTTP_METHOD_OPTIONS_S = "OPTIONS";
 const char * HTTP_METHOD_POST_S = "POST";
 const char * HTTP_METHOD_PUT_S = "PUT";
 const char * HTTP_METHOD_TRACE_S = "TRACE";

 // status

 const int HTTP_STATUS_CONTINUE = 100;
 const int HTTP_STATUS_SWITCHING_PROTOCOLS = 101;

 const int HTTP_STATUS_OK = 200;
 const int HTTP_STATUS_CREATED = 201;
 const int HTTP_STATUS_ACCEPTED = 202;
 const int HTTP_STATUS_NON_AUTHORITATIVE_INFORMATION = 203;
 const int HTTP_STATUS_NO_CONTENT = 204;
 const int HTTP_STATUS_RESET_CONTENT = 205;
 const int HTTP_STATUS_PARTIAL_CONTENT = 206;

 const int HTTP_STATUS_MULTIPLE_CHOICES = 300;
 const int HTTP_STATUS_MOVED_PERMANENTLY = 301;
 const int HTTP_STATUS_FOUND = 302;
 const int HTTP_STATUS_SEE_OTHER = 303;
 const int HTTP_STATUS_NOT_MODIFIED = 304;
 const int HTTP_STATUS_USE_PROXY = 305;
 const int HTTP_STATUS_TEMPORARY_REDIRECT = 307;

 const int HTTP_STATUS_BAD_REQUEST = 400;
 const int HTTP_STATUS_UNAUTHORIZED = 401;
 const int HTTP_STATUS_PAYMENT_REQUIRED = 402;
 const int HTTP_STATUS_FORBIDDEN = 403;
 const int HTTP_STATUS_NOT_FOUND = 404;
 const int HTTP_STATUS_METHOD_NOT_ALLOWED = 405;
 const int HTTP_STATUS_NOT_ACCEPTABLE = 406;
 const int HTTP_STATUS_PROXY_AUTHENICATION_REQUIRED = 407;
 const int HTTP_STATUS_REQUEST_TIME_OUT = 408;
 const int HTTP_STATUS_CONFLICT = 409;
 const int HTTP_STATUS_GONE = 410;
 const int HTTP_STATUS_LENGTH_REQUIRED = 411;
 const int HTTP_STATUS_PRECONDITION_FAILED = 412;
 const int HTTP_STATUS_REQUEST_ENTITY_TOO_LARGE = 413;
 const int HTTP_STATUS_REQUEST_URI_TOO_LARGE = 414;
 const int HTTP_STATUS_UNSUPPORTED_MEDIA_TYPE = 415;
 const int HTTP_STATUS_REQUEST_RANGE_NOT_SATISFIABLE = 416;
 const int HTTP_STATUS_EXPECTATION_FAILED = 417;

 const int HTTP_STATUS_INTERNAL_SERVER_ERROR = 500;
 const int HTTP_STATUS_NOT_IMPLEMENTED = 501;
 const int HTTP_STATUS_BAD_GATEWAY = 502;
 const int HTTP_STATUS_SERVICE_UNAVAILABLE = 503;
 const int HTTP_STATUS_GATEWAY_TIME_OUT = 504;
 const int HTTP_STATUS_HTTP_VERSION_NOT_SUPPORTED = 505;

 const char * HTTP_STATUS_OK_S = "200 OK";
 const char * HTTP_STATUS_CREATED_S = "201 Created";
 const char * HTTP_STATUS_ACCEPTED_S = "202 Accepted";
 const char * HTTP_STATUS_NON_AUTHORITATIVE_INFORMATION_S ="203 Non-Authoritative Information";
 const char * HTTP_STATUS_NO_CONTENT_S = "204 No Content";
 const char * HTTP_STATUS_RESET_CONTENT_S = "205 Reset Content";
 const char * HTTP_STATUS_PARTIAL_CONTENT_S = "206 Partial Content";

 const char * HTTP_STATUS_MULTIPLE_CHOICES_S = "300 Multiple Choices";
 const char * HTTP_STATUS_MOVED_PERMANENTLY_S = "301 Moved Permanently";
 const char * HTTP_STATUS_FOUND_S = "302 Found";
 const char * HTTP_STATUS_SEE_OTHER_S = "303 See Other";
 const char * HTTP_STATUS_NOT_MODIFIED_S = "304 Use Modified";
 const char * HTTP_STATUS_USE_PROXY_S = "305 Use Proxy";
 const char * HTTP_STATUS_TEMPORARY_REDIRECT_S = "307 Temporary Redirect";

 const char * HTTP_STATUS_BAD_REQUEST_S = "400 Bad Request";
 const char * HTTP_STATUS_UNAUTHORIZED_S = "401 Unauthorized";
 const char * HTTP_STATUS_PAYMENT_REQUIRED_S = "402 Payment Required";
 const char * HTTP_STATUS_FORBIDDEN_S = "403 Forbidden";
 const char * HTTP_STATUS_NOT_FOUND_S = "404 Not Found";
 const char * HTTP_STATUS_METHOD_NOT_ALLOWED_S = "405 Not Allowed";
 const char * HTTP_STATUS_NOT_ACCEPTABLE_S = "406 Not Acceptable";
 const char * HTTP_STATUS_PROXY_AUTHENICATION_REQUIRED_S = "407 Authentication Required";
 const char * HTTP_STATUS_REQUEST_TIME_OUT_S = "408 Request Time-out";
 const char * HTTP_STATUS_CONFLICT_S = "409 Conflict";
 const char * HTTP_STATUS_GONE_S = "410 Gone";
 const char * HTTP_STATUS_LENGTH_REQUIRED_S = "411 Length Required";
 const char * HTTP_STATUS_PRECONDITION_FAILED_S = "412 Precondition Required";
 const char * HTTP_STATUS_REQUEST_ENTITY_TOO_LARGE_S = "413 Request Entity Too Large";
 const char * HTTP_STATUS_REQUEST_URI_TOO_LARGE_S = "414 Request-URI Too Large";
 const char * HTTP_STATUS_UNSUPPORTED_MEDIA_TYPE_S = "415 Unsupported Media Type";
 const char * HTTP_STATUS_REQUEST_RANGE_NOT_SATISFIABLE_S= "416 Requested range not satisfiable";
 const char * HTTP_STATUS_EXPECTATION_FAILED_S = "417 Expectation Failed";

 const char * HTTP_STATUS_INTERNAL_SERVER_ERROR_S= "500 Internal Server Error";
 const char * HTTP_STATUS_NOT_IMPLEMENTED_S = "501 Not Implemented";
 const char * HTTP_STATUS_BAD_GATEWAY_S = "502 Bad Gateway";
 const char * HTTP_STATUS_SERVICE_UNAVAILABLE_S = "503 Service Unavailable";
 const char * HTTP_STATUS_GATEWAY_TIME_OUT_S = "504 Gateway Time-out";
 const char * HTTP_STATUS_HTTP_VERSION_NOT_SUPPORTED_S = "505 HTTP Version not supported";

 const char * SP = " ";
 const char * CRLF = "\r\n";
 const char * TWO_POINTS =":";
 */

struct string {
	char *_string;
};

struct requestLine {
	char *_method;
	char *_requestUri;
	char *_httpVersion;
};

struct header {
	char *_name;
	char *_value;
};

struct statusLine {
	char *_status;
	char *_httpVersion;
};

typedef List_Elements::Doubly_Linked<header> HeaderElement;
typedef List_Elements::Doubly_Linked<string> CharElement;

struct httpRequest {
	requestLine _requestLine;
	List<header, HeaderElement> _headers;
	char* _body;
};

struct httpResponse {
	statusLine _statusLine;
	List<header, HeaderElement> _headers;
	char* _body;

};

class HTTPRequestParser {

public:
	httpRequest _request;
	bool _parsed;
	HTTPRequestParser(const char * input) {
		_input = strcpy(_input, input);
		_parsed = false;
		parse();
	}
	httpRequest getRequest() {return _request;}
	bool didParse() {return _parsed;}

private :
	char * _input;
	bool parseRequestLine(char* requestLine);
	bool parseHeaders(char * oneHeader);
	void parse();

};

class HTTPResponseWriter {

public:

	HTTPResponseWriter(httpResponse * httpResponse) {
		_response = new char[80];
		_httpResponse = httpResponse;
		write();
	}

	char* getResponse();

private:
	httpResponse * _httpResponse;
	char* _response;
	void write();

};

class HTTPServer : public TCP::ServerSocket {
public:

	class HTTPServerHandler {

	public :
		char * path;
		HTTPServerHandler(const char * path) {
			strcpy(this->path, path);
		}
		~HTTPServerHandler () {};
		virtual httpResponse * onRequest(httpRequest req, List<char, PathParser::ArgsElement> * args);
	};

	class JsonRestResource {
	public:
		char * path;

		JsonRestResource(const char * path) {
			strcpy(this->path , path);
		}
		~JsonRestResource() {}

		virtual char * onRequest(List<char, PathParser::ArgsElement> * args);
	};

	class HTTPRestHandler : public HTTPServerHandler {

	public:
		struct restHandler {
			JsonRestResource * _handler;
			char* _path;
		};

		typedef List_Elements::Doubly_Linked<restHandler> JsonResourceElement;
		List<restHandler, JsonResourceElement> restResources;

		HTTPRestHandler(const char * path) : HTTPServerHandler(path) {}

		httpResponse * onRequest(httpRequest req, List<char, PathParser::ArgsElement> * args) {

			JsonRestResource * selected_handler;
			PathParser * parsed_path;

			JsonResourceElement *h = restResources.head();
			db<TCP>(TRC) << "HTTPRestHandler attempting to get resource for request on path=" << req._requestLine._requestUri <<"\n";
			while(h != 0) {
				//get path parser to verify if path is compatible
				PathParser * parser = new PathParser(req._requestLine._requestUri, h->object()->_path);
				if(parser->didMatch()) {
					selected_handler = h->object()->_handler;
					parsed_path = parser;
					db<TCP>(TRC) << "HTTPRestHandler found resource for request on path=" << req._requestLine._requestUri <<"\n";
					break;
				}
				h = h->next();
			}

			httpResponse * response;
			if(h) {
				db<TCP>(TRC) << "HTTPRestHandler found resource for request on path=" << req._requestLine._requestUri <<"\n";
				char * jsonResponse = h->object()->_handler->onRequest(&parsed_path->args);

				db<TCP>(TRC) << "HTTPRestHandler BUILDING RESPONSE\n";
				//build response for Rest Json web service

				response = new httpResponse();

				response->_statusLine._httpVersion = new char[strlen("HTTP/1.1")];
				strcpy(response->_statusLine._httpVersion, "HTTP/1.1");
				db<TCP>(TRC) << "HTTPRestHandler setting httpVersion\n";

				response->_statusLine._status = new char[strlen("200 OK")];
				strcpy(response->_statusLine._status, "200 OK");
				db<TCP>(TRC) << "HTTPRestHandler setting statusCode\n";

				response->_body = new char[strlen(jsonResponse)];
				strcpy(response->_body, jsonResponse);
				db<TCP>(TRC) << "HTTPRestHandler setting body\n";


				//headers
				//content len
				header * contentLenghHeader = new header();
				contentLenghHeader->_name = new char[strlen("Content-Length")];
				strcpy(contentLenghHeader->_name,"Content-Length" );
				int lenInt = strlen(jsonResponse);
				char * c = new (&lenInt) char[sizeof(lenInt)];
				contentLenghHeader->_value = new char[strlen( c)];
				strcpy(contentLenghHeader->_value,  c);
				response->_headers.insert(new HeaderElement(contentLenghHeader));
				db<TCP>(TRC) << "HTTPRestHandler setting contentLen\n";

				//content type
				header * contentTypeHeader = new header();
				contentTypeHeader->_name = new char[strlen("Content-Type")];
				strcpy(contentTypeHeader->_name,"Content-Type" );
				contentTypeHeader->_value = new char[strlen("application/json")];
				strcpy(contentTypeHeader->_value, "application/json");
				response->_headers.insert(new HeaderElement(contentTypeHeader));
				db<TCP>(TRC) << "HTTPRestHandler did build response on path=" << req._requestLine._requestUri <<"\n";
				return response;
			} else {
				db<TCP>(TRC)  << "HTTPRestHandler resource not found for path=" << req._requestLine._requestUri <<"\n";
				response = new httpResponse();
				response->_statusLine._httpVersion = new char[strlen("HTTP/1.1")];
				response->_statusLine._status = new char[strlen("404 not found")];
				strcpy(response->_statusLine._httpVersion,"HTTP/1.1");
				strcpy(response->_statusLine._status,"404 not found");
				return response;
			}
		}

		bool addHandler(JsonRestResource * h) {
			//verify if path is already mapped
			bool possible = true;
			if(restResources.size() > 0) {
				JsonResourceElement * element = restResources.head();
				while(element) {
					if(strcmp(element->object()->_path, h->path) == 0) {
						possible = true;
					}
					element = element->next();
				}
			}
			if(possible) {
				restHandler * ha = new restHandler();
				ha->_handler = h;
				ha->_path = new char[strlen(h->path)];
				strcpy(ha->_path, h->path);
				restResources.insert_tail(new JsonResourceElement(ha));
				db<TCP>(TRC) << "\nHTTP REST adding handler path=" << ha->_path << " \n";
			}
			return possible;
		}

		~HTTPRestHandler() {
			//do nothing
		}

	};

	HTTPServer(int port) : TCP::ServerSocket(TCP::Address(tcp()->ip()->address(),port), true, tcp()) {}

	struct handler {
		HTTPServerHandler * _handler;
		char* _path;
	};

	typedef List_Elements::Doubly_Linked<handler> HandlerElement;
	List<handler, HandlerElement> handlers;

	bool addHandler(HTTPServerHandler * h) {
		//verify if path is already mapped
		bool possible = true;
		if(handlers.size() > 0) {
			HandlerElement * element = handlers.head();
			while(element) {
				if(strcmp(element->object()->_path, h->path) == 0) {
					possible = true;
				}
				element = element->next();
			}
		}
		if(possible) {
			handler * ha = new handler();
			ha->_handler = h;
			ha->_path = new char[strlen(h->path)];
			strcpy(ha->_path, h->path);
			db<TCP>(TRC) << "\nHTTP adding handler path=" << ha->_path << " \n";
			handlers.insert_tail(new HandlerElement(ha));
		}
		return possible;
	}

	char * receivedLocal(const char *data,u16 size) {
		received(data, size);
		return response;
	}

private:

	char* response;
	httpResponse * resourceNotFoundRe;

	TCP::Socket* incoming(const TCP::Address& from) {
		//TODO: strategy to accept multiple connections
		return this;
	}

	void connected() {}

	void closed() {
		listen();
	}

	void error(short err) {}

	void sent(u16 size) {}

	void received(const char *data,u16 size) {
		//get req from parser
		HTTPRequestParser * parser = new HTTPRequestParser(data);
		if(!parser->_parsed) {
			return;
		}

		httpRequest req = parser->_request;

		HTTPServerHandler * selected_handler;

		PathParser * parsed_path;

		HandlerElement *h = handlers.head();
		while(h != 0) {
			//get path parser to verify if path is compatible
			db<TCP>(TRC) << "\nHTTP testing handler path=" << h->object()->_path << " for path="<< req._requestLine._requestUri << " \n";
			PathParser * parser = new PathParser(req._requestLine._requestUri, h->object()->_path);
			if(parser->didMatch()) {
				selected_handler = h->object()->_handler;
				parsed_path = parser;
				db<TCP>(TRC) << "HTTPServer handler found for request on path =" <<req._requestLine._requestUri <<"\n";
				break;
			}
			h = h->next();
		}

		httpResponse * res;
		if(selected_handler != 0) {
			res = selected_handler->onRequest(req, &parsed_path->args);
			db<TCP>(TRC) << "HTTPServer handler informed response for request on path =" <<req._requestLine._requestUri <<"\n";
		} else {
			db<TCP>(TRC) << "\nHTTP could not find handler\n";
			res = resourceNotFoundResponse();
		}
		//get writter to convert response to
		HTTPResponseWriter * wrt = new HTTPResponseWriter(res);
		response = wrt->getResponse();
		db<TCP>(TRC) << "HTTPServer handler responsed=" << response << " for request on path =" <<req._requestLine._requestUri <<"\n";
		send(response, strlen(response));
		close();
	}

	httpResponse * resourceNotFoundResponse() {
		if(resourceNotFoundRe == 0) {
			resourceNotFoundRe = new httpResponse();
			resourceNotFoundRe->_statusLine._httpVersion = new char[strlen("HTTP/1.1")];
			resourceNotFoundRe->_statusLine._status = new char[strlen("404 not found")];
			strcpy(resourceNotFoundRe->_statusLine._httpVersion,"HTTP/1.1");
			strcpy(resourceNotFoundRe->_statusLine._status,"404 not found");
		}
		return resourceNotFoundRe;
	}

};

__END_SYS

#endif /* HTTP_H_ */
