#include <ieee1451_tim.h>

// This work is licensed under the EPOS Software License v1.0.
// A copy of this license is available at the EPOS system source tree root.
// A copy of this license is also available online at:
// http://epos.lisha.ufsc.br/EPOS+Software+License+v1.0
// Note that EPOS Software License applies to both source code and executables.
//#include <mutex.h>
#include <thread.h>

#define DATASET_SIZE    160
#define READ_INTERVAL   TIME_500_MS * 125

__USING_SYS

OStream cout;

class IEEE1451_Audio_Sensor : public IEEE1451_Transducer
{
public:
    IEEE1451_Audio_Sensor(bool tim_im, bool polling);
    ~IEEE1451_Audio_Sensor();

protected:
    void init_teds();
    IEEE1451_TEDS_TIM *get_teds(char id);
    //bool running() { return run; };
    void start();
    void stop();
    void read_data_set(unsigned short trans_id, unsigned int offset);
    void send_data_set(bool first = false, bool last = false);

public:
    int execute();

private:
    unsigned short _channel_number;

    bool _tim_im;
    bool _polling;

    static char _data_set[][DATASET_SIZE];
    //Mutex _data_set_mutex;
    int _pos;

    Thread *_execute_thread;

    char *_channel_array;
    char *_temp_sensor_utn_array;
    IEEE1451_TEDS_TIM *_channel_teds;
    IEEE1451_TEDS_TIM *_temp_sensor_utn_teds;
};

//-------------------------------------------

IEEE1451_Audio_Sensor::IEEE1451_Audio_Sensor(bool tim_im, bool polling)
{
    _channel_number = 0x01; //It just supports one transducer per mote!
    _tim_im = tim_im;
    _polling = polling;
    _pos = 0;
    _execute_thread = 0;

    init_teds();
}

IEEE1451_Audio_Sensor::~IEEE1451_Audio_Sensor()
{
    if (_execute_thread)
        delete _execute_thread;

    if (_channel_array)
        kfree(_channel_array);
    if (_temp_sensor_utn_array)
        kfree(_temp_sensor_utn_array);

    if (_channel_teds)
        delete _channel_teds;
    if (_temp_sensor_utn_teds)
        delete _temp_sensor_utn_teds;
}

void IEEE1451_Audio_Sensor::init_teds()
{
    _channel_array = new (kmalloc(123)) char[123];
    _channel_array[0] = 0x00; _channel_array[1] = 0x00; _channel_array[2] = 0x00; _channel_array[3] = 0x77; _channel_array[4] = TEDS_TRANSDUCER_CHANNEL_TEDS_ID; _channel_array[5] = 0x04; _channel_array[6] = 0x00; _channel_array[7] = TEDS_TRANSDUCER_CHANNEL; _channel_array[8] = 0x01; _channel_array[9] = 0x01; _channel_array[10] = TEDS_TRANSDUCER_CHANNEL_CAL_KEY; _channel_array[11] = 0x01; _channel_array[12] = 0x00; _channel_array[13] = TEDS_TRANSDUCER_CHANNEL_CHAN_TYPE; _channel_array[14] = 0x01; _channel_array[15] = 0x00; _channel_array[16] = TEDS_TRANSDUCER_CHANNEL_PHY_UNITS; _channel_array[17] = 0x06; _channel_array[18] = TEDS_TRANSDUCER_CHANNEL_UNIT_TYPE; _channel_array[19] = 0x01; _channel_array[20] = 0x00; _channel_array[21] = TEDS_TRANSDUCER_CHANNEL_AUDIOS; _channel_array[22] = 0x01; _channel_array[23] = 0x82; _channel_array[24] = TEDS_TRANSDUCER_CHANNEL_LOW_LIMIT; _channel_array[25] = 0x04; _channel_array[26] = 0x43; _channel_array[27] = 0x69; _channel_array[28] = 0x00; _channel_array[29] = 0x00; _channel_array[30] = TEDS_TRANSDUCER_CHANNEL_HI_LIMIT; _channel_array[31] = 0x04; _channel_array[32] = 0x43; _channel_array[33] = 0xb0; _channel_array[34] = 0x80; _channel_array[35] = 0x00; _channel_array[36] = TEDS_TRANSDUCER_CHANNEL_O_ERROR; _channel_array[37] = 0x04; _channel_array[38] = 0x3f; _channel_array[39] = 0x00; _channel_array[40] = 0x00; _channel_array[41] = 0x00; _channel_array[42] = TEDS_TRANSDUCER_CHANNEL_SELF_TEST; _channel_array[43] = 0x01; _channel_array[44] = 0x00; _channel_array[45] = TEDS_TRANSDUCER_CHANNEL_SAMPLE; _channel_array[46] = 0x09; _channel_array[47] = TEDS_TRANSDUCER_CHANNEL_DAT_MODEL; _channel_array[48] = 0x01; _channel_array[49] = 0x00; _channel_array[50] = TEDS_TRANSDUCER_CHANNEL_MOD_LENGTH; _channel_array[51] = 0x01; _channel_array[52] = 0x01; _channel_array[53] = TEDS_TRANSDUCER_CHANNEL_SIG_BITS; _channel_array[54] = 0x01; _channel_array[55] = 0x08; _channel_array[56] = TEDS_TRANSDUCER_CHANNEL_DATA_SET; _channel_array[57] = 0x1c; _channel_array[58] = TEDS_TRANSDUCER_CHANNEL_REPEATS; _channel_array[59] = 0x02; _channel_array[60] = 0x00; _channel_array[61] = 0xa0; _channel_array[62] = TEDS_TRANSDUCER_CHANNEL_S_ORIGIN; _channel_array[63] = 0x04; _channel_array[64] = 0x00; _channel_array[65] = 0x00; _channel_array[66] = 0x00; _channel_array[67] = 0x00; _channel_array[68] = TEDS_TRANSDUCER_CHANNEL_STEP_SIZE; _channel_array[69] = 0x04; _channel_array[70] = 0x41; _channel_array[71] = 0x20; _channel_array[72] = 0x00; _channel_array[73] = 0x00; _channel_array[74] = TEDS_TRANSDUCER_CHANNEL_S_UNITS; _channel_array[75] = 0x06; _channel_array[76] = TEDS_TRANSDUCER_CHANNEL_UNIT_TYPE; _channel_array[77] = 0x01; _channel_array[78] = 0x00; _channel_array[79] = TEDS_TRANSDUCER_CHANNEL_SECONDS; _channel_array[80] = 0x01; _channel_array[81] = 0x82; _channel_array[82] = TEDS_TRANSDUCER_CHANNEL_PRE_TRIGG; _channel_array[83] = 0x02; _channel_array[84] = 0x00; _channel_array[85] = 0x00; _channel_array[86] = TEDS_TRANSDUCER_CHANNEL_R_SETUP_T; _channel_array[87] = 0x04; _channel_array[88] = 0x00; _channel_array[89] = 0x00; _channel_array[90] = 0x00; _channel_array[91] = 0x00; _channel_array[92] = TEDS_TRANSDUCER_CHANNEL_S_PERIOD; _channel_array[93] = 0x04; _channel_array[94] = 0x00; _channel_array[95] = 0x00; _channel_array[96] = 0x00; _channel_array[97] = 0x00; _channel_array[98] = TEDS_TRANSDUCER_CHANNEL_WARM_UP_T; _channel_array[99] = 0x04; _channel_array[100] = 0x00; _channel_array[101] = 0x00; _channel_array[102] = 0x00; _channel_array[103] = 0x00; _channel_array[104] = TEDS_TRANSDUCER_CHANNEL_R_DELAY_T; _channel_array[105] = 0x04; _channel_array[106] = 0x3f; _channel_array[107] = 0x00; _channel_array[108] = 0x00; _channel_array[109] = 0x00; _channel_array[110] = TEDS_TRANSDUCER_CHANNEL_SAMPLING; _channel_array[111] = 0x06; _channel_array[112] = TEDS_TRANSDUCER_CHANNEL_SAMP_MODE; _channel_array[113] = 0x01; _channel_array[114] = 0x04; _channel_array[115] = TEDS_TRANSDUCER_CHANNEL_S_DEFAULT; _channel_array[116] = 0x01; _channel_array[117] = 0x04; _channel_array[118] = TEDS_TRANSDUCER_CHANNEL_DATA_XMIT; _channel_array[119] = 0x01; /*_channel_array[120] = 0x02;*/ _channel_array[121] = 0xf4; _channel_array[122] = 0xa5; //checksum errado
    _channel_teds = new IEEE1451_TEDS_TIM(_channel_array, 123);

    if (!_tim_im)
        _channel_array[120] = 0x01;
    else if (_tim_im)
        _channel_array[120] = 0x02;

    _temp_sensor_utn_array = new (kmalloc(37)) char[37];
    _temp_sensor_utn_array[0] = 0x00; _temp_sensor_utn_array[1] = 0x00; _temp_sensor_utn_array[2] = 0x00; _temp_sensor_utn_array[3] = 0x27; _temp_sensor_utn_array[4] = TEDS_USER_TRANSDUCER_NAME_TEDS_ID; _temp_sensor_utn_array[5] = 0x04; _temp_sensor_utn_array[6] = 0x00; _temp_sensor_utn_array[7] = TEDS_USER_TRANSDUCER_NAME; _temp_sensor_utn_array[8] = 0x01; _temp_sensor_utn_array[9] = 0x01; _temp_sensor_utn_array[10] = TEDS_USER_TRANSDUCER_NAME_FORMAT; _temp_sensor_utn_array[11] = 0x01; _temp_sensor_utn_array[12] = 0x00; _temp_sensor_utn_array[13] = TEDS_USER_TRANSDUCER_NAME_TC_NAME; _temp_sensor_utn_array[14] = 0x14; _temp_sensor_utn_array[15] = 0x4f; _temp_sensor_utn_array[16] = 0x6e; _temp_sensor_utn_array[17] = 0x62; _temp_sensor_utn_array[18] = 0x6f; _temp_sensor_utn_array[19] = 0x61; _temp_sensor_utn_array[20] = 0x72; _temp_sensor_utn_array[21] = 0x64; _temp_sensor_utn_array[22] = 0x20; _temp_sensor_utn_array[23] = 0x41; _temp_sensor_utn_array[24] = 0x75; _temp_sensor_utn_array[25] = 0x64; _temp_sensor_utn_array[26] = 0x69; _temp_sensor_utn_array[27] = 0x6f; _temp_sensor_utn_array[28] = 0x20; _temp_sensor_utn_array[29] = 0x53; _temp_sensor_utn_array[30] = 0x65; _temp_sensor_utn_array[31] = 0x6e; _temp_sensor_utn_array[32] = 0x73; _temp_sensor_utn_array[33] = 0x6f; _temp_sensor_utn_array[34] = 0x72; _temp_sensor_utn_array[35] = 0xf5; _temp_sensor_utn_array[36] = 0x92; //checksum errado
    _temp_sensor_utn_teds = new IEEE1451_TEDS_TIM(_temp_sensor_utn_array, 37);
}

IEEE1451_TEDS_TIM *IEEE1451_Audio_Sensor::get_teds(char id)
{
    if (id == TEDS_TRANSDUCER_CHANNEL)
        return _channel_teds;
    else if (id == TEDS_USER_TRANSDUCER_NAME)
        return _temp_sensor_utn_teds;
    return 0;
}

void IEEE1451_Audio_Sensor::start()
{
    cout << "Audio sensor start\n";
    _execute_thread->resume();
}

void IEEE1451_Audio_Sensor::stop()
{
    cout << "Audio sensor stop\n";
    _execute_thread->suspend();
}

void IEEE1451_Audio_Sensor::read_data_set(unsigned short trans_id, unsigned int offset)
{
    cout << "Reading data set (polling)...\n";

    unsigned int size = sizeof(IEEE1451_Data_Set_Read_Reply) + DATASET_SIZE;
    char buffer[size];

    IEEE1451_Data_Set_Read_Reply *reply = (IEEE1451_Data_Set_Read_Reply *) buffer;
    char *data = (char *) (buffer + sizeof(IEEE1451_Data_Set_Read_Reply));

    reply->_header._success = true;
    reply->_header._length = DATASET_SIZE + sizeof(reply->_offset);
    reply->_offset = 0;

    //_data_set_mutex.lock();
    memcpy(data, _data_set[_pos], DATASET_SIZE);
    _pos = ((_pos + 1) >= 109) ? 0 : (_pos + 1);
    //_data_set_mutex.unlock();

    IEEE1451_TIM::get_instance()->send_msg(trans_id, buffer, size);
}

void IEEE1451_Audio_Sensor::send_data_set(bool first, bool last)
{
    cout << "Sending data set (tim_im)...\n";

    unsigned int size = sizeof(IEEE1451_Command) + ((first || last) ? 1 : DATASET_SIZE);
    char buffer[size];

    IEEE1451_Command *cmd = (IEEE1451_Command *) buffer;
    char *data = (char *) (buffer + sizeof(IEEE1451_Command));

    cmd->_channel_number = _channel_number;
    cmd->_command = COMMAND_CLASS_READ_TRANSDUCER_CHANNEL_DATA_SET_SEGMENT;
    cmd->_length = ((first || last) ? 1 : DATASET_SIZE);

    if ((!first) && (!last))
    {
        //_data_set_mutex.lock();
        memcpy(data, _data_set[_pos], DATASET_SIZE);
        _pos = ((_pos + 1) >= 109) ? 0 : (_pos + 1);
        //_data_set_mutex.unlock();

    }else if (first)
        data[0] = 1;

    else if (last)
        data[0] = 2;

    IEEE1451_TIM::get_instance()->send_msg(0, buffer, size);
}

int IEEE1451_Audio_Sensor::execute()
{
    cout << "Audio sensor execute thread created\n";

    _execute_thread = Thread::self();
    IEEE1451_TIM *tim = IEEE1451_TIM::get_instance();
    tim->execute();

    tim->connect();
    _execute_thread->suspend();

    while (true)
    {
        if ((_tim_im) && (!_polling))
            tim->disconnect();

        Alarm::delay(READ_INTERVAL);

        if ((_tim_im) && (!_polling))
            tim->connect();

        if (_tim_im)
        {
            send_data_set(true, false);

            for (unsigned short i = 0; i < 30/*109 * 1*/; i++)
            {
                send_data_set();
                Alarm::delay(TIME_50_MS);
            }

            send_data_set(false, true);
        }
    }

    return 0;
}

//-------------------------------------------

int main()
{
    Alarm::delay(TIME_500_MS * 4);
    kout << "+++++ Starting wtim +++++\n";

    IP *ip = IP::instance();
    ip->set_address(IP::Address(10, 0, 0, 113));
    ip->set_gateway(IP::Address(10, 0, 0, 1));
    ip->set_netmask(IP::Address(255, 255, 255, 0));

    Alarm::delay(TIME_500_MS * 4);

    IEEE1451_TIM *tim = IEEE1451_TIM::get_instance();
    tim->set_ncap_address(IP::Address(10, 0, 0, 110));

    IEEE1451_Audio_Sensor sensor(true, true);
    sensor.execute();
    return 0;
}

//-------------------------------------------

char IEEE1451_Audio_Sensor::_data_set[][DATASET_SIZE] = {
    {   //0x80, 0x80, 0x01, 0x6a, 0x00, 0x20, 0x7a, 0x10,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xfd, 0x7f, 0xfd, 0xff,
        0x7e, 0x7e, 0xff, 0x7f, 0x7e, 0x7c, 0x7c, 0x7d,
        0x7d, 0x7f, 0x7f, 0x7f, 0x7f, 0x7e, 0x7e, 0xfe,
        0x7e, 0xff, 0xfd, 0xfe, 0xfc, 0xfa, 0xfc, 0xfe,
        0xfe, 0xff, 0xff, 0xfd, 0x7f, 0xfe, 0xfe, 0x7d,
        0xff, 0x7e, 0x7d, 0x7e, 0x7d, 0x7e, 0x7d, 0x7f,
        0xff, 0x7f, 0xff, 0xff, 0xff, 0x7f, 0x7f, 0x7f,
        0xff, 0xff, 0xff, 0xff, 0x7f, 0x7f, 0x7f, 0xff,
        0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0x7f,
        0x7f, 0x7e, 0x7f, 0x7e, 0x7f, 0x7e, 0x7e, 0x7e,
        0xff, 0x7e, 0xfb, 0x7a, 0x7e, 0xf4, 0x7a, 0x7f,
        0x79, 0xfa, 0xf7, 0x7e, 0xfd, 0x7e, 0xfe, 0x78,
        0x7e, 0xf9, 0xfd, 0xfb, 0xfa, 0x7b, 0x7d, 0x7c,
        0x7d, 0xfc, 0xfd, 0xfc, 0xff, 0xff, 0x7b, 0x7c,
        0x7e, 0x7c, 0xfb, 0xf9, 0xfd, 0x7f, 0x78, 0x7c,
        0x7e, 0x7f, 0xf9, 0x7b, 0xff, 0x7e, 0x7a, 0x7c,
        0x7d, 0xfc, 0xfd, 0xfc, 0x7f, 0x77, 0x7a, 0xff,
        0xfd, 0xfa, 0xff, 0xfd, 0xfa, 0x77, 0x7a, 0x7c,
        0xfd, 0xf5, 0xfe, 0xfc, 0x7a, 0x78, 0xfd, 0x7e,
        0xfc, 0xfc, 0xfc, 0xf8, 0x7c, 0x79, 0x7e, 0x7f,
        0xfa, 0xfb, 0xfb, 0xfc},

    {   //0x80, 0x00, 0x01, 0x6b, 0x00, 0x20, 0x7a, 0xb0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x77, 0x7b, 0x7c, 0x7f,
        0xf6, 0xfc, 0xfb, 0xfd, 0x7b, 0x79, 0x7a, 0xfe,
        0xfc, 0xfc, 0xfc, 0x7e, 0x79, 0x7c, 0xfe, 0xfc,
        0xfa, 0xf8, 0xfb, 0x7e, 0x7b, 0x7b, 0x7f, 0xfa,
        0xfd, 0xfe, 0xfd, 0x7b, 0x78, 0x7b, 0x7f, 0xfc,
        0xfb, 0xfe, 0xfe, 0x7a, 0x73, 0x79, 0xff, 0xfc,
        0xfb, 0xfd, 0xff, 0x7b, 0x79, 0xfc, 0xfa, 0xf9,
        0xf9, 0xfb, 0xfc, 0x7b, 0x7c, 0x7f, 0xfd, 0xfa,
        0xfd, 0xfe, 0x7d, 0x7a, 0x7c, 0x7e, 0xfc, 0xfa,
        0xfc, 0xfe, 0x7a, 0x78, 0x7c, 0x7d, 0xfb, 0xf9,
        0xf9, 0xfb, 0x7b, 0x7e, 0xff, 0xfc, 0xf9, 0xfb,
        0xfa, 0x7e, 0x77, 0x7d, 0x7d, 0xfc, 0xf8, 0xfe,
        0xfd, 0x77, 0x75, 0x7c, 0x7e, 0xfd, 0xfe, 0x7d,
        0x7b, 0x74, 0x76, 0x7d, 0x7e, 0xfb, 0xfd, 0xfe,
        0x7c, 0x76, 0x7c, 0xfe, 0xf9, 0xf8, 0xfd, 0xfc,
        0x7b, 0x7a, 0x7e, 0x7e, 0xf9, 0xfb, 0xfb, 0xfe,
        0x79, 0x7c, 0x7e, 0xfd, 0xfa, 0xff, 0xfe, 0x7a,
        0x74, 0x7b, 0x7e, 0xfa, 0xfb, 0xfe, 0xfd, 0x7c,
        0x7a, 0xfd, 0xfc, 0xf6, 0xf9, 0xfc, 0xfd, 0x78,
        0x7c, 0xfe, 0xfa, 0xf8, 0xfb, 0xf9, 0xfe, 0x79,
        0x7e, 0x7e, 0xfc, 0xfa},

    {   //0x80, 0x00, 0x01, 0x6c, 0x00, 0x20, 0x7b, 0x50,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xfc, 0xfd, 0x79, 0x79,
        0x7e, 0xff, 0xfb, 0xfc, 0xfe, 0x7b, 0x76, 0x7a,
        0x7d, 0xfd, 0xfb, 0xfc, 0xfb, 0x7d, 0x79, 0x7e,
        0xfd, 0xf8, 0xf9, 0xfd, 0xfc, 0x7b, 0x7a, 0xfe,
        0xfb, 0xf8, 0xfb, 0xfc, 0x7d, 0x76, 0x7b, 0x7e,
        0x7f, 0xfc, 0xfd, 0xfe, 0x7a, 0x77, 0x7c, 0x7d,
        0xfd, 0xfd, 0xfe, 0xfe, 0x77, 0x77, 0x7c, 0xfe,
        0xf8, 0xfb, 0xfd, 0xfe, 0x7a, 0x7e, 0xfe, 0xfc,
        0xfa, 0xfe, 0xfe, 0x7b, 0x77, 0x7d, 0xfe, 0xfa,
        0xfc, 0xff, 0xff, 0x78, 0x7a, 0x7d, 0xff, 0xfa,
        0xfc, 0xfd, 0x7d, 0x79, 0x7d, 0x7e, 0xfb, 0xf9,
        0xfc, 0xfb, 0x7c, 0x7c, 0xfe, 0xff, 0xf9, 0xfa,
        0xfd, 0x7e, 0x75, 0x79, 0x7c, 0x7f, 0xf9, 0xfd,
        0xfe, 0x7b, 0x76, 0x7b, 0x7e, 0xfc, 0xfa, 0x7e,
        0xfc, 0x7c, 0x79, 0x7e, 0x7f, 0xf8, 0xf9, 0xfb,
        0xfd, 0x79, 0x7d, 0xfd, 0xf9, 0xf6, 0xfb, 0xf9,
        0x7e, 0x7a, 0x7c, 0x7f, 0xfa, 0xff, 0xff, 0xfe,
        0x7a, 0x7a, 0x7d, 0x7f, 0xfa, 0xfa, 0xfd, 0xff,
        0x78, 0x7d, 0xff, 0xfd, 0xf7, 0xfe, 0xfb, 0x7d,
        0x77, 0x7c, 0x7e, 0xfb, 0xfb, 0xfc, 0xfb, 0x79,
        0x79, 0x7e, 0x7e, 0xf8},

    {   //0x80, 0x00, 0x01, 0x6d, 0x00, 0x20, 0x7b, 0xf0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xfd, 0x7f, 0x7e, 0x78,
        0x7d, 0x7d, 0xfe, 0xfc, 0xff, 0xfc, 0x7b, 0x77,
        0x7e, 0x7f, 0xfa, 0xfa, 0xfc, 0xfc, 0x7b, 0x7a,
        0x7c, 0x7f, 0xfa, 0xfd, 0xfd, 0x7d, 0x76, 0x7b,
        0x7d, 0xfd, 0xfb, 0xfe, 0xfd, 0x7c, 0x77, 0x7c,
        0xff, 0xf9, 0xfc, 0xfe, 0xfc, 0x7b, 0x7c, 0x7d,
        0xfd, 0xf8, 0xfe, 0xfd, 0x7f, 0x7b, 0x7f, 0x7d,
        0xfc, 0xf8, 0xfb, 0xfb, 0x7b, 0x78, 0x7d, 0xff,
        0xfa, 0xfa, 0xfc, 0xfc, 0x77, 0x78, 0x7c, 0x7e,
        0xfb, 0xfe, 0xfe, 0x7d, 0x77, 0x7c, 0x7e, 0xfc,
        0xf9, 0xfd, 0xff, 0x79, 0x76, 0x7c, 0x7e, 0xfa,
        0xfa, 0xfd, 0xfd, 0x7a, 0x7c, 0xfd, 0xfc, 0xf7,
        0xfa, 0xfb, 0x7e, 0x78, 0x7c, 0x7d, 0xfd, 0xfc,
        0xfe, 0xfe, 0x7d, 0x7b, 0x7d, 0xff, 0xfa, 0xfd,
        0xfe, 0xfe, 0x78, 0x7b, 0x7e, 0xfc, 0xf9, 0xfb,
        0xf8, 0xfe, 0x7a, 0x7d, 0xfe, 0xf9, 0xf9, 0xfc,
        0xfc, 0x7c, 0x7b, 0xfd, 0xff, 0xfa, 0xfc, 0xfd,
        0xfd, 0x77, 0x79, 0x7d, 0xfc, 0xf8, 0xfe, 0x7f,
        0x7b, 0x74, 0x79, 0x7e, 0xfc, 0xf9, 0xfe, 0xfe,
        0x78, 0x78, 0x7d, 0xff, 0xfb, 0xfc, 0xfa, 0xfd,
        0x79, 0x7c, 0xfe, 0xf9},

    {   //0x80, 0x00, 0x01, 0x6e, 0x00, 0x20, 0x7c, 0x90,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xfa, 0x7d, 0x7f, 0x79,
        0x76, 0x7a, 0x7d, 0xfc, 0xfb, 0xfc, 0xfe, 0x79,
        0x78, 0x7e, 0x7e, 0xfc, 0xfe, 0xfe, 0x7d, 0x76,
        0x7a, 0x7d, 0xfd, 0xfa, 0xfe, 0xfd, 0x7c, 0x76,
        0x7d, 0xfe, 0xf9, 0xfa, 0xfc, 0xfb, 0x7e, 0x7c,
        0xff, 0xfc, 0xf9, 0xfa, 0xfb, 0xfd, 0x7b, 0x7d,
        0x7d, 0xfb, 0xfa, 0xfe, 0xfb, 0x7d, 0x79, 0x7e,
        0xfc, 0xf8, 0xf9, 0xfa, 0xfb, 0x7a, 0x7a, 0x7c,
        0xfe, 0xfa, 0xfd, 0xfe, 0x7f, 0x7b, 0xff, 0x7e,
        0xfd, 0xfb, 0x7e, 0xfb, 0x7c, 0x78, 0x7d, 0x7e,
        0xfc, 0xfb, 0xfe, 0xff, 0x79, 0x7d, 0xfd, 0xfe,
        0xf9, 0xfb, 0xfc, 0x7e, 0x76, 0x7a, 0x7c, 0xfd,
        0xfa, 0xfe, 0xfe, 0x7a, 0x77, 0x7b, 0xfe, 0xf9,
        0xfc, 0xfd, 0x7e, 0x76, 0x79, 0x7c, 0xff, 0xfb,
        0xff, 0xfc, 0x7d, 0x79, 0x7d, 0x7f, 0xfb, 0xfa,
        0xfd, 0xfd, 0x7a, 0x7b, 0x7f, 0xff, 0xf8, 0xfb,
        0xfd, 0xfc, 0x7b, 0x7e, 0x7e, 0xfe, 0xf9, 0xfc,
        0xfc, 0x7e, 0x79, 0x7b, 0x7e, 0xfa, 0xf8, 0xfc,
        0xfd, 0x79, 0x77, 0x7d, 0xfe, 0xf9, 0xfd, 0xfb,
        0xfe, 0x7b, 0x7c, 0xfe, 0xfd, 0xf8, 0x7e, 0xfb,
        0x79, 0x77, 0x7d, 0x6f},

    {   //0x80, 0x00, 0x01, 0x6f, 0x00, 0x20, 0x7d, 0x30,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xed, 0xfa, 0x79, 0x77,
        0x7c, 0xf7, 0x7b, 0xfd, 0xff, 0x7a, 0x79, 0xf5,
        0x77, 0xfd, 0x7d, 0x78, 0xfd, 0x78, 0xf9, 0xfc,
        0xff, 0x7e, 0x7d, 0x7e, 0xfa, 0xfe, 0xf7, 0xfa,
        0x7e, 0x7d, 0x77, 0xf9, 0xfe, 0xf6, 0xfd, 0x77,
        0x7a, 0x76, 0xfd, 0xfa, 0xfa, 0xfc, 0xff, 0x70,
        0x75, 0x7e, 0xf7, 0xf3, 0x7e, 0xf8, 0x76, 0x76,
        0x7b, 0xf8, 0xef, 0xfb, 0xf8, 0x78, 0x7a, 0x7b,
        0xfc, 0xf6, 0xf8, 0xf9, 0x7b, 0x7a, 0x77, 0xfa,
        0xf9, 0xf5, 0xfb, 0x7a, 0x7d, 0x78, 0xfc, 0xfb,
        0xf6, 0xf9, 0x7c, 0x7d, 0x7b, 0x7e, 0xfc, 0xfa,
        0xfe, 0x7f, 0x79, 0x7f, 0x7f, 0x7d, 0xfc, 0x7e,
        0x7e, 0x7c, 0xfe, 0xfb, 0x7d, 0x7c, 0x7d, 0x78,
        0x7b, 0xff, 0xfa, 0xfd, 0x79, 0x7a, 0x76, 0x7d,
        0xfa, 0xfb, 0xfb, 0x7a, 0x78, 0x76, 0x7b, 0xfb,
        0xf9, 0xfa, 0x7f, 0x78, 0x74, 0x7a, 0xfd, 0xf4,
        0xfa, 0xfc, 0x7d, 0x72, 0x7a, 0x7f, 0xf5, 0xfb,
        0xfd, 0x7d, 0x73, 0x7a, 0xff, 0xf8, 0xf9, 0xfc,
        0x7b, 0x7a, 0x78, 0x7f, 0xf9, 0xf9, 0xf6, 0x7a,
        0x7e, 0x7d, 0xff, 0xfa, 0xfc, 0xf9, 0x7f, 0xfe,
        0xfc, 0xfc, 0xff, 0xfc},

    {   //0x80, 0x00, 0x01, 0x70, 0x00, 0x20, 0x7d, 0xd0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xfd, 0x7e, 0xfd, 0xfd,
        0xfa, 0x7b, 0x7e, 0x7c, 0x7b, 0xfc, 0xfb, 0xf8,
        0x7f, 0x7a, 0x77, 0x79, 0xfe, 0xf4, 0xf5, 0xfa,
        0x7d, 0x74, 0x7c, 0xfe, 0xf5, 0xf7, 0xfa, 0xfc,
        0x74, 0x79, 0x7c, 0xf9, 0xf7, 0xfb, 0xfb, 0x75,
        0x76, 0x7d, 0xfb, 0xf8, 0xf9, 0x7e, 0x7a, 0x78,
        0x7b, 0xfa, 0xfb, 0xf8, 0x7c, 0x7a, 0x7d, 0x7b,
        0xfd, 0xfd, 0xfa, 0x7d, 0x7a, 0x7f, 0x7f, 0x7d,
        0x7e, 0xfe, 0x7d, 0xff, 0x7f, 0xfd, 0x7d, 0x7b,
        0x7b, 0x76, 0x7e, 0x7e, 0xfa, 0x7f, 0x79, 0x7a,
        0x77, 0xff, 0xfb, 0xf8, 0xfb, 0x7d, 0x76, 0x75,
        0x7b, 0xfa, 0xf4, 0xf9, 0xfa, 0x76, 0x75, 0x7e,
        0xf7, 0xf2, 0xf8, 0xf8, 0x78, 0x75, 0x7b, 0xfd,
        0xf7, 0xfb, 0xfd, 0x77, 0x76, 0x77, 0xfd, 0xfc,
        0xf9, 0xfc, 0x7a, 0x7e, 0x79, 0xfb, 0xfb, 0xf8,
        0xfc, 0x7b, 0xfe, 0xfc, 0xfa, 0xfb, 0xf8, 0xfe,
        0x7f, 0xfc, 0xfa, 0xfd, 0x7c, 0x7e, 0x7b, 0xfe,
        0xfe, 0xfa, 0xfa, 0x7b, 0x7c, 0x76, 0x7a, 0xfc,
        0xf8, 0xfb, 0x7e, 0x78, 0x74, 0x7b, 0xfd, 0xf6,
        0xfb, 0xfa, 0x7a, 0x73, 0x7c, 0xfc, 0xf2, 0xfa,
        0xf8, 0x7d, 0x75, 0x7f},

    {   //0x80, 0x00, 0x01, 0x71, 0x00, 0x20, 0x7e, 0x70,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xfc, 0xf4, 0xf7, 0xfb,
        0x7c, 0x76, 0x77, 0xfe, 0xfb, 0xfc, 0xfd, 0x76,
        0x7b, 0x79, 0x7d, 0xff, 0x7e, 0xfc, 0x7a, 0x7e,
        0x7e, 0x7f, 0xff, 0xfd, 0xff, 0x7e, 0x7d, 0x7f,
        0xfa, 0x7c, 0xfd, 0x7c, 0x7d, 0xfb, 0xfa, 0xf9,
        0x7b, 0x7e, 0x7a, 0x7a, 0xfc, 0xf9, 0xf5, 0xfc,
        0x7d, 0x78, 0x78, 0xfe, 0xf9, 0xfc, 0xfb, 0x7c,
        0x72, 0x7a, 0x7c, 0xf6, 0xf7, 0xfa, 0xff, 0x72,
        0x79, 0x7e, 0xf9, 0xfb, 0xfc, 0x7c, 0x76, 0x76,
        0x7d, 0xf9, 0xfa, 0xf7, 0x7d, 0x7d, 0x7a, 0x7b,
        0xfd, 0xfe, 0xf9, 0x7b, 0x7c, 0xfe, 0xff, 0xfd,
        0xfc, 0xfa, 0x7d, 0x7e, 0xfb, 0xfa, 0xfb, 0xfd,
        0x7d, 0x7b, 0xfe, 0xfc, 0xf8, 0xfe, 0x7e, 0x7b,
        0x78, 0xfd, 0xf9, 0xf6, 0xfe, 0x7d, 0x78, 0x77,
        0x7b, 0xfb, 0xf8, 0xfc, 0xfd, 0x75, 0x7a, 0xfe,
        0xf6, 0xf5, 0xfd, 0xfc, 0x74, 0x75, 0x7c, 0xf9,
        0xf6, 0xfa, 0xfc, 0x79, 0x77, 0x7b, 0xfb, 0xfa,
        0xfa, 0x7e, 0x78, 0x7a, 0x79, 0xfe, 0x7f, 0xfb,
        0x7e, 0x78, 0x7d, 0x7c, 0xfd, 0xff, 0xfb, 0x7f,
        0x7b, 0x7f, 0xfc, 0x7f, 0x7e, 0xfd, 0x7b, 0xff,
        0xff, 0xf8, 0xf9, 0x7e},

    {   //0x80, 0x00, 0x01, 0x72, 0x00, 0x20, 0x7f, 0x10,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x7e, 0x76, 0x7c, 0xfc,
        0xf8, 0xfc, 0x7e, 0x78, 0x75, 0x7d, 0xfa, 0xf5,
        0xfa, 0xf9, 0x79, 0x76, 0x7d, 0xfe, 0xf6, 0xfd,
        0xfb, 0x76, 0x74, 0x7e, 0xfc, 0xf4, 0xf8, 0xfa,
        0x7d, 0x79, 0x7c, 0xf9, 0xf9, 0xf8, 0xf9, 0x7c,
        0x7c, 0x7a, 0xfb, 0xf9, 0xf8, 0xfb, 0x79, 0x7e,
        0x7d, 0x7e, 0x7e, 0xff, 0x7e, 0x7a, 0x7d, 0xff,
        0xff, 0x7c, 0xfe, 0x7c, 0x7d, 0xff, 0xfc, 0xfb,
        0x7c, 0xfe, 0x7a, 0xff, 0xfb, 0xfc, 0xf8, 0x7f,
        0x7c, 0x76, 0x79, 0xfe, 0xfa, 0xfc, 0xfe, 0x78,
        0x72, 0x79, 0x7f, 0xf6, 0xf9, 0xf8, 0x7d, 0x74,
        0x7c, 0x7c, 0xf8, 0xf7, 0xfa, 0x7e, 0x77, 0x7a,
        0xff, 0xfa, 0xfa, 0xf8, 0x7d, 0x7b, 0x79, 0x7f,
        0xfc, 0xfe, 0xfa, 0x7c, 0xff, 0x7d, 0x7d, 0xff,
        0xff, 0xfd, 0x79, 0x7b, 0x7f, 0xfc, 0x7e, 0xfd,
        0x7f, 0x7b, 0x7e, 0xff, 0xf9, 0x7d, 0xfe, 0x7d,
        0x7a, 0xfe, 0xfc, 0xf4, 0xfc, 0x7e, 0x79, 0x7c,
        0xfb, 0xf7, 0xf6, 0xfb, 0x7f, 0x77, 0x7e, 0x7f,
        0xf7, 0xf7, 0xfc, 0x7e, 0x71, 0x7a, 0x7f, 0xf9,
        0xf8, 0xfc, 0xfe, 0x77, 0x78, 0x7d, 0xfa, 0xf9,
        0xf9, 0xff, 0x7a, 0x79},

    {   //0x80, 0x00, 0x01, 0x73, 0x00, 0x20, 0x7f, 0xb0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x7c, 0xfa, 0xfb, 0xf9,
        0x7c, 0x7c, 0x7f, 0x7c, 0xfd, 0xfd, 0xfb, 0x7e,
        0x7c, 0x7e, 0xfe, 0x7c, 0x7d, 0x7e, 0x7a, 0x7e,
        0x7e, 0xfb, 0x7c, 0x7c, 0x7d, 0x78, 0x7e, 0x7e,
        0xfb, 0x7e, 0x79, 0x78, 0x77, 0xfe, 0xfb, 0xf9,
        0xfc, 0x7f, 0x77, 0x78, 0xff, 0xfb, 0xf3, 0xf8,
        0xf9, 0x76, 0x76, 0x7f, 0xfb, 0xf3, 0xfb, 0xfa,
        0x79, 0x76, 0x7b, 0xfb, 0xf8, 0xfb, 0xfb, 0x79,
        0x79, 0x78, 0xfd, 0xfb, 0xf9, 0xfd, 0x7b, 0x7f,
        0x7b, 0xff, 0xfe, 0xfb, 0xfb, 0x7a, 0xff, 0xfd,
        0xfd, 0xfd, 0x78, 0xef, 0x7f, 0x6c, 0xef, 0xf4,
        0xfd, 0x77, 0x6d, 0xfa, 0xf5, 0xf4, 0x79, 0x71,
        0xff, 0xf9, 0xf9, 0x7c, 0x7d, 0xfa, 0xf8, 0xfd,
        0x74, 0x6f, 0x7d, 0xee, 0xf2, 0xfc, 0x72, 0x6f,
        0xfd, 0xf7, 0xf1, 0x76, 0x77, 0xfa, 0x7d, 0xff,
        0x7e, 0x79, 0xfb, 0xef, 0x7f, 0x75, 0x70, 0xfd,
        0xf3, 0xf6, 0xf9, 0x6d, 0x76, 0xfc, 0xf7, 0xfa,
        0x75, 0x7c, 0xfb, 0xf8, 0xfb, 0x78, 0x73, 0xf8,
        0xf6, 0xfc, 0x78, 0x76, 0xf7, 0xf9, 0xf8, 0x75,
        0x6e, 0xff, 0xf3, 0xf0, 0x7f, 0x70, 0x74, 0xfa,
        0xf3, 0xf9, 0x79, 0x79},

    {   //0x80, 0x00, 0x01, 0x74, 0x00, 0x20, 0x80, 0x50,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xfb, 0xfd, 0xfe, 0x77,
        0x7c, 0xf5, 0xf2, 0xfd, 0x6d, 0x6f, 0xfc, 0xee,
        0xf5, 0xfe, 0x72, 0x74, 0xfc, 0xf9, 0xf9, 0x77,
        0xfb, 0xf6, 0xfe, 0x78, 0x71, 0xfe, 0xf1, 0xef,
        0x7f, 0x72, 0x77, 0xf9, 0xf4, 0xfe, 0x77, 0x74,
        0xfb, 0xf7, 0xff, 0x73, 0x76, 0xf7, 0xf7, 0xfc,
        0x75, 0x78, 0xfb, 0xf5, 0xfb, 0x70, 0x74, 0xfa,
        0xf0, 0xfa, 0x75, 0x70, 0x7d, 0xf3, 0xf5, 0xfd,
        0x77, 0xfe, 0xfb, 0xfe, 0x79, 0x77, 0xf8, 0xf0,
        0xf5, 0x73, 0x6e, 0x7c, 0xef, 0xef, 0xfc, 0x78,
        0x77, 0xfb, 0xf9, 0xfc, 0x79, 0x7d, 0xf7, 0xfe,
        0x77, 0x70, 0x7d, 0xf3, 0xef, 0xfc, 0x70, 0x76,
        0xfb, 0xf3, 0xfb, 0x7a, 0x78, 0xfb, 0xf5, 0xf9,
        0x78, 0x75, 0xf7, 0xf6, 0xfa, 0x78, 0x76, 0xfd,
        0xf7, 0xf8, 0x72, 0x71, 0xff, 0xf3, 0xf8, 0x76,
        0x6f, 0x76, 0xf9, 0xf8, 0x7e, 0x72, 0x7a, 0xfc,
        0x7f, 0x7a, 0x71, 0xfa, 0xf1, 0xf3, 0x77, 0x6c,
        0x79, 0xf4, 0xef, 0xfb, 0x79, 0x74, 0x7e, 0xf7,
        0xfa, 0x7d, 0x7c, 0xf5, 0xf9, 0x7c, 0x6f, 0x78,
        0xf3, 0xf1, 0xf7, 0x70, 0x72, 0xfd, 0xf3, 0xf7,
        0x7c, 0x79, 0xfe, 0xf2},

    {   //0x80, 0x00, 0x01, 0x75, 0x00, 0x20, 0x80, 0xf0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xf9, 0x7a, 0x76, 0xf6,
        0xf1, 0xfb, 0x77, 0x71, 0xfd, 0xf8, 0xf7, 0x78,
        0x71, 0xfd, 0xf1, 0xf2, 0x79, 0x70, 0x78, 0xf5,
        0xf4, 0xfe, 0x77, 0x7b, 0xfc, 0xff, 0x7b, 0x73,
        0xfc, 0xf2, 0xf2, 0x7c, 0x6d, 0x77, 0xf7, 0xee,
        0xf6, 0x7b, 0x78, 0x7c, 0xfb, 0xfd, 0x7a, 0x7a,
        0xf6, 0xf7, 0x79, 0x6f, 0x70, 0xf7, 0xf3, 0xf8,
        0x73, 0x6f, 0x7d, 0xf8, 0xf7, 0x7b, 0x79, 0x7d,
        0xf8, 0xfa, 0x79, 0x72, 0xfe, 0xf5, 0xfc, 0x78,
        0x73, 0xfd, 0xf7, 0xf5, 0x7a, 0x6f, 0x7e, 0xf5,
        0xf3, 0x7c, 0x73, 0x7a, 0xfa, 0xf4, 0xfa, 0x7c,
        0x7d, 0xf9, 0xfe, 0x7a, 0x73, 0x7b, 0xf5, 0xf5,
        0xfd, 0x6f, 0x70, 0xfa, 0xef, 0xf3, 0x7d, 0x78,
        0x7d, 0xf8, 0xfa, 0x7d, 0x7b, 0xf7, 0xf4, 0x7e,
        0x74, 0x72, 0xf7, 0xef, 0xf3, 0x7a, 0x6f, 0x7d,
        0xf7, 0xf6, 0x7d, 0x78, 0x7e, 0xfa, 0xf9, 0x79,
        0x73, 0xfe, 0xf4, 0xf9, 0x7a, 0x73, 0x7d, 0xf7,
        0xf7, 0x7f, 0x70, 0x78, 0xf5, 0xf0, 0xfd, 0x73,
        0x7b, 0xfc, 0xf6, 0xfb, 0x78, 0x78, 0xfd, 0xfc,
        0x7a, 0x72, 0x76, 0xf5, 0xf4, 0xfa, 0x71, 0x70,
        0xfc, 0xf3, 0xf3, 0x7a},

    {   //0x80, 0x00, 0x01, 0x76, 0x00, 0x20, 0x81, 0x90,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x78, 0x7b, 0xfd, 0xfd,
        0x7d, 0x7b, 0xfa, 0xf3, 0xfe, 0x74, 0x6f, 0xfd,
        0xf2, 0xf3, 0x7d, 0x6e, 0x7a, 0xfa, 0xf4, 0xfe,
        0x76, 0x7f, 0xf8, 0xf7, 0x7d, 0x75, 0x7d, 0xf6,
        0xf9, 0x7d, 0x72, 0x79, 0xf9, 0xfb, 0xfb, 0x73,
        0x77, 0xf7, 0xf2, 0xfa, 0x74, 0x77, 0xff, 0xf7,
        0xf9, 0x79, 0x79, 0xfb, 0xf9, 0x7d, 0x76, 0x77,
        0xf5, 0xf2, 0xf9, 0x74, 0x6e, 0xfe, 0xf6, 0xf5,
        0x7e, 0x79, 0x7e, 0xfe, 0xf7, 0xff, 0x7b, 0xfb,
        0xf2, 0xf9, 0x77, 0x72, 0xfc, 0xef, 0xf2, 0xfa,
        0x71, 0x7a, 0xfa, 0xf7, 0xfd, 0x76, 0x7d, 0xff,
        0xfa, 0x7c, 0x72, 0x79, 0xf5, 0xf4, 0x7c, 0x72,
        0x79, 0xf8, 0xf6, 0xf9, 0x75, 0x74, 0xfa, 0xf5,
        0xf8, 0x76, 0x77, 0xfa, 0xf6, 0xf8, 0x7a, 0x78,
        0xfb, 0xf7, 0x7f, 0x75, 0x73, 0xfa, 0xf2, 0xf7,
        0x79, 0x6e, 0x7c, 0xf6, 0xf4, 0xfe, 0x76, 0x7a,
        0x7e, 0xfd, 0x79, 0x77, 0xfe, 0xf4, 0xf8, 0x76,
        0x6f, 0x7b, 0xf1, 0xf2, 0xfb, 0x71, 0x75, 0xfe,
        0xf9, 0xf9, 0x75, 0x7c, 0xfd, 0xfb, 0x7e, 0x72,
        0x77, 0xfa, 0xf4, 0xfd, 0x74, 0x75, 0xfb, 0xf9,
        0xfc, 0x76, 0x73, 0xfa},

    {   //0x80, 0x00, 0x01, 0x77, 0x00, 0x20, 0x82, 0x30,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xf4, 0xf3, 0x7a, 0x75,
        0xfb, 0xf8, 0xf7, 0x7c, 0x7a, 0xfc, 0xf5, 0xf9,
        0x7a, 0x75, 0xfb, 0xef, 0xf3, 0xfe, 0x71, 0x7b,
        0xf5, 0xf2, 0xfa, 0x76, 0x7d, 0xff, 0xfc, 0x7e,
        0x78, 0xfd, 0xf5, 0xf5, 0x7b, 0x71, 0x77, 0xf6,
        0xf0, 0xf9, 0x75, 0x74, 0xfd, 0xf9, 0xf8, 0x74,
        0x79, 0xf8, 0xf9, 0xfe, 0x73, 0x78, 0xfa, 0xf3,
        0xfb, 0x73, 0x72, 0xfe, 0xf8, 0xfa, 0x7c, 0x71,
        0x7d, 0xf7, 0xf7, 0x7a, 0x71, 0x7f, 0xf9, 0xf8,
        0x7b, 0x78, 0xfe, 0xf9, 0xfa, 0x79, 0x75, 0xff,
        0xf4, 0xf5, 0x7e, 0x6e, 0x75, 0xfa, 0xf4, 0xfa,
        0x74, 0x7a, 0xfe, 0x7e, 0x7e, 0x78, 0xfe, 0xf5,
        0xf4, 0x7c, 0x6d, 0x74, 0xf8, 0xf1, 0xf5, 0x7a,
        0x72, 0xff, 0xfb, 0xf9, 0x7d, 0x7a, 0xf8, 0xfb,
        0xfb, 0x76, 0x75, 0xfa, 0xf4, 0xf6, 0x79, 0x76,
        0xfc, 0xf5, 0xf7, 0xfe, 0x75, 0x7d, 0xf5, 0xf4,
        0xfd, 0x72, 0x7e, 0xf9, 0xf6, 0xfd, 0x78, 0xfe,
        0xfa, 0xf9, 0x7a, 0x74, 0x7d, 0xf4, 0xf2, 0xfd,
        0x72, 0x73, 0xfd, 0xf2, 0xf6, 0x79, 0x79, 0xf9,
        0xfe, 0xff, 0x77, 0x7d, 0xf7, 0xf4, 0xfc, 0x6e,
        0x77, 0x79, 0xea, 0xfe},

    {   //0x80, 0x00, 0x01, 0x78, 0x00, 0x20, 0x82, 0xd0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x6d, 0xf8, 0x7d, 0x75,
        0xfe, 0xf3, 0x73, 0x72, 0xfb, 0xee, 0x75, 0x6b,
        0x7c, 0xed, 0xf2, 0x6d, 0x78, 0xf7, 0xfd, 0x78,
        0xfc, 0xfb, 0x71, 0x7d, 0xef, 0xf5, 0x6d, 0x6d,
        0xed, 0xf2, 0x75, 0x72, 0xfa, 0xef, 0x7b, 0x73,
        0x7f, 0xfe, 0x7d, 0xf4, 0xef, 0x77, 0x6b, 0x7e,
        0xeb, 0xf5, 0x6e, 0x76, 0xf0, 0x7f, 0x70, 0x77,
        0xf7, 0xf6, 0x7e, 0xf4, 0xfc, 0x6d, 0x74, 0xef,
        0xeb, 0x7c, 0x6c, 0xfa, 0xf0, 0x7d, 0x75, 0x7f,
        0xef, 0x7c, 0x7b, 0xf8, 0x78, 0x71, 0x7e, 0xec,
        0xfd, 0x6b, 0x7a, 0xed, 0xf5, 0x74, 0x79, 0xf9,
        0xfd, 0x7a, 0xf6, 0xfa, 0x73, 0x7a, 0xf2, 0xf2,
        0x6d, 0x73, 0xee, 0xf2, 0x73, 0x6d, 0xfc, 0xf8,
        0xfe, 0xf6, 0xf6, 0x74, 0x73, 0xf1, 0xec, 0x7b,
        0x6f, 0xf1, 0xf2, 0x77, 0x6e, 0xfe, 0xef, 0x7c,
        0xfe, 0xfa, 0x75, 0x6f, 0xfb, 0xec, 0xff, 0x6c,
        0x79, 0xef, 0x7f, 0x72, 0x79, 0xf5, 0xfd, 0x76,
        0xfa, 0x7b, 0x70, 0x7b, 0xef, 0xf7, 0x6a, 0x72,
        0xee, 0xf1, 0x75, 0x75, 0xf8, 0xfc, 0x79, 0xf9,
        0xf7, 0x73, 0x75, 0xf5, 0xf2, 0x71, 0x6f, 0xef,
        0xee, 0x77, 0x6d, 0xff},

    {   //0x80, 0x00, 0x01, 0x79, 0x00, 0x20, 0x83, 0x70,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xf2, 0xfe, 0xfa, 0xf4,
        0x78, 0x6f, 0xfa, 0xed, 0xfe, 0x6d, 0xf8, 0xed,
        0x7b, 0x6e, 0x78, 0xf1, 0x7e, 0x7c, 0xfa, 0x76,
        0x6f, 0x7d, 0xed, 0xf8, 0x6d, 0x76, 0xf0, 0xf9,
        0x71, 0x77, 0xf3, 0xf8, 0x77, 0xfe, 0xfe, 0x76,
        0x7c, 0xef, 0xef, 0x6f, 0x6f, 0xef, 0xed, 0x7a,
        0x73, 0xf9, 0xf6, 0x7b, 0xfb, 0xf4, 0x77, 0x76,
        0xf8, 0xf0, 0x78, 0x6d, 0xf4, 0xed, 0x7e, 0x6d,
        0x7a, 0xf3, 0x7f, 0xff, 0xf7, 0x7c, 0x6f, 0xff,
        0xee, 0xfa, 0x6d, 0x7d, 0xed, 0x7f, 0x6e, 0x76,
        0xef, 0xf4, 0x7e, 0xf6, 0x7d, 0x70, 0x7c, 0xee,
        0xf1, 0x6f, 0x6f, 0xf4, 0xf7, 0x70, 0x71, 0xf6,
        0xf3, 0x77, 0x7c, 0xfb, 0x76, 0x77, 0xf4, 0xee,
        0x72, 0x6c, 0xf5, 0xee, 0x7d, 0x6f, 0xff, 0xf6,
        0x7b, 0x7f, 0xf5, 0x7b, 0x71, 0xfe, 0xf1, 0x7d,
        0x6c, 0xfb, 0xec, 0xfc, 0x6e, 0x73, 0xf4, 0xfd,
        0x7d, 0xf9, 0x7e, 0x6f, 0x74, 0xf2, 0xf7, 0x6e,
        0x79, 0xed, 0xf7, 0x6f, 0x72, 0xf1, 0xf1, 0x7b,
        0xfb, 0x7f, 0x74, 0x78, 0xf0, 0xed, 0x74, 0x6f,
        0xf7, 0xf1, 0x77, 0x71, 0xf9, 0xf0, 0x7d, 0x7a,
        0xf7, 0x7c, 0x78, 0xf9},

    {   //0x80, 0x00, 0x01, 0x7a, 0x00, 0x20, 0x84, 0x10,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xed, 0x7b, 0x6b, 0xfc,
        0xed, 0xfb, 0x6f, 0x7e, 0xf4, 0x7d, 0x79, 0xf6,
        0xfc, 0x77, 0x7e, 0xf0, 0xfc, 0x6b, 0x7c, 0xec,
        0xf5, 0x70, 0x6f, 0xf5, 0xf8, 0x7b, 0xf9, 0xfd,
        0x75, 0x74, 0xf5, 0xf2, 0x71, 0x6f, 0xef, 0xf4,
        0x6f, 0x6e, 0xfb, 0xf1, 0x78, 0x7e, 0xfe, 0x74,
        0x74, 0xf7, 0xec, 0x7b, 0x6f, 0xf7, 0xee, 0x7c,
        0x70, 0xfb, 0xee, 0xfd, 0x78, 0xfc, 0x7d, 0x76,
        0xfd, 0xee, 0x7f, 0x6a, 0x7a, 0xef, 0xf9, 0x6f,
        0x76, 0xf4, 0x7e, 0x75, 0xfc, 0xf9, 0x75, 0x79,
        0xf3, 0xf6, 0x6f, 0x78, 0xed, 0xf0, 0x76, 0x6e,
        0xf9, 0xf6, 0x7a, 0xfd, 0xfb, 0x77, 0x70, 0xfa,
        0xf1, 0x76, 0x6f, 0xf5, 0xee, 0x76, 0x6c, 0xfe,
        0xef, 0xfe, 0xff, 0xf8, 0x78, 0x72, 0xfd, 0xec,
        0xfb, 0x6f, 0xfe, 0xef, 0xfc, 0x6e, 0x7d, 0xef,
        0xf9, 0x78, 0xfe, 0xfe, 0x77, 0x7e, 0xef, 0xf9,
        0x6b, 0x74, 0xf0, 0xf3, 0x74, 0x73, 0xf2, 0xf8,
        0x77, 0xfd, 0xf8, 0x7a, 0x78, 0xf5, 0xf4, 0x6f,
        0x6f, 0xef, 0xef, 0x78, 0x6e, 0xfe, 0xf0, 0x7d,
        0xfb, 0xf8, 0x7b, 0x75, 0x7e, 0xee, 0x7c, 0x6f,
        0xf7, 0xee, 0x7c, 0x6b},

    {   //0x80, 0x00, 0x01, 0x7b, 0x00, 0x20, 0x84, 0xb0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x78, 0xef, 0xfc, 0x7a,
        0xfc, 0x78, 0x71, 0x7c, 0xee, 0xf7, 0x6f, 0x7c,
        0xef, 0xf8, 0x71, 0x77, 0xef, 0xf4, 0x78, 0xfe,
        0xfb, 0x78, 0x7a, 0xf4, 0xf2, 0x6e, 0x6f, 0xf1,
        0xf1, 0x76, 0x6f, 0xf5, 0xf3, 0x77, 0x7c, 0xf9,
        0x7e, 0x75, 0xfb, 0xf5, 0x71, 0x6c, 0xf7, 0xee,
        0x7b, 0x6e, 0x78, 0xf2, 0x7c, 0x7b, 0xfa, 0x7f,
        0x78, 0x7c, 0xef, 0xfd, 0x70, 0xfb, 0xec, 0xf9,
        0x6e, 0x77, 0xee, 0xf5, 0x79, 0xf9, 0xfd, 0x73,
        0x77, 0xf3, 0xf5, 0x6e, 0x76, 0xf2, 0xf6, 0x6f,
        0x6f, 0xf3, 0xf3, 0x7a, 0x7a, 0xfd, 0x7a, 0x78,
        0xf5, 0xee, 0x74, 0x6e, 0xf4, 0xef, 0x7c, 0x6e,
        0xfb, 0xf1, 0x79, 0x7d, 0xf8, 0xfc, 0x79, 0xfd,
        0xf0, 0x7c, 0x6e, 0xfa, 0xec, 0xfd, 0x6f, 0x76,
        0xf1, 0xfb, 0x7b, 0xf8, 0xfa, 0x7a, 0x78, 0xf3,
        0xfb, 0x6e, 0x79, 0xef, 0xf8, 0x6d, 0x6f, 0xf5,
        0xf5, 0x7a, 0xff, 0xfd, 0x76, 0x75, 0xf5, 0xf0,
        0x74, 0x72, 0xf4, 0xf2, 0x75, 0x6e, 0xf6, 0xef,
        0x7d, 0x79, 0xfe, 0x7d, 0x7a, 0xf8, 0xee, 0x7e,
        0x6d, 0xfa, 0xee, 0xfe, 0x6f, 0x7d, 0xf0, 0x7b,
        0x78, 0xfc, 0xfc, 0x7a},

    {   //0x80, 0x00, 0x01, 0x7c, 0x00, 0x20, 0x85, 0x50,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x7b, 0xf3, 0x7e, 0x6d,
        0x7c, 0xed, 0xf8, 0x6f, 0x72, 0xf3, 0xf8, 0x77,
        0xfc, 0xfb, 0x7c, 0x77, 0xf9, 0xf5, 0x71, 0x78,
        0xf0, 0xf1, 0x73, 0x6e, 0xf8, 0xf0, 0x7c, 0x7c,
        0xfa, 0x79, 0x76, 0xfb, 0xee, 0x79, 0x70, 0xf9,
        0xf3, 0x79, 0x6d, 0xfd, 0xef, 0xfe, 0x79, 0xfd,
        0x7f, 0x79, 0xfc, 0xef, 0xfd, 0x6e, 0xfe, 0xee,
        0xfa, 0x70, 0x78, 0xef, 0xfb, 0x75, 0x7e, 0xfb,
        0x7c, 0x78, 0xf8, 0xfd, 0x6d, 0x74, 0xf2, 0xf5,
        0x70, 0x6f, 0xfb, 0xf5, 0x77, 0xfd, 0xfa, 0x7f,
        0x78, 0x7d, 0xea, 0x6b, 0x76, 0xe9, 0xfd, 0x75,
        0x76, 0xfd, 0xfe, 0xf4, 0xfb, 0x7d, 0x72, 0xfa,
        0xfc, 0xfc, 0xf2, 0x74, 0x79, 0xf6, 0xee, 0x6c,
        0x73, 0xe8, 0xf2, 0x6b, 0x7b, 0xed, 0x72, 0x78,
        0xed, 0xef, 0x66, 0x75, 0xe7, 0xff, 0x6e, 0x7e,
        0xef, 0x78, 0x77, 0xf2, 0x7e, 0x78, 0xf7, 0xf1,
        0x7d, 0x72, 0xfd, 0xf6, 0xf3, 0xf9, 0x6f, 0x74,
        0xf1, 0xf7, 0x73, 0x7c, 0xf4, 0x77, 0x6e, 0xf2,
        0xfa, 0x69, 0xfe, 0xe9, 0x78, 0x64, 0xf9, 0xea,
        0x77, 0x7a, 0xef, 0x79, 0x6c, 0xf7, 0xee, 0x76,
        0x72, 0xf4, 0xfc, 0x7a},

    {   //0x80, 0x00, 0x01, 0x7d, 0x00, 0x20, 0x85, 0xf0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x78, 0x73, 0xfe, 0xf3,
        0x7f, 0x6d, 0x7b, 0xf9, 0x73, 0xfe, 0xed, 0x75,
        0x6a, 0xf1, 0xec, 0x70, 0x6d, 0xeb, 0xf2, 0x6c,
        0xfe, 0xf2, 0x76, 0x7a, 0xed, 0xf8, 0x6b, 0x78,
        0xf3, 0xfa, 0xfd, 0x7c, 0x78, 0xf9, 0xf4, 0x7b,
        0x78, 0xf3, 0xf8, 0x6f, 0xf9, 0xf3, 0x6e, 0x78,
        0xe8, 0xf3, 0x66, 0x76, 0xe9, 0x7e, 0x74, 0xee,
        0xf7, 0x6e, 0xff, 0xed, 0xff, 0x70, 0xf3, 0xf3,
        0x7a, 0x78, 0x75, 0x7f, 0xf4, 0xf7, 0x72, 0x75,
        0xf5, 0x7a, 0x7a, 0xed, 0xfa, 0x6b, 0xfd, 0xea,
        0x78, 0x6a, 0xef, 0xeb, 0x6e, 0x77, 0xf1, 0x7d,
        0x74, 0xef, 0xef, 0x6d, 0x72, 0xef, 0xf7, 0x7d,
        0x7d, 0x79, 0x7e, 0xf6, 0x7d, 0x71, 0xfe, 0xf7,
        0x71, 0x7a, 0xf2, 0x73, 0x70, 0xec, 0xed, 0x68,
        0x6b, 0xec, 0xf8, 0x6e, 0xf7, 0xf3, 0x6e, 0x74,
        0xef, 0xf7, 0x6e, 0xfb, 0xef, 0x79, 0x77, 0x77,
        0x7e, 0xf7, 0xf6, 0x79, 0x6f, 0xf8, 0xfd, 0x76,
        0xf3, 0xf4, 0x6d, 0x75, 0xec, 0xfd, 0x68, 0xff,
        0xe7, 0x76, 0x6f, 0xf1, 0xfa, 0x72, 0xf8, 0xeb,
        0x76, 0x6f, 0xf3, 0xf1, 0x7d, 0x7b, 0x7d, 0x7d,
        0xf9, 0xfb, 0x74, 0x7a},

    {   //0x80, 0x00, 0x01, 0x7e, 0x00, 0x20, 0x86, 0x90,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xf4, 0x78, 0x74, 0xf3,
        0x7d, 0x6f, 0xf3, 0xe9, 0x71, 0x68, 0xee, 0xef,
        0x6f, 0x7c, 0xee, 0x78, 0x6e, 0xf1, 0xf1, 0x70,
        0x7a, 0xed, 0xfd, 0x73, 0x77, 0x7e, 0xf7, 0xf5,
        0xfe, 0x6f, 0x7d, 0xf9, 0x75, 0xf9, 0xef, 0x74,
        0x6f, 0xf0, 0xf4, 0x6b, 0x74, 0xe9, 0xfc, 0x6c,
        0xf8, 0xf3, 0x74, 0x7d, 0xeb, 0xfe, 0x6c, 0xf8,
        0xee, 0xfd, 0x7c, 0xfe, 0xfd, 0xfb, 0xf7, 0x7b,
        0x7b, 0xf1, 0xfd, 0x73, 0xfc, 0xff, 0x6f, 0xfd,
        0xe9, 0x7c, 0x65, 0xfb, 0xec, 0x74, 0x74, 0xf0,
        0xfb, 0x6d, 0xfa, 0xee, 0x71, 0x71, 0xef, 0xf7,
        0x72, 0x77, 0xff, 0xfa, 0xf6, 0xf9, 0x72, 0x78,
        0xf6, 0x7b, 0x7e, 0xf1, 0x7d, 0x6e, 0xf9, 0xf0,
        0x6d, 0x6c, 0xeb, 0xf0, 0x6a, 0x7b, 0xf0, 0x79,
        0x74, 0xee, 0xf6, 0x6a, 0x7c, 0xed, 0xfb, 0x78,
        0xfc, 0xfb, 0x7e, 0xfa, 0x7f, 0x76, 0xf9, 0xf6,
        0x74, 0x7c, 0xfb, 0x73, 0x7a, 0xed, 0xf6, 0x68,
        0x77, 0xeb, 0x7c, 0x70, 0xf6, 0xf5, 0x6e, 0x7a,
        0xed, 0x7c, 0x6f, 0xf1, 0xee, 0x75, 0x72, 0x7e,
        0xf9, 0xfa, 0xf7, 0x7a, 0x74, 0xf8, 0xfe, 0x7a,
        0xf6, 0xf8, 0x74, 0xfc},

    {   //0x80, 0x00, 0x01, 0x7f, 0x00, 0x20, 0x87, 0x30,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xec, 0x75, 0x6b, 0xef,
        0xeb, 0x6e, 0x70, 0xf1, 0xfd, 0x6f, 0xf4, 0xee,
        0x6d, 0x71, 0xee, 0xf7, 0x75, 0x7c, 0xf7, 0xfd,
        0xfe, 0xfc, 0x77, 0xfe, 0xf3, 0x7c, 0x79, 0xf8,
        0x7b, 0x75, 0xf0, 0xef, 0x6b, 0x6d, 0xec, 0xf8,
        0x6d, 0x7e, 0xf1, 0x71, 0x6e, 0xef, 0xfc, 0x6c,
        0xfb, 0xeb, 0x7c, 0x6f, 0x7e, 0xf9, 0xfa, 0xf6,
        0xfc, 0x72, 0xfd, 0xfa, 0x7a, 0xfb, 0xf8, 0x76,
        0x78, 0xf0, 0x7e, 0x6b, 0xf9, 0xe8, 0x77, 0x6c,
        0xf8, 0xf8, 0x70, 0xfd, 0xeb, 0x71, 0x6d, 0xf1,
        0xf3, 0x75, 0x77, 0xf5, 0xfc, 0x7c, 0xfd, 0x7b,
        0x7e, 0xf4, 0xfa, 0x79, 0xfd, 0xff, 0x78, 0xf6,
        0xec, 0x77, 0x6c, 0xef, 0xef, 0x70, 0x79, 0xee,
        0x7c, 0x6c, 0xf6, 0xf4, 0x6d, 0x77, 0xec, 0xfd,
        0x6c, 0x79, 0xf6, 0xfd, 0xfd, 0xfa, 0x77, 0x7c,
        0xf7, 0x7e, 0xff, 0xf5, 0x7f, 0x76, 0xf4, 0xf6,
        0x6c, 0x7a, 0xe9, 0xfa, 0x6b, 0xfe, 0xf0, 0x76,
        0x78, 0xec, 0x7f, 0x69, 0xfb, 0xee, 0x7a, 0x72,
        0xf7, 0xf7, 0x79, 0xfc, 0x7e, 0x7b, 0xf8, 0xf6,
        0x78, 0x7b, 0xfb, 0x7b, 0xfe, 0xee, 0xff, 0x69,
        0xfb, 0xee, 0x72, 0x6f},

    {   //0x80, 0x00, 0x01, 0x80, 0x00, 0x20, 0x87, 0xd0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xf4, 0xf8, 0x6c, 0xff,
        0xef, 0x6f, 0x71, 0xee, 0xf5, 0x6d, 0x74, 0xf5,
        0xfb, 0x7f, 0xfa, 0x7b, 0x78, 0xf9, 0xfd, 0x7d,
        0xf9, 0xf9, 0x7a, 0xf9, 0xf0, 0x74, 0x75, 0xeb,
        0xed, 0x6e, 0x78, 0xef, 0x7b, 0x71, 0xef, 0xf4,
        0x6b, 0x7b, 0xee, 0x7d, 0x6d, 0xff, 0xf3, 0x75,
        0x78, 0xfe, 0x7a, 0xfc, 0xf4, 0x7e, 0x78, 0x7f,
        0x7d, 0x7e, 0xef, 0xf5, 0x6c, 0x7b, 0xeb, 0x7e,
        0x6f, 0xf7, 0xee, 0x72, 0x77, 0xee, 0x78, 0x6e,
        0xf2, 0xef, 0x6f, 0x6d, 0xf7, 0xf9, 0x7b, 0xfc,
        0x7d, 0x73, 0xfe, 0xfa, 0x7c, 0x7f, 0xf9, 0x7c,
        0x7b, 0xf4, 0x77, 0x6f, 0xf1, 0xed, 0x71, 0x6e,
        0xf1, 0xfa, 0x71, 0xf5, 0xef, 0x6f, 0x71, 0xee,
        0xf5, 0x6f, 0x7c, 0xee, 0x7a, 0x75, 0xfe, 0x7b,
        0x7e, 0xf7, 0xfb, 0x76, 0x7d, 0xfe, 0x7d, 0xf5,
        0xf1, 0x70, 0x72, 0xec, 0xf7, 0x6f, 0xfe, 0xed,
        0x7a, 0x6f, 0xf0, 0xfc, 0x6d, 0xfe, 0xec, 0x79,
        0x6a, 0xfa, 0xf0, 0x7d, 0xfc, 0xfa, 0x77, 0x7b,
        0xfa, 0x7c, 0x7a, 0xfc, 0xfb, 0x76, 0xf8, 0x7b,
        0x6d, 0xf9, 0xea, 0x7f, 0x6c, 0xf4, 0xf2, 0x77,
        0x79, 0xe6, 0x6d, 0x73},

    {   //0x80, 0x00, 0x01, 0x81, 0x00, 0x20, 0x88, 0x70,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xef, 0x7c, 0xf3, 0x63,
        0xe8, 0xef, 0x65, 0xed, 0x77, 0x7c, 0xed, 0x6f,
        0xf7, 0x78, 0x75, 0xec, 0x6c, 0xee, 0xfa, 0x65,
        0xe7, 0x7d, 0x6d, 0xf2, 0x7d, 0xeb, 0x6b, 0x75,
        0xe4, 0x65, 0xfb, 0xe8, 0x6d, 0x7d, 0x70, 0xf1,
        0xef, 0x68, 0xeb, 0x7b, 0x6d, 0xe9, 0x6c, 0xfc,
        0xf7, 0x70, 0xec, 0x6e, 0xfd, 0x7d, 0x6b, 0xdf,
        0x7a, 0x6a, 0xee, 0x76, 0xf5, 0x6f, 0xfe, 0xea,
        0x62, 0xed, 0xef, 0x6a, 0xf8, 0x78, 0xea, 0x78,
        0x6a, 0xea, 0x6d, 0xf5, 0xed, 0x6b, 0xf3, 0x71,
        0xfd, 0xee, 0x6f, 0xee, 0x6b, 0x7e, 0xe5, 0x65,
        0xfe, 0xf4, 0xf9, 0xf6, 0x69, 0xea, 0x77, 0x6b,
        0xe1, 0x75, 0x6e, 0x79, 0x7c, 0xe8, 0x6a, 0xfa,
        0xf2, 0x68, 0xeb, 0x76, 0x79, 0xf0, 0x71, 0xeb,
        0x75, 0x77, 0xf7, 0x68, 0xe5, 0xf4, 0x68, 0xf4,
        0x79, 0xef, 0x76, 0x77, 0xe6, 0x65, 0xfb, 0xe7,
        0x6c, 0xfc, 0x75, 0xea, 0xf6, 0x65, 0xea, 0x74,
        0x7b, 0xea, 0x6c, 0xf8, 0x76, 0x7c, 0xec, 0x6b,
        0xf1, 0x6f, 0x70, 0xe2, 0x6a, 0x75, 0xf6, 0xfe,
        0xef, 0x67, 0xf0, 0xf7, 0x65, 0xe4, 0xfb, 0x6e,
        0x79, 0x77, 0xe5, 0x6e},

    {   //0x80, 0x00, 0x01, 0x82, 0x00, 0x20, 0x89, 0x10,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x75, 0xeb, 0x6a, 0xef,
        0xfd, 0x6f, 0xf1, 0x6e, 0xed, 0x7b, 0x6e, 0xf1,
        0x68, 0xeb, 0xeb, 0x67, 0xf9, 0x7b, 0xf1, 0xfe,
        0x6d, 0xe5, 0x6a, 0x72, 0xe5, 0x6e, 0x7e, 0x72,
        0xee, 0xeb, 0x63, 0xef, 0x7d, 0x71, 0xe9, 0x70,
        0xfb, 0x78, 0x73, 0xe8, 0x6e, 0xf7, 0xfe, 0x6b,
        0xe4, 0x73, 0x6e, 0xf6, 0x7b, 0xec, 0x69, 0xfb,
        0xec, 0x64, 0xeb, 0xf0, 0x6f, 0x79, 0x6f, 0xe6,
        0x77, 0x6c, 0xe8, 0x6d, 0xf9, 0xf8, 0x6d, 0xf2,
        0x6e, 0xef, 0xf8, 0x6a, 0xef, 0x6b, 0xf5, 0xe7,
        0x6a, 0xfd, 0x77, 0xf6, 0xf3, 0x6a, 0xe7, 0x78,
        0x6b, 0xe7, 0x73, 0x7a, 0x73, 0xf6, 0xe5, 0x64,
        0xfb, 0xf2, 0x6f, 0xe9, 0x78, 0xfe, 0x7d, 0x6f,
        0xe8, 0x72, 0x7f, 0xf3, 0x68, 0xe7, 0xfe, 0x6c,
        0xf6, 0x79, 0xe9, 0x6d, 0x72, 0xe8, 0x67, 0xf3,
        0xeb, 0x73, 0x7d, 0x6d, 0xe7, 0xf6, 0x66, 0xea,
        0x74, 0x7a, 0xf1, 0x6e, 0xef, 0x6f, 0xf5, 0xeb,
        0x6a, 0xee, 0x70, 0x7a, 0xe5, 0x6c, 0x7d, 0x7b,
        0xfb, 0xed, 0x66, 0xed, 0xf6, 0x66, 0xeb, 0x79,
        0x77, 0x72, 0x76, 0xe3, 0x67, 0x76, 0xed, 0x6d,
        0xed, 0x7a, 0x7b, 0xfd},

    {   //0x80, 0x00, 0x01, 0x83, 0x00, 0x20, 0x89, 0xb0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x6b, 0xe9, 0x78, 0x72,
        0xee, 0x68, 0xed, 0xf4, 0x6c, 0xf8, 0x72, 0xea,
        0x78, 0x69, 0xe8, 0x6a, 0x7c, 0xe9, 0x74, 0x7c,
        0x6a, 0xec, 0xec, 0x64, 0xee, 0x7d, 0x74, 0xee,
        0x6e, 0xf2, 0x73, 0x7e, 0xe6, 0x69, 0xf7, 0x7b,
        0x70, 0xe4, 0x73, 0x7d, 0x7e, 0x7d, 0xe7, 0x6a,
        0xf5, 0xed, 0x68, 0xed, 0xfb, 0x7b, 0x79, 0x6f,
        0xe1, 0x6f, 0x6b, 0xeb, 0x6e, 0xef, 0xfc, 0x7a,
        0xf3, 0x6a, 0xeb, 0xf9, 0x6e, 0xeb, 0x6e, 0xf4,
        0xed, 0x6c, 0xf7, 0x72, 0xed, 0xf3, 0x66, 0xe8,
        0x76, 0x75, 0xe9, 0x79, 0xfb, 0x6b, 0xf6, 0xe6,
        0x66, 0xf4, 0xf4, 0x73, 0xf0, 0x6e, 0xf6, 0x77,
        0x70, 0xe6, 0x6b, 0x79, 0xfe, 0x6c, 0xe7, 0x7a,
        0x78, 0x7d, 0x70, 0xe9, 0x6d, 0x76, 0xea, 0x6a,
        0xf5, 0xfa, 0x76, 0x7c, 0x69, 0xe4, 0x7d, 0x63,
        0xed, 0x72, 0xf8, 0xf8, 0x75, 0xef, 0x69, 0xf2,
        0xee, 0x6b, 0xee, 0x71, 0x7f, 0xed, 0x6d, 0xf7,
        0x74, 0xf5, 0xed, 0x64, 0xef, 0xfe, 0x6d, 0xec,
        0x78, 0xfa, 0x6c, 0x78, 0xe2, 0x67, 0x77, 0xf2,
        0x70, 0xf1, 0x6f, 0xf5, 0xfc, 0x6c, 0xe6, 0x74,
        0x76, 0xf2, 0x6b, 0xe9},

    {   //0x80, 0x00, 0x01, 0x84, 0x00, 0x20, 0x8a, 0x50,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xfb, 0x76, 0xf7, 0x6e,
        0xe7, 0x75, 0x6e, 0xe7, 0x6f, 0xfa, 0xf2, 0x7a,
        0xfa, 0x6a, 0xe8, 0xec, 0x64, 0xef, 0x7c, 0xfb,
        0xf3, 0x72, 0xed, 0x6c, 0xff, 0xe8, 0x6b, 0xf5,
        0x7a, 0x79, 0xe9, 0x6f, 0xfb, 0x76, 0x7e, 0xe7,
        0x67, 0xfa, 0xf2, 0x6c, 0xed, 0x7e, 0xf5, 0x75,
        0x6e, 0xe0, 0x6e, 0x6e, 0xee, 0x74, 0xf3, 0x71,
        0x7e, 0xf3, 0x6a, 0xe8, 0xfe, 0x6b, 0xf2, 0x6c,
        0xed, 0xf5, 0x72, 0xf3, 0x6c, 0xec, 0xfa, 0x67,
        0xea, 0x75, 0x7b, 0xf3, 0x75, 0xf6, 0x69, 0xef,
        0xe7, 0x64, 0xfa, 0xfe, 0x7c, 0xf3, 0x6d, 0xed,
        0x6f, 0x71, 0xe7, 0x6d, 0xfa, 0xff, 0x74, 0xec,
        0x6f, 0x7e, 0x7b, 0x74, 0xe6, 0x6b, 0x75, 0xee,
        0x6d, 0xf0, 0x7e, 0xf9, 0x7b, 0x67, 0xe3, 0x7d,
        0x6a, 0xef, 0x75, 0xf3, 0x73, 0x78, 0xed, 0x68,
        0xee, 0xef, 0x6c, 0xf1, 0x70, 0xf2, 0xef, 0x6f,
        0xf2, 0x6d, 0xf2, 0xee, 0x68, 0xed, 0xfe, 0x77,
        0xf2, 0x75, 0xf2, 0x6b, 0x7e, 0xe2, 0x68, 0x79,
        0xfa, 0x7b, 0xf1, 0x6e, 0xef, 0xfe, 0x6b, 0xe7,
        0x72, 0x7b, 0xfd, 0x74, 0xeb, 0x76, 0x7f, 0xfd,
        0x75, 0xe9, 0xfc, 0x69},

    {   //0x80, 0x00, 0x01, 0x85, 0x00, 0x20, 0x8a, 0xf0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xe8, 0x70, 0xf7, 0xf5,
        0x6f, 0xec, 0x63, 0xec, 0xef, 0x68, 0xf1, 0x76,
        0xf2, 0x7e, 0x6e, 0xed, 0x6b, 0xf5, 0xed, 0x69,
        0xf4, 0x6f, 0xfd, 0xef, 0x6c, 0xf1, 0x6d, 0x7b,
        0xea, 0x6a, 0xf5, 0xfa, 0x7b, 0xef, 0x72, 0xf0,
        0x77, 0x74, 0xe3, 0x70, 0x75, 0xf7, 0x78, 0xee,
        0x6f, 0xf6, 0xfb, 0x67, 0xea, 0x7a, 0x70, 0xf4,
        0x71, 0xed, 0x77, 0x77, 0xf0, 0x6a, 0xeb, 0xf8,
        0x6b, 0xf2, 0x70, 0xf6, 0xfc, 0x75, 0xef, 0x68,
        0xf4, 0xed, 0x69, 0xf6, 0x7b, 0xf7, 0xfb, 0x6d,
        0xee, 0x6e, 0x7a, 0xe8, 0x6c, 0xff, 0x77, 0x7b,
        0xed, 0x6d, 0xf2, 0x78, 0x70, 0xe8, 0x6e, 0x7f,
        0xf6, 0x77, 0xed, 0x70, 0xf7, 0x7d, 0x6d, 0xe3,
        0x7c, 0x72, 0xf5, 0x77, 0xed, 0x76, 0xff, 0xef,
        0x67, 0xee, 0xf9, 0x6e, 0xf4, 0x71, 0xed, 0x7d,
        0x6f, 0xee, 0x6b, 0xf1, 0xef, 0x6b, 0xf6, 0x75,
        0xf9, 0xf6, 0x70, 0xed, 0x6b, 0x7c, 0xe8, 0x6b,
        0xfa, 0x7f, 0xfb, 0xf5, 0x6c, 0xee, 0x78, 0x71,
        0xe6, 0x75, 0x7a, 0x7d, 0x7b, 0xea, 0x6f, 0xf7,
        0xfd, 0x6b, 0xea, 0x72, 0x74, 0xf7, 0x74, 0xed,
        0x71, 0x7d, 0xf6, 0x69},

    {   //0x80, 0x00, 0x01, 0x86, 0x00, 0x20, 0x8b, 0x90,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xea, 0xf7, 0x6e, 0xf7,
        0x76, 0xef, 0x77, 0x74, 0xec, 0x69, 0xf7, 0xee,
        0x6e, 0xf9, 0x72, 0xf1, 0xfb, 0x6b, 0xef, 0x6d,
        0x7f, 0xec, 0x6e, 0xfa, 0x76, 0xfd, 0xf2, 0x6e,
        0xee, 0x72, 0x71, 0xe8, 0x6e, 0x7d, 0xfe, 0xfe,
        0xee, 0x6b, 0xf2, 0xfa, 0x6c, 0xe9, 0x7e, 0x78,
        0x7f, 0x74, 0xeb, 0x6f, 0x7d, 0xf1, 0x6b, 0xed,
        0xfe, 0x71, 0xf5, 0x75, 0xee, 0x7a, 0x75, 0xef,
        0x6a, 0xef, 0xf0, 0x6d, 0xfb, 0x73, 0xf2, 0xff,
        0x70, 0xe9, 0x6e, 0x7b, 0xec, 0x6f, 0xfb, 0x73,
        0xf4, 0xef, 0x6b, 0xee, 0x77, 0x77, 0xe9, 0x71,
        0xfc, 0x7a, 0x7a, 0xed, 0x6d, 0xf6, 0x7e, 0x6c,
        0xe9, 0x77, 0x76, 0xfb, 0x7c, 0xec, 0x6d, 0xfc,
        0xf2, 0x69, 0xec, 0xf9, 0x75, 0x7e, 0x71, 0xea,
        0x7b, 0x79, 0xec, 0x6d, 0xf2, 0xf5, 0x71, 0xf5,
        0x71, 0xf0, 0xff, 0x6e, 0xee, 0x6c, 0xf7, 0xea,
        0x72, 0xf8, 0x77, 0xf3, 0xf5, 0x6d, 0xec, 0x78,
        0x74, 0xeb, 0x76, 0xfd, 0x77, 0xfb, 0xeb, 0x6b,
        0xf7, 0xfe, 0x6f, 0xec, 0x76, 0x7c, 0x7c, 0x75,
        0xed, 0x6f, 0xff, 0xf8, 0x6a, 0xed, 0x7c, 0x72,
        0xfd, 0x75, 0xec, 0x70},

    {   //0x80, 0x00, 0x01, 0x87, 0x00, 0x20, 0x8c, 0x30,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x75, 0xee, 0x6b, 0xf4,
        0xf3, 0x76, 0xff, 0x6e, 0xed, 0xfd, 0x6e, 0xee,
        0x71, 0xfc, 0xf3, 0x70, 0xf7, 0x71, 0xf5, 0xf3,
        0x6d, 0xef, 0x70, 0x7a, 0xeb, 0x72, 0xfe, 0x74,
        0xfb, 0xee, 0x6a, 0xf0, 0xfc, 0x6e, 0xed, 0x76,
        0xfd, 0x7a, 0x7d, 0xe7, 0x6d, 0x7e, 0xf4, 0x6f,
        0xed, 0x7c, 0x7e, 0xfc, 0x71, 0xec, 0x75, 0x7a,
        0xf0, 0x6c, 0xf0, 0xf8, 0x73, 0xf9, 0x74, 0xed,
        0x79, 0x6e, 0xec, 0x6f, 0xf9, 0xef, 0x77, 0xfb,
        0x6e, 0xf0, 0xf3, 0x6c, 0xee, 0x78, 0x79, 0xf3,
        0x6f, 0xf9, 0x75, 0xfa, 0xed, 0x6c, 0xf3, 0x7b,
        0x72, 0xeb, 0x76, 0xfe, 0x77, 0x7c, 0xec, 0x6c,
        0xf8, 0xf2, 0x6d, 0xf0, 0x7b, 0x7e, 0x7b, 0x73,
        0xe8, 0x71, 0x74, 0xf1, 0x6f, 0xf0, 0xfc, 0x7b,
        0xf8, 0x6d, 0xee, 0x7d, 0x71, 0xee, 0x6d, 0xf8,
        0xf3, 0x75, 0xfa, 0x71, 0xee, 0xf8, 0x6b, 0xee,
        0x77, 0x7c, 0xef, 0x79, 0xf8, 0x6c, 0xf8, 0xec,
        0x6b, 0xf4, 0xfc, 0x77, 0xf0, 0x74, 0xf8, 0x7a,
        0x79, 0xeb, 0x6d, 0xfc, 0xfc, 0x6f, 0xeb, 0x7b,
        0x7e, 0x78, 0x72, 0xeb, 0x6c, 0x77, 0xef, 0x6e,
        0xf5, 0xff, 0x7d, 0x7d},

    {   //0x80, 0x00, 0x01, 0x88, 0x00, 0x20, 0x8c, 0xd0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x6d, 0xe8, 0x7e, 0x6d,
        0xef, 0x72, 0xf8, 0xfa, 0x78, 0xf4, 0x6d, 0xf3,
        0xf8, 0x6d, 0xef, 0x74, 0xff, 0xef, 0x75, 0xf9,
        0x73, 0xf3, 0xee, 0x6a, 0xf0, 0x7e, 0x77, 0xec,
        0x7d, 0xf4, 0x6f, 0x7c, 0xe9, 0x6c, 0xfd, 0xfa,
        0x74, 0xf3, 0x75, 0xf9, 0xfe, 0x76, 0xea, 0x75,
        0x7b, 0xf5, 0x6f, 0xed, 0xfc, 0x7e, 0x7f, 0x6e,
        0xeb, 0x76, 0x70, 0xee, 0x71, 0xf9, 0xf9, 0x7c,
        0xfa, 0x6d, 0xec, 0xf4, 0x6c, 0xf1, 0x79, 0xfc,
        0xf9, 0x75, 0xf3, 0x6d, 0xfc, 0xf1, 0x6d, 0xf3,
        0x77, 0x79, 0xf2, 0x73, 0xfa, 0x71, 0x7d, 0xee,
        0x68, 0xfe, 0xfa, 0x71, 0xf3, 0x79, 0xf8, 0x76,
        0x71, 0xe8, 0x73, 0x78, 0xf5, 0x75, 0xf4, 0x75,
        0xfd, 0xfb, 0x6f, 0xea, 0x7c, 0x71, 0xf4, 0x70,
        0xf3, 0xfb, 0x79, 0xf9, 0x6d, 0xee, 0xfc, 0x6d,
        0xef, 0x78, 0xfd, 0xf8, 0x79, 0xf6, 0x6d, 0xf3,
        0xee, 0x6b, 0xf7, 0x7d, 0xfd, 0xf3, 0x76, 0xef,
        0x72, 0x7a, 0xec, 0x6e, 0xfa, 0x7e, 0x77, 0xf2,
        0x74, 0xf9, 0x7a, 0x7a, 0xea, 0x6f, 0x7d, 0xf2,
        0x74, 0xf4, 0x7a, 0xf8, 0x7d, 0x6c, 0xe9, 0x7b,
        0x73, 0xf5, 0x77, 0xf2},

    {   //0x80, 0x00, 0x01, 0x89, 0x00, 0x20, 0x8d, 0x70,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x7a, 0x7d, 0xf5, 0x6d,
        0xf0, 0xfa, 0x6f, 0xf5, 0x71, 0xf8, 0xf5, 0x79,
        0xf2, 0x6e, 0xf3, 0xf1, 0x6c, 0xf1, 0x7f, 0xfe,
        0xf9, 0x75, 0xf5, 0x6e, 0x7f, 0xea, 0x6e, 0x7f,
        0x7f, 0x7e, 0xf2, 0x73, 0xf0, 0x7c, 0x75, 0xea,
        0x75, 0xfd, 0xfa, 0x77, 0xef, 0x74, 0xfd, 0xfe,
        0x72, 0xea, 0x77, 0x72, 0xf4, 0x75, 0xf7, 0x7b,
        0xfc, 0xf9, 0x6b, 0xed, 0xf9, 0x6e, 0xf7, 0x79,
        0xf5, 0x79, 0x74, 0xf0, 0x6e, 0xf8, 0xf2, 0x6d,
        0xfa, 0x73, 0xfd, 0xf6, 0x74, 0xf4, 0x6f, 0xfe,
        0xed, 0x6c, 0xf8, 0xfc, 0x7a, 0xf3, 0x73, 0xf5,
        0x76, 0x76, 0xeb, 0x74, 0x7c, 0x7a, 0x7c, 0xef,
        0x6e, 0xfd, 0xf7, 0x6e, 0xf1, 0x7a, 0x7d, 0xfa,
        0x72, 0xee, 0x7b, 0x7a, 0xfa, 0x71, 0xed, 0x7c,
        0x73, 0xf2, 0x76, 0xf5, 0xfd, 0x7a, 0xf6, 0x6c,
        0xf4, 0xf2, 0x71, 0xf5, 0x76, 0xf5, 0xf7, 0x71,
        0xf2, 0x74, 0xfd, 0xf0, 0x73, 0xf8, 0x76, 0x7b,
        0xef, 0x73, 0xf9, 0x76, 0x7c, 0xec, 0x70, 0xf7,
        0xfa, 0x79, 0xf3, 0x75, 0xf7, 0x7c, 0x73, 0xeb,
        0x7a, 0x78, 0xfd, 0x77, 0xef, 0x76, 0x7e, 0xf5,
        0x6f, 0xf0, 0x7c, 0x77},

    {   //0x80, 0x00, 0x01, 0x8a, 0x00, 0x20, 0x8e, 0x10,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xf7, 0x71, 0xf2, 0xff,
        0x75, 0xfa, 0x6f, 0xf0, 0xfa, 0x6f, 0xf4, 0x78,
        0xfc, 0xfb, 0x78, 0xf1, 0x6f, 0xfb, 0xee, 0x70,
        0xf8, 0x7c, 0xf9, 0xf6, 0x6f, 0xf3, 0x78, 0x7b,
        0xee, 0x71, 0xfb, 0x77, 0x73, 0xef, 0x77, 0xf9,
        0x7e, 0x78, 0xec, 0x74, 0xff, 0xf3, 0x7a, 0xf1,
        0x75, 0xfe, 0x7f, 0x6c, 0xee, 0xff, 0x73, 0x7f,
        0x73, 0xf1, 0x7b, 0x78, 0xf3, 0x6f, 0xf6, 0xfd,
        0x74, 0xf6, 0x70, 0xf7, 0xfb, 0x72, 0xf7, 0x71,
        0xf4, 0xf2, 0x6e, 0xf6, 0x7b, 0xff, 0xf7, 0x75,
        0xf2, 0x73, 0x7b, 0xed, 0x72, 0xfa, 0x7e, 0xfd,
        0xf2, 0x6e, 0xf9, 0xff, 0x76, 0xef, 0x75, 0x7e,
        0x7e, 0x72, 0xef, 0x78, 0xff, 0x7e, 0x6f, 0xee,
        0x77, 0x77, 0xf5, 0x77, 0xf3, 0x7b, 0x7f, 0xf5,
        0x6c, 0xef, 0xf6, 0x74, 0xfa, 0x73, 0xf6, 0x7e,
        0x72, 0xf1, 0x71, 0xfe, 0xf7, 0x71, 0xf6, 0x73,
        0xfa, 0xf2, 0x74, 0xf5, 0x72, 0xff, 0xef, 0x6f,
        0xf8, 0xfd, 0x7d, 0xf2, 0x78, 0xf0, 0x7c, 0x74,
        0xeb, 0x79, 0x7d, 0xfe, 0x7a, 0xf0, 0x72, 0xfd,
        0xfa, 0x73, 0xef, 0x7b, 0x7f, 0xfb, 0x75, 0xef,
        0xff, 0x7e, 0xf9, 0x71},

    {   //0x80, 0x00, 0x01, 0x8b, 0x00, 0x20, 0x8e, 0xb0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xf1, 0x7f, 0x72, 0xf5,
        0x78, 0xf5, 0x7c, 0x78, 0xf3, 0x6d, 0xf8, 0xf1,
        0x73, 0xfc, 0x74, 0xf8, 0xfb, 0x6e, 0xf3, 0x75,
        0x7a, 0xf4, 0x72, 0xf8, 0x75, 0x7a, 0xf1, 0x72,
        0xf8, 0x77, 0x78, 0xee, 0x72, 0xfe, 0x7f, 0x78,
        0xf2, 0x73, 0xf6, 0xfd, 0x6e, 0xee, 0x7d, 0x7b,
        0xfd, 0x7a, 0xed, 0x76, 0x7a, 0xf5, 0x72, 0xf2,
        0xfe, 0x79, 0xfd, 0x6f, 0xf5, 0xfd, 0x7a, 0xf5,
        0x72, 0xf5, 0xf9, 0x70, 0xf8, 0x78, 0xf7, 0xfd,
        0x70, 0xf2, 0x70, 0x7e, 0xef, 0x78, 0xfb, 0x76,
        0xf7, 0xf2, 0x71, 0xf4, 0x7b, 0x78, 0xf2, 0x74,
        0xfa, 0x79, 0x78, 0xee, 0x73, 0xfb, 0x7e, 0x75,
        0xee, 0x77, 0x7b, 0xfb, 0x79, 0xf2, 0x77, 0xfd,
        0xf8, 0x6e, 0xef, 0xf9, 0x7a, 0xfc, 0x76, 0xf0,
        0x78, 0x74, 0xf0, 0x73, 0xf6, 0xf5, 0x7c, 0xf5,
        0x71, 0xf7, 0xf7, 0x76, 0xf5, 0x73, 0xfd, 0xf5,
        0x70, 0xfa, 0x78, 0xfe, 0xfa, 0x6f, 0xf3, 0x75,
        0x76, 0xef, 0x79, 0xfd, 0x76, 0x7f, 0xef, 0x6f,
        0xf7, 0xf8, 0x78, 0xf2, 0x78, 0xfd, 0xfe, 0x76,
        0xee, 0x77, 0x7d, 0xfe, 0x6f, 0xef, 0x79, 0x76,
        0xfa, 0x74, 0xf3, 0x7b},

    {   //0x80, 0x00, 0x01, 0x8c, 0x00, 0x20, 0x8f, 0x50,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x7b, 0xf3, 0x6f, 0xf7,
        0xf6, 0x78, 0xfd, 0x74, 0xf3, 0xfd, 0x70, 0xf3,
        0x75, 0x7f, 0xf7, 0x77, 0xf6, 0x72, 0xfd, 0xf5,
        0x74, 0xf6, 0x76, 0x7b, 0xf5, 0x6f, 0x7e, 0x7c,
        0xff, 0xf2, 0x70, 0xf6, 0x7e, 0x71, 0xef, 0x7c,
        0xfb, 0x7d, 0x7a, 0xed, 0x73, 0xff, 0xf5, 0x74,
        0xf4, 0x7c, 0xfd, 0xfa, 0x71, 0xef, 0xfe, 0x7b,
        0xf6, 0x73, 0xf1, 0xf9, 0x78, 0xfa, 0x77, 0xf2,
        0x7e, 0x78, 0xef, 0x71, 0xfd, 0xf4, 0x7a, 0xfa,
        0x73, 0xf6, 0xf7, 0x6e, 0xf7, 0x78, 0x7a, 0xf8,
        0x77, 0xf7, 0x76, 0x7e, 0xef, 0x75, 0xf8, 0x7c,
        0x79, 0xef, 0x73, 0xff, 0x7e, 0x7b, 0xf2, 0x71,
        0xfb, 0xfd, 0x6f, 0xf1, 0xff, 0xfe, 0x7e, 0x76,
        0xed, 0x77, 0x7a, 0xf2, 0x76, 0xf7, 0x7c, 0x7b,
        0xfa, 0x70, 0xf3, 0xfb, 0x76, 0xf6, 0x70, 0xfa,
        0xf9, 0x74, 0xfa, 0x74, 0xfa, 0xfe, 0x73, 0xf1,
        0x73, 0x78, 0xf5, 0x79, 0xfb, 0x74, 0xfa, 0xf1,
        0x6f, 0xfa, 0x7c, 0x7a, 0xf4, 0x79, 0xf5, 0x7a,
        0x77, 0xee, 0x78, 0xfb, 0xfa, 0x79, 0xef, 0x78,
        0x7c, 0xfb, 0x79, 0xf0, 0x75, 0x7b, 0xf6, 0x6e,
        0xf6, 0xfc, 0x7d, 0xfd},

    {   //0x80, 0x00, 0x01, 0x8d, 0x00, 0x20, 0x8f, 0xf0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x72, 0xef, 0x7d, 0x73,
        0xf3, 0x78, 0xf9, 0xfd, 0x7c, 0xf4, 0x71, 0xf8,
        0xf6, 0x74, 0xf6, 0x75, 0xfb, 0xf4, 0x73, 0xfa,
        0x77, 0xfc, 0xf9, 0x70, 0xf5, 0x79, 0x75, 0xf5,
        0x79, 0x7f, 0x74, 0x7c, 0xef, 0x6f, 0x7d, 0xfc,
        0x78, 0xf6, 0x79, 0xf9, 0xfe, 0x74, 0xef, 0x7b,
        0xff, 0xfa, 0x74, 0xf1, 0x7d, 0x7b, 0xf9, 0x77,
        0xef, 0x7c, 0x79, 0xf0, 0x72, 0xfa, 0xf9, 0x7d,
        0xfb, 0x6f, 0xf3, 0xf7, 0x72, 0xf3, 0x7b, 0xfd,
        0xfd, 0x78, 0xf4, 0x73, 0x7f, 0xf3, 0x76, 0xf6,
        0x7b, 0x7d, 0xf2, 0x75, 0xfb, 0x79, 0x7b, 0xf4,
        0x6f, 0xf9, 0xff, 0x72, 0xf7, 0x7b, 0xfc, 0x79,
        0x78, 0xee, 0x74, 0x7a, 0xf9, 0x76, 0xf8, 0x78,
        0xff, 0xfa, 0x6f, 0xf2, 0x7f, 0x78, 0xf8, 0x76,
        0xf3, 0xff, 0x78, 0xf9, 0x75, 0xf4, 0xfd, 0x73,
        0xf3, 0x75, 0x7c, 0xf8, 0x7c, 0xf8, 0x71, 0xf7,
        0xf4, 0x6e, 0xf8, 0x7f, 0x7f, 0xf9, 0x76, 0xf5,
        0x77, 0x77, 0xf0, 0x76, 0xfd, 0x7b, 0x79, 0xf3,
        0x79, 0xfb, 0xfe, 0x7a, 0xef, 0x75, 0xfb, 0xf6,
        0x73, 0xf4, 0x7a, 0xfa, 0x78, 0x76, 0xf1, 0xf9,
        0x6d, 0xf1, 0x79, 0x77},

    {   //0x80, 0x00, 0x01, 0x8e, 0x00, 0x20, 0x90, 0x90,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xe7, 0x63, 0xe9, 0xfd,
        0x6a, 0xed, 0xf7, 0x74, 0x70, 0xe7, 0x6c, 0xfe,
        0xee, 0x65, 0xeb, 0xee, 0x60, 0xe5, 0x7d, 0x65,
        0xe1, 0x6f, 0x75, 0xee, 0x71, 0xfb, 0xf3, 0x70,
        0x7b, 0xec, 0x76, 0x6d, 0xe6, 0x6e, 0x6d, 0xe0,
        0x67, 0x7e, 0xeb, 0x64, 0xed, 0xf3, 0x67, 0xee,
        0xfd, 0x6f, 0xf0, 0x7e, 0x74, 0xf7, 0xf9, 0x6d,
        0xf0, 0xfa, 0x64, 0xe4, 0x7e, 0x66, 0xe3, 0x6a,
        0x7c, 0xe7, 0x64, 0xed, 0xfb, 0x69, 0xe8, 0x7f,
        0x6e, 0xf1, 0xf9, 0x73, 0xf2, 0xfb, 0x6b, 0xeb,
        0xfe, 0x69, 0xe7, 0x70, 0x6f, 0xe5, 0x69, 0xf8,
        0xee, 0x62, 0xe7, 0xf9, 0x64, 0xe8, 0x7c, 0x6f,
        0xef, 0x78, 0x74, 0xf8, 0xfa, 0x75, 0xed, 0x75,
        0x6d, 0xe4, 0x6b, 0x7a, 0xe5, 0x65, 0xeb, 0xee,
        0x63, 0xe7, 0x7d, 0x6b, 0xe8, 0x78, 0x74, 0xf1,
        0x79, 0xfb, 0xf0, 0x70, 0x7b, 0xea, 0x6f, 0x76,
        0xe6, 0x69, 0x7c, 0xe6, 0x63, 0xef, 0xf7, 0x65,
        0xe2, 0x6f, 0x6e, 0xe8, 0x6c, 0xfb, 0xf0, 0x6d,
        0x7d, 0xf4, 0x72, 0x7c, 0xea, 0x6a, 0x77, 0xe6,
        0x68, 0xf9, 0xf2, 0x67, 0xe9, 0x79, 0x6a, 0xe5,
        0x6b, 0x72, 0xe5, 0x68},

    {   //0x80, 0x00, 0x01, 0x8f, 0x00, 0x20, 0x91, 0x30,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x7b, 0xf0, 0x6c, 0xf8,
        0xf8, 0x70, 0x7f, 0xef, 0x70, 0x7d, 0xed, 0x67,
        0xee, 0xf3, 0x66, 0xe2, 0x6d, 0x6f, 0xdf, 0x67,
        0x78, 0xeb, 0x6c, 0xf8, 0xf6, 0x6e, 0xf8, 0xfd,
        0x71, 0xf1, 0xf6, 0x6c, 0xee, 0xf0, 0x68, 0xec,
        0xfe, 0x6a, 0xe2, 0x70, 0x6e, 0xe3, 0x67, 0xf8,
        0xea, 0x69, 0xec, 0x7c, 0x6f, 0xec, 0x76, 0x6e,
        0xf0, 0xf4, 0x6b, 0xee, 0xf4, 0x65, 0xea, 0x79,
        0x6d, 0xe7, 0x69, 0xfa, 0xe8, 0x64, 0xf2, 0xf5,
        0x69, 0xe9, 0xff, 0x6b, 0xec, 0x77, 0x6f, 0xee,
        0x7f, 0x6e, 0xf2, 0xf9, 0x6d, 0xec, 0x70, 0x74,
        0xe4, 0x66, 0xf4, 0xe9, 0x5f, 0xe9, 0xf7, 0x68,
        0xea, 0x77, 0x7a, 0xee, 0x75, 0x77, 0xef, 0x7c,
        0x73, 0xec, 0x74, 0x72, 0xea, 0x6a, 0x7f, 0xe9,
        0x63, 0xef, 0xeb, 0x62, 0xec, 0xfc, 0x6d, 0xe6,
        0x6d, 0x7c, 0xec, 0x6c, 0xfa, 0xf0, 0x6f, 0x75,
        0xea, 0x75, 0x71, 0xea, 0x69, 0xfc, 0xe9, 0x64,
        0xeb, 0xf6, 0x67, 0xe5, 0x71, 0x71, 0xe8, 0x6c,
        0xf8, 0xe9, 0x69, 0x7e, 0xed, 0x6f, 0x7e, 0xef,
        0x73, 0x7a, 0xef, 0x6c, 0xf6, 0xf0, 0x66, 0xe5,
        0xfe, 0x66, 0xe4, 0x6b},

    {   //0x80, 0x00, 0x01, 0x90, 0x00, 0x20, 0x91, 0xd0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x79, 0xe5, 0x68, 0xf9,
        0xf0, 0x6b, 0xf9, 0xf3, 0x6f, 0x7b, 0xf3, 0x70,
        0xf9, 0xf9, 0x69, 0xe9, 0xf9, 0x67, 0xe3, 0x78,
        0x6a, 0xe4, 0x6c, 0x79, 0xec, 0x67, 0xef, 0xf5,
        0x68, 0xef, 0xff, 0x72, 0xf3, 0xfc, 0x6e, 0xf1,
        0xf5, 0x69, 0xe9, 0x7e, 0x6a, 0xe1, 0x75, 0x70,
        0xea, 0x6a, 0xf5, 0xee, 0x67, 0xea, 0xfb, 0x68,
        0xea, 0x7c, 0x6e, 0xf0, 0xfb, 0x74, 0xf5, 0xff,
        0x6b, 0xeb, 0x7a, 0x6c, 0xe7, 0x69, 0x7d, 0xe9,
        0x64, 0xee, 0xf7, 0x67, 0xe7, 0x7e, 0x6a, 0xed,
        0x7f, 0x73, 0xf1, 0x7d, 0x75, 0xf1, 0x77, 0x76,
        0xea, 0x6c, 0x79, 0xe3, 0x68, 0xff, 0xeb, 0x63,
        0xeb, 0xf8, 0x66, 0xe9, 0x76, 0x73, 0xec, 0x73,
        0x78, 0xf0, 0x76, 0xfe, 0xee, 0x6c, 0x78, 0xe9,
        0x6b, 0x7a, 0xe9, 0x67, 0xf9, 0xee, 0x66, 0xeb,
        0x7a, 0x6b, 0xe4, 0x6c, 0x71, 0xeb, 0x6c, 0xfc,
        0xef, 0x70, 0x7a, 0xf0, 0x70, 0x7a, 0xeb, 0x6a,
        0xf8, 0xea, 0x68, 0xee, 0xff, 0x6b, 0xe4, 0x6f,
        0x6f, 0xe4, 0x6b, 0x79, 0xe9, 0x6b, 0x7c, 0xf5,
        0x70, 0xf8, 0xf8, 0x6f, 0xf9, 0xf0, 0x6c, 0xf2,
        0xf2, 0x67, 0xe8, 0x7b},

    {   //0x80, 0x00, 0x01, 0x91, 0x00, 0x20, 0x92, 0x70,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x68, 0xe4, 0x6b, 0x75,
        0xe5, 0x6a, 0x7c, 0xef, 0x6e, 0xf1, 0xf8, 0x6f,
        0xf0, 0xfd, 0x6f, 0xf1, 0xfa, 0x69, 0xeb, 0xf2,
        0x68, 0xe9, 0x75, 0x6e, 0xe2, 0x6b, 0x7f, 0xea,
        0x67, 0xee, 0xf6, 0x6c, 0xed, 0x7f, 0x6f, 0xef,
        0x7d, 0x6a, 0xef, 0xf8, 0x6b, 0xeb, 0xfe, 0x6e,
        0xea, 0x6f, 0x78, 0xea, 0x65, 0xf2, 0xea, 0x64,
        0xed, 0xf5, 0x6a, 0xeb, 0x7c, 0x6f, 0xef, 0x7c,
        0x72, 0xf2, 0x7c, 0x6d, 0xec, 0x78, 0x75, 0xeb,
        0x6a, 0xf7, 0xe9, 0x64, 0xf0, 0xee, 0x65, 0xec,
        0xfc, 0x6b, 0xed, 0x74, 0x7c, 0xed, 0x70, 0x77,
        0xef, 0x77, 0x74, 0xed, 0x6f, 0x76, 0xe9, 0x67,
        0xfb, 0xed, 0x64, 0xea, 0xf7, 0x68, 0xeb, 0x75,
        0x78, 0xeb, 0x6c, 0x7b, 0xec, 0x6e, 0x7c, 0xec,
        0x6f, 0x79, 0xea, 0x71, 0x7d, 0xef, 0x6a, 0xf1,
        0xf3, 0x66, 0xe8, 0x7d, 0x6d, 0xe5, 0x6d, 0x79,
        0xe9, 0x6d, 0xf9, 0xeb, 0x6e, 0x7c, 0xed, 0x72,
        0xff, 0xef, 0x6e, 0xf6, 0xf3, 0x6b, 0xeb, 0xff,
        0x68, 0xe3, 0x77, 0x6c, 0xe6, 0x6b, 0x7f, 0xea,
        0x6a, 0xf6, 0xf5, 0x6d, 0xf8, 0xf8, 0x6f, 0xf9,
        0xf3, 0x6d, 0xf2, 0xfe},

    {   //0x80, 0x00, 0x01, 0x92, 0x00, 0x20, 0x93, 0x10,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x68, 0xe7, 0x7c, 0x69,
        0xe6, 0x6f, 0x72, 0xea, 0x6a, 0xfb, 0xf3, 0x69,
        0xed, 0xfa, 0x69, 0xef, 0xf9, 0x6e, 0xf2, 0xf9,
        0x6d, 0xee, 0xfb, 0x6a, 0xe9, 0x75, 0x6f, 0xe6,
        0x6c, 0xfe, 0xee, 0x68, 0xee, 0xf4, 0x69, 0xeb,
        0xfa, 0x6b, 0xee, 0xfe, 0x6d, 0xf2, 0xfa, 0x75,
        0xf0, 0x77, 0x72, 0xe8, 0x6e, 0x76, 0xe8, 0x67,
        0xf5, 0xed, 0x65, 0xed, 0xfe, 0x6b, 0xea, 0x7a,
        0x6d, 0xf1, 0x7c, 0x78, 0xf1, 0x77, 0x77, 0xed,
        0x6f, 0x79, 0xe9, 0x68, 0x7f, 0xe7, 0x68, 0xf5,
        0xf0, 0x69, 0xeb, 0x7b, 0x6e, 0xe9, 0x72, 0x7b,
        0xec, 0x70, 0x79, 0xee, 0x76, 0x7d, 0xec, 0x6b,
        0x7c, 0xe8, 0x6a, 0xf9, 0xed, 0x68, 0xef, 0xff,
        0x68, 0xe9, 0x71, 0x73, 0xe5, 0x6c, 0x76, 0xec,
        0x6f, 0xfe, 0xf0, 0x73, 0xff, 0xf0, 0x6e, 0xff,
        0xee, 0x69, 0xef, 0xf2, 0x67, 0xeb, 0x76, 0x6d,
        0xe5, 0x6c, 0x75, 0xe8, 0x6d, 0x7e, 0xed, 0x6f,
        0xfd, 0xf4, 0x73, 0xf1, 0xf6, 0x6c, 0xf1, 0xf4,
        0x69, 0xec, 0xfa, 0x6a, 0xe6, 0x75, 0x6f, 0xe8,
        0x6b, 0xfc, 0xeb, 0x6c, 0xf9, 0xf8, 0x6e, 0xf3,
        0xf9, 0x6d, 0xef, 0xf9},

    {   //0x80, 0x00, 0x01, 0x93, 0x00, 0x20, 0x93, 0xb0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x6b, 0xed, 0xf9, 0x6a,
        0xe9, 0xfa, 0x6d, 0xec, 0x6d, 0x79, 0xea, 0x68,
        0xf6, 0xee, 0x68, 0xee, 0xf8, 0x6d, 0xef, 0xfd,
        0x71, 0xf0, 0xff, 0x6b, 0xef, 0xfe, 0x6d, 0xeb,
        0x75, 0x77, 0xe9, 0x6b, 0xf9, 0xec, 0x66, 0xed,
        0xee, 0x67, 0xee, 0xfc, 0x6d, 0xee, 0x7b, 0x73,
        0xef, 0x7c, 0x74, 0xef, 0x75, 0x71, 0xea, 0x6e,
        0x7b, 0xeb, 0x65, 0xf4, 0xee, 0x66, 0xee, 0xf7,
        0x6b, 0xef, 0x78, 0x6f, 0xef, 0x71, 0x7c, 0xed,
        0x6e, 0x75, 0xec, 0x72, 0x76, 0xea, 0x6e, 0x7e,
        0xeb, 0x67, 0xf5, 0xf8, 0x68, 0xe8, 0x7c, 0x6d,
        0xeb, 0x6f, 0x7b, 0xea, 0x6f, 0x7d, 0xeb, 0x6f,
        0x7a, 0xec, 0x6e, 0x7d, 0xed, 0x6e, 0xf7, 0xf7,
        0x6a, 0xea, 0xfc, 0x6a, 0xe6, 0x75, 0x70, 0xe7,
        0x6e, 0x7d, 0xed, 0x6f, 0xfa, 0xef, 0x6f, 0x7d,
        0xef, 0x6f, 0xfa, 0xf3, 0x6b, 0xed, 0xfc, 0x6a,
        0xe8, 0x78, 0x6c, 0xe4, 0x73, 0x74, 0xea, 0x6d,
        0xfb, 0xf2, 0x6c, 0xf5, 0xfa, 0x6c, 0xf7, 0xf9,
        0x6c, 0xf3, 0xf5, 0x6a, 0xed, 0x7c, 0x69, 0xe7,
        0x72, 0x70, 0xe9, 0x6b, 0xfe, 0xef, 0x6b, 0xf3,
        0xf7, 0x6d, 0xec, 0xf7},

    {   //0x80, 0x00, 0x01, 0x94, 0x00, 0x20, 0x94, 0x50,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x6a, 0xef, 0xf6, 0x6d,
        0xef, 0xf9, 0x6f, 0xec, 0x7b, 0x6e, 0xea, 0x6c,
        0x7e, 0xe7, 0x6a, 0xf4, 0xf3, 0x68, 0xee, 0xf8,
        0x6b, 0xee, 0xfa, 0x6d, 0xf1, 0x7e, 0x6f, 0xef,
        0x7d, 0x78, 0xed, 0x6d, 0x77, 0xe8, 0x6b, 0xfc,
        0xe9, 0x68, 0xf2, 0xf2, 0x6a, 0xed, 0x7d, 0x70,
        0xeb, 0x77, 0x72, 0xf0, 0x77, 0x78, 0xef, 0x6f,
        0x77, 0xea, 0x6b, 0x7e, 0xeb, 0x66, 0xf9, 0xee,
        0x6a, 0xf3, 0x7e, 0x6e, 0xeb, 0x71, 0x71, 0xea,
        0x6f, 0x76, 0xef, 0x6f, 0x76, 0xee, 0x73, 0x7b,
        0xed, 0x69, 0xfb, 0xeb, 0x69, 0xef, 0xf8, 0x6c,
        0xeb, 0x74, 0x6d, 0xea, 0x6e, 0x78, 0xe8, 0x6f,
        0x78, 0xee, 0x72, 0xff, 0xf2, 0x71, 0xf6, 0xf0,
        0x6d, 0xf3, 0xf5, 0x68, 0xeb, 0xfc, 0x6a, 0xe9,
        0x6f, 0x71, 0xe8, 0x6d, 0x7d, 0xed, 0x6f, 0xff,
        0xf4, 0x70, 0xfb, 0xf7, 0x6e, 0xef, 0xf9, 0x6a,
        0xeb, 0xf5, 0x6a, 0xea, 0xfd, 0x6e, 0xe7, 0x71,
        0x7b, 0xeb, 0x6b, 0xf3, 0xef, 0x6d, 0xf5, 0xf9,
        0x6e, 0xf4, 0xfa, 0x6d, 0xee, 0xf8, 0x6a, 0xee,
        0x7b, 0x6b, 0xeb, 0x75, 0x77, 0xed, 0x6a, 0xfb,
        0xed, 0x68, 0xf0, 0xef},

    {   //0x80, 0x00, 0x01, 0x95, 0x00, 0x20, 0x94, 0xf0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x6a, 0xef, 0xfb, 0x6e,
        0xef, 0xfb, 0x75, 0xef, 0x7d, 0x6e, 0xeb, 0x7a,
        0x72, 0xea, 0x6d, 0xfd, 0xeb, 0x67, 0xf4, 0xf1,
        0x67, 0xee, 0xf6, 0x6c, 0xf3, 0x7e, 0x72, 0xf0,
        0x77, 0x76, 0xee, 0x74, 0x76, 0xec, 0x6f, 0x79,
        0xe7, 0x6c, 0xfc, 0xed, 0x67, 0xef, 0xf7, 0x69,
        0xee, 0x7e, 0x70, 0xed, 0x74, 0x75, 0xee, 0x72,
        0x7a, 0xec, 0x6e, 0x76, 0xeb, 0x6e, 0x7b, 0xed,
        0x6e, 0xf5, 0xf0, 0x6a, 0xec, 0xff, 0x6c, 0xe7,
        0x77, 0x74, 0xeb, 0x70, 0x7b, 0xee, 0x6f, 0x79,
        0xed, 0x6f, 0x78, 0xee, 0x6c, 0xfb, 0xf2, 0x6c,
        0xef, 0x7d, 0x6c, 0xe8, 0x77, 0x6c, 0xe9, 0x71,
        0x76, 0xec, 0x6e, 0x7f, 0xf1, 0x72, 0xf8, 0xf2,
        0x6e, 0xf9, 0xf0, 0x6d, 0xf1, 0xf7, 0x6b, 0xe9,
        0x7c, 0x6c, 0xe8, 0x72, 0x74, 0xe9, 0x6f, 0x7b,
        0xf0, 0x6d, 0xf6, 0xf6, 0x6c, 0xef, 0xf5, 0x6c,
        0xf2, 0xf8, 0x6b, 0xf0, 0xf8, 0x6c, 0xed, 0x76,
        0x6f, 0xe9, 0x6d, 0x7b, 0xec, 0x6c, 0xf3, 0xf5,
        0x6a, 0xf2, 0xfa, 0x6c, 0xf1, 0xf8, 0x6a, 0xf5,
        0xf9, 0x6e, 0xf1, 0x7b, 0x73, 0xec, 0x70, 0x78,
        0xea, 0x69, 0xfb, 0xeb},

    {   //0x80, 0x00, 0x01, 0x96, 0x00, 0x20, 0x95, 0x90,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x69, 0xef, 0xf7, 0x6c,
        0xed, 0xf9, 0x70, 0xee, 0xfa, 0x71, 0xf0, 0x7b,
        0x73, 0xed, 0x74, 0x79, 0xeb, 0x6a, 0xff, 0xeb,
        0x68, 0xf6, 0xed, 0x6c, 0xef, 0xfb, 0x6f, 0xed,
        0x78, 0x77, 0xeb, 0x76, 0x77, 0xec, 0x77, 0x76,
        0xed, 0x6f, 0x7b, 0xea, 0x69, 0xfd, 0xef, 0x69,
        0xee, 0xf8, 0x6d, 0xee, 0x73, 0x71, 0xec, 0x6e,
        0x73, 0xec, 0x70, 0x79, 0xee, 0x70, 0x79, 0xee,
        0x75, 0xfb, 0xee, 0x6b, 0xf2, 0xf4, 0x69, 0xec,
        0x7c, 0x6e, 0xeb, 0x71, 0x72, 0xec, 0x71, 0x7b,
        0xec, 0x71, 0x77, 0xf0, 0x6f, 0xfe, 0xf4, 0x6e,
        0xf0, 0xf5, 0x6a, 0xef, 0xfc, 0x6a, 0xe9, 0x7d,
        0x6f, 0xea, 0x6f, 0x7c, 0xeb, 0x6e, 0xfa, 0xf0,
        0x6f, 0xf9, 0xf7, 0x6f, 0xf5, 0xf7, 0x6c, 0xee,
        0xfb, 0x68, 0xeb, 0x7e, 0x6b, 0xed, 0x71, 0x75,
        0xee, 0x6a, 0xff, 0xf0, 0x6b, 0xef, 0xf1, 0x6e,
        0xf4, 0xf7, 0x6e, 0xf3, 0xf6, 0x6f, 0xec, 0xfa,
        0x6f, 0xeb, 0x77, 0x76, 0xea, 0x6f, 0xfb, 0xee,
        0x6a, 0xf3, 0xef, 0x6a, 0xf0, 0xf3, 0x6b, 0xf6,
        0x7e, 0x6f, 0xf6, 0x7c, 0x71, 0xed, 0x70, 0x75,
        0xed, 0x76, 0x7c, 0xf8},

    {   //0x80, 0x00, 0x01, 0x97, 0x00, 0x20, 0x96, 0x30,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xfd, 0x7b, 0xfa, 0xfe,
        0xfc, 0x75, 0x7f, 0xed, 0x6e, 0x7c, 0xf0, 0x6a,
        0xfd, 0xec, 0x66, 0xfb, 0xe5, 0x64, 0xfe, 0xe2,
        0x62, 0xfc, 0xe2, 0x64, 0xf8, 0xe9, 0x62, 0xf0,
        0xea, 0x66, 0xf1, 0xef, 0x6c, 0xfa, 0xf3, 0x70,
        0xfe, 0xf5, 0x75, 0xf7, 0xfd, 0x7a, 0xfd, 0x7d,
        0xf6, 0x75, 0x7a, 0xec, 0x6b, 0x72, 0xe7, 0x67,
        0x76, 0xe1, 0x67, 0x7a, 0xe6, 0x66, 0xfe, 0xe7,
        0x66, 0x7f, 0xe7, 0x65, 0xfb, 0xe7, 0x64, 0xef,
        0xe9, 0x64, 0xeb, 0xf0, 0x65, 0xea, 0xf6, 0x68,
        0xee, 0xf3, 0x6d, 0xf5, 0xf7, 0x77, 0xf8, 0xf6,
        0x7d, 0xfd, 0xfe, 0x7c, 0xfc, 0x78, 0xff, 0x75,
        0xfc, 0xf0, 0x6b, 0xf9, 0xf3, 0x6b, 0xec, 0xf6,
        0x67, 0xeb, 0xfd, 0x67, 0xe8, 0x7d, 0x64, 0xe4,
        0xf5, 0x61, 0xe7, 0x7e, 0x64, 0xe3, 0x7b, 0x66,
        0xe3, 0x70, 0x6c, 0xe1, 0x6b, 0x6f, 0xe5, 0x6e,
        0x76, 0xec, 0x6d, 0x78, 0xeb, 0x74, 0x73, 0xec,
        0x74, 0x78, 0xed, 0x78, 0x7a, 0xff, 0xf7, 0xfe,
        0x75, 0xf7, 0x7e, 0x74, 0xec, 0x79, 0x71, 0xea,
        0x6f, 0x7d, 0xea, 0x69, 0x78, 0xe5, 0x6d, 0x7a,
        0xe7, 0x67, 0x7d, 0xe2},

    {   //0x80, 0x00, 0x01, 0x98, 0x00, 0x20, 0x96, 0xd0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x65, 0xfe, 0xe3, 0x61,
        0xf4, 0xe5, 0x60, 0xf9, 0xea, 0x66, 0xf4, 0xf6,
        0x66, 0xed, 0xef, 0x6d, 0xee, 0xf5, 0x6a, 0xef,
        0xf5, 0x6a, 0xf5, 0xfb, 0x72, 0xf5, 0x7b, 0x77,
        0xff, 0xfd, 0xf7, 0x70, 0xfb, 0xf8, 0x6c, 0xf1,
        0xf7, 0x68, 0xf4, 0xf0, 0x6c, 0xef, 0xf9, 0x6a,
        0xeb, 0xf6, 0x66, 0xe9, 0xfe, 0x62, 0xe3, 0x79,
        0x61, 0xe2, 0x75, 0x68, 0xe2, 0x6f, 0x6a, 0xe5,
        0x75, 0x6c, 0xe8, 0x73, 0x6c, 0xe8, 0x72, 0x71,
        0xeb, 0x73, 0xfc, 0xec, 0x6e, 0x7e, 0xf0, 0x76,
        0xfd, 0xf7, 0x7f, 0x7c, 0xfe, 0xf3, 0x75, 0x76,
        0xec, 0x77, 0x7c, 0xed, 0x6f, 0x77, 0xea, 0x72,
        0x6f, 0xe6, 0x6d, 0x73, 0xe2, 0x69, 0x78, 0xe5,
        0x67, 0xf7, 0xe8, 0x61, 0xf5, 0xe8, 0x62, 0xf5,
        0xea, 0x64, 0xf3, 0xe8, 0x68, 0xf8, 0xec, 0x68,
        0xee, 0xef, 0x66, 0xf0, 0xfc, 0x6f, 0xf0, 0xff,
        0x79, 0xf6, 0xfc, 0x7e, 0x7d, 0x7e, 0x7b, 0x7e,
        0xf6, 0x78, 0x6f, 0xf7, 0xf8, 0x6e, 0xf0, 0xf5,
        0x69, 0xef, 0xef, 0x65, 0xef, 0xf5, 0x64, 0xe9,
        0x7f, 0x62, 0xe6, 0xfd, 0x69, 0xe5, 0x77, 0x69,
        0xe6, 0x78, 0x68, 0xe8},

    {   //0x80, 0x00, 0x01, 0x99, 0x00, 0x20, 0x97, 0x70,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x6e, 0x68, 0xe1, 0x6f,
        0x6a, 0xe4, 0x74, 0x74, 0xe7, 0x71, 0x79, 0xed,
        0x75, 0x7e, 0xfc, 0x78, 0x7f, 0xf5, 0xfd, 0x76,
        0xfe, 0xfb, 0x7d, 0xfd, 0xfc, 0x78, 0x76, 0xf2,
        0x77, 0x70, 0xec, 0x6e, 0x7a, 0xe3, 0x6a, 0x73,
        0xe4, 0x6a, 0x7b, 0xe4, 0x67, 0x79, 0xe8, 0x69,
        0xfe, 0xea, 0x66, 0xf5, 0xe5, 0x66, 0xf7, 0xea,
        0x65, 0xef, 0xed, 0x63, 0xee, 0xf5, 0x69, 0xf1,
        0xf9, 0x6b, 0xf5, 0xef, 0x6f, 0x7e, 0xfc, 0x79,
        0xfc, 0xfa, 0x7c, 0x7e, 0xfd, 0xfc, 0x7e, 0xfe,
        0xfb, 0x74, 0xf4, 0xf3, 0x69, 0xf3, 0xf6, 0x66,
        0xea, 0xfd, 0x67, 0xe7, 0xf7, 0x6a, 0xe8, 0xfc,
        0x64, 0xe6, 0xf9, 0x63, 0xea, 0x7b, 0x67, 0xe5,
        0x75, 0x6c, 0xe5, 0x6f, 0x72, 0xe5, 0x6d, 0x70,
        0xe7, 0x72, 0x73, 0xf0, 0x72, 0x75, 0xec, 0x79,
        0x74, 0xef, 0x78, 0x7d, 0xf8, 0x7d, 0x7a, 0x79,
        0xf2, 0x7e, 0x70, 0xef, 0x7c, 0x79, 0xec, 0x70,
        0x76, 0xeb, 0x6e, 0x7c, 0xea, 0x67, 0x79, 0xe6,
        0x6c, 0x77, 0xe8, 0x66, 0x7b, 0xe2, 0x64, 0xfc,
        0xe6, 0x63, 0xf5, 0xeb, 0x64, 0xf3, 0xec, 0x6c,
        0xf4, 0xf5, 0x6a, 0xf0},

    {   //0x80, 0x00, 0x01, 0x9a, 0x00, 0x20, 0x98, 0x10,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xf0, 0x6c, 0xf1, 0xf8,
        0x6c, 0xf2, 0xf5, 0x6f, 0xf7, 0xf6, 0xfe, 0xfc,
        0xfe, 0xfd, 0x75, 0xf8, 0xf6, 0x6b, 0xf7, 0xf7,
        0x6b, 0xef, 0xfb, 0x6a, 0xf1, 0xf2, 0x6c, 0xee,
        0xfb, 0x66, 0xe9, 0xfc, 0x65, 0xe7, 0x7e, 0x64,
        0xe2, 0x7b, 0x64, 0xe4, 0x7a, 0x6c, 0xe7, 0x6f,
        0x6c, 0xe8, 0x7a, 0x6e, 0xeb, 0x77, 0x70, 0xe9,
        0x72, 0x76, 0xee, 0x73, 0xfb, 0xef, 0x74, 0x7b,
        0xf7, 0x7f, 0x7b, 0xfd, 0xf3, 0x76, 0x7d, 0xed,
        0x72, 0x74, 0xed, 0x77, 0x78, 0xec, 0x6f, 0x74,
        0xe8, 0x73, 0x77, 0xe8, 0x6d, 0x77, 0xe6, 0x6c,
        0x71, 0xe7, 0x6a, 0x7d, 0xe5, 0x64, 0x7c, 0xe4,
        0x66, 0xfe, 0xe7, 0x65, 0xfc, 0xe9, 0x6a, 0xfe,
        0xed, 0x6c, 0xf4, 0xee, 0x6a, 0xf4, 0xf3, 0x71,
        0xf7, 0xf8, 0x71, 0x7f, 0xf4, 0x74, 0x79, 0xf5,
        0x75, 0x7e, 0xf2, 0x78, 0x73, 0xf7, 0xf9, 0x6e,
        0xf4, 0xf7, 0x6b, 0xf1, 0xf3, 0x6b, 0xf0, 0xf7,
        0x6c, 0xeb, 0x7f, 0x68, 0xec, 0xfc, 0x6b, 0xec,
        0x7d, 0x68, 0xeb, 0xfa, 0x69, 0xe9, 0x7e, 0x69,
        0xe4, 0x7b, 0x69, 0xea, 0x7c, 0x6e, 0xed, 0x7b,
        0x71, 0xef, 0x7e, 0x78},

    {   //0x80, 0x00, 0x01, 0x9b, 0x00, 0x20, 0x98, 0xb0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xf5, 0x7a, 0x76, 0xee,
        0xfd, 0x6f, 0xf8, 0xff, 0x75, 0xf7, 0xf9, 0x77,
        0x7c, 0xef, 0x75, 0x75, 0xed, 0x6d, 0xfe, 0xe7,
        0x6d, 0x75, 0xec, 0x6f, 0x7a, 0xe9, 0x6f, 0x79,
        0xe9, 0x6e, 0x7b, 0xeb, 0x69, 0xfd, 0xe7, 0x67,
        0x7c, 0xeb, 0x67, 0xf7, 0xeb, 0x6a, 0xf8, 0xee,
        0x6e, 0xfd, 0xf1, 0x6c, 0x7c, 0xed, 0x6f, 0x7a,
        0xf4, 0x75, 0xfa, 0xf5, 0x76, 0x7c, 0xf9, 0x7e,
        0x79, 0xf8, 0x7a, 0x71, 0xee, 0xf6, 0x6d, 0xee,
        0xfa, 0x6c, 0xeb, 0xfc, 0x6b, 0xee, 0xf7, 0x6b,
        0xee, 0xf8, 0x68, 0xe9, 0xef, 0x69, 0xeb, 0x7f,
        0x6b, 0xe9, 0x79, 0x6b, 0xec, 0x77, 0x6c, 0xeb,
        0x75, 0x6c, 0xea, 0xfc, 0x6f, 0xee, 0x7e, 0x6e,
        0xed, 0xfd, 0x70, 0xf4, 0x7a, 0x76, 0xf5, 0xfd,
        0x76, 0x7e, 0xef, 0x7b, 0x7d, 0xf2, 0x73, 0xfd,
        0xf0, 0x6f, 0x79, 0xf0, 0x6e, 0xfe, 0xec, 0x6b,
        0x77, 0xe8, 0x6f, 0x73, 0xea, 0x6b, 0x78, 0xe8,
        0x69, 0x7b, 0xeb, 0x69, 0xfc, 0xed, 0x6b, 0x7d,
        0xec, 0x6f, 0x7f, 0xee, 0x6a, 0xf9, 0xeb, 0x6c,
        0xfa, 0xef, 0x6c, 0xf9, 0xef, 0x6f, 0x7a, 0xf6,
        0x7d, 0x7b, 0xf9, 0x7c},

    {   //0x80, 0x00, 0x01, 0x9c, 0x00, 0x20, 0x99, 0x50,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x74, 0xf1, 0xfc, 0x72,
        0xef, 0xfe, 0x73, 0xef, 0xfe, 0x6d, 0xf7, 0xf8,
        0x6f, 0xf0, 0xfb, 0x6a, 0xe9, 0xf5, 0x6a, 0xe9,
        0xf9, 0x6a, 0xe7, 0xf8, 0x68, 0xeb, 0xf8, 0x6e,
        0xec, 0x7c, 0x6c, 0xec, 0xfc, 0x6d, 0xee, 0x79,
        0x6c, 0xed, 0x75, 0x6e, 0xf1, 0x78, 0x79, 0xf1,
        0x7c, 0x71, 0xf8, 0xf5, 0x72, 0xfb, 0xf6, 0x6e,
        0xfe, 0xf0, 0x73, 0x7b, 0xf1, 0x79, 0x7f, 0xf0,
        0x6f, 0x77, 0xec, 0x70, 0x77, 0xeb, 0x6b, 0x7a,
        0xe8, 0x6a, 0x7b, 0xe9, 0x6d, 0xfc, 0xeb, 0x6a,
        0x79, 0xe9, 0x6d, 0x7b, 0xea, 0x6a, 0x7d, 0xea,
        0x6e, 0x7d, 0xf2, 0x6e, 0xf7, 0xf3, 0x6e, 0x7f,
        0xf6, 0x74, 0xfe, 0xf2, 0x76, 0x78, 0xef, 0x7b,
        0x76, 0xf2, 0x76, 0x79, 0xf1, 0x7c, 0x6d, 0xf8,
        0xf5, 0x6e, 0xf0, 0xf4, 0x6c, 0xed, 0xf6, 0x6b,
        0xf0, 0xfe, 0x6e, 0xec, 0x7d, 0x69, 0xed, 0xfd,
        0x6d, 0xeb, 0xfc, 0x69, 0xec, 0xf8, 0x68, 0xec,
        0x7a, 0x6b, 0xe9, 0x7d, 0x6c, 0xf0, 0xfe, 0x74,
        0xf0, 0xfe, 0x72, 0xf4, 0xf9, 0x75, 0xf6, 0x7e,
        0x73, 0xf0, 0xf5, 0x70, 0xfb, 0xf5, 0x77, 0xf9,
        0xf0, 0x73, 0x78, 0xed},

    {   //0x80, 0x00, 0x01, 0x9d, 0x00, 0x20, 0x99, 0xf0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x71, 0x77, 0xeb, 0x6e,
        0xfc, 0xe7, 0x6e, 0x7a, 0xec, 0x71, 0x7b, 0xec,
        0x6e, 0x76, 0xec, 0x6c, 0x7f, 0xea, 0x6a, 0xfa,
        0xe8, 0x6a, 0x7e, 0xed, 0x6a, 0xfe, 0xee, 0x6c,
        0x7a, 0xf1, 0x72, 0x7d, 0xf2, 0x71, 0x7a, 0xee,
        0x77, 0x79, 0xf8, 0x76, 0x7e, 0xf7, 0x7b, 0x76,
        0xf4, 0xfc, 0x76, 0xef, 0xfb, 0x6f, 0xec, 0xf7,
        0x6a, 0xef, 0xfe, 0x6c, 0xed, 0xfa, 0x6c, 0xef,
        0xf7, 0x6d, 0xed, 0xfa, 0x68, 0xec, 0xf6, 0x68,
        0xee, 0x75, 0x6a, 0xeb, 0x7a, 0x6e, 0xee, 0x7b,
        0x6e, 0xed, 0x7d, 0x6a, 0xf0, 0xf7, 0x6f, 0xf7,
        0x7e, 0x6f, 0xf1, 0xf8, 0x75, 0xf7, 0xfb, 0x78,
        0xff, 0xfa, 0x70, 0x7b, 0xee, 0x70, 0x7b, 0xf3,
        0x6e, 0xfb, 0xee, 0x70, 0x7e, 0xee, 0x6f, 0x7c,
        0xeb, 0x6b, 0x75, 0xe7, 0x70, 0x79, 0xeb, 0x6c,
        0x7d, 0xe9, 0x6d, 0xfe, 0xed, 0x6c, 0xfa, 0xef,
        0x6b, 0x7a, 0xee, 0x73, 0xfd, 0xed, 0x6d, 0x7e,
        0xed, 0x6e, 0xfe, 0xef, 0x73, 0x7d, 0xf0, 0x7d,
        0x75, 0xf1, 0xf7, 0x78, 0xf3, 0xfd, 0x74, 0xee,
        0xfb, 0x70, 0xf3, 0x7c, 0x6f, 0xf0, 0xfe, 0x6c,
        0xf3, 0xf8, 0x6d, 0xee},

    {   //0x80, 0x00, 0x01, 0x9e, 0x00, 0x20, 0x9a, 0x90,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xfc, 0x68, 0xeb, 0xfb,
        0x69, 0xed, 0x7d, 0x6a, 0xec, 0xfa, 0x6c, 0xee,
        0xfb, 0x71, 0xee, 0x7d, 0x6c, 0xef, 0xfd, 0x6d,
        0xef, 0x7d, 0x6f, 0xec, 0xfa, 0x72, 0xf6, 0xfd,
        0x79, 0xf9, 0xfa, 0x6e, 0x7d, 0xf1, 0x6f, 0x7f,
        0xf3, 0x6f, 0xfe, 0xef, 0x73, 0x78, 0xf1, 0x77,
        0x7d, 0xee, 0x6e, 0x79, 0xeb, 0x6e, 0x7e, 0xea,
        0x6c, 0x7d, 0xea, 0x6b, 0x7b, 0xed, 0x6d, 0x7e,
        0xed, 0x6d, 0x79, 0xea, 0x70, 0x7c, 0xed, 0x6c,
        0x7b, 0xef, 0x6f, 0x7d, 0xf8, 0x71, 0xfc, 0xf4,
        0x71, 0x79, 0xf4, 0x76, 0x77, 0xf0, 0x7a, 0x71,
        0xee, 0x7f, 0x71, 0xf2, 0xfe, 0x77, 0xee, 0xfb,
        0x6e, 0xf2, 0xf4, 0x6e, 0xef, 0xf7, 0x6a, 0xee,
        0xfb, 0x6d, 0xef, 0x7d, 0x70, 0xea, 0xfa, 0x6c,
        0xee, 0xfa, 0x6c, 0xed, 0xfd, 0x6a, 0xed, 0xfa,
        0x6d, 0xee, 0x7e, 0x6f, 0xed, 0xfd, 0x72, 0xf7,
        0xfb, 0x76, 0xf7, 0xf7, 0x70, 0xf7, 0xf5, 0x76,
        0xf6, 0xfa, 0x70, 0xf6, 0xf0, 0x71, 0x7f, 0xf2,
        0x73, 0x7c, 0xed, 0x72, 0x78, 0xeb, 0x72, 0x79,
        0xed, 0x6c, 0x7d, 0xec, 0x6c, 0x7a, 0xf1, 0x70,
        0xfe, 0xec, 0x6f, 0x7a},

    {   //0x80, 0x00, 0x01, 0x9f, 0x00, 0x20, 0x9b, 0x30,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xea, 0x6e, 0x7f, 0xec,
        0x6a, 0x7e, 0xed, 0x6d, 0x7a, 0xf3, 0x6f, 0x7d,
        0xf0, 0x74, 0x7a, 0xf1, 0x78, 0x77, 0xf6, 0x6f,
        0x76, 0xf1, 0x77, 0x79, 0xf8, 0x7c, 0x7d, 0xf1,
        0xfa, 0x70, 0xf3, 0xfc, 0x6d, 0xef, 0x7d, 0x6b,
        0xed, 0xf9, 0x6d, 0xef, 0xfb, 0x6d, 0xee, 0xfa,
        0x6d, 0xf0, 0xf8, 0x6e, 0xee, 0xfd, 0x6a, 0xec,
        0xfa, 0x6e, 0xec, 0x7c, 0x70, 0xed, 0x7f, 0x70,
        0xf4, 0xfd, 0x6f, 0xf3, 0xfb, 0x6c, 0xf4, 0xf2,
        0x73, 0xf8, 0xfe, 0x75, 0xfc, 0xf7, 0x74, 0x7c,
        0xf9, 0x72, 0xf2, 0xfe, 0x74, 0xf6, 0xf6, 0x72,
        0xf0, 0xef, 0x66, 0xf1, 0xeb, 0x6c, 0x71, 0xe8,
        0x70, 0x6d, 0xe5, 0x74, 0x71, 0xec, 0xfe, 0x76,
        0xf8, 0x72, 0xf7, 0xf8, 0x6e, 0xee, 0xf9, 0x6c,
        0xf2, 0xeb, 0x68, 0x7b, 0xe7, 0x6e, 0x73, 0xeb,
        0x72, 0x6e, 0xef, 0xfd, 0x74, 0xfd, 0xf7, 0x7c,
        0x7d, 0xf8, 0x7b, 0xfd, 0x7b, 0x7c, 0xef, 0x6e,
        0x73, 0xea, 0x76, 0x6c, 0xea, 0xfd, 0x68, 0xec,
        0xef, 0x66, 0xfa, 0xef, 0x6a, 0xfb, 0xf9, 0x6c,
        0xf3, 0xfb, 0x77, 0xef, 0x77, 0x77, 0xef, 0x74,
        0x72, 0xee, 0x76, 0x6f},

    {   //0x80, 0x00, 0x01, 0xa0, 0x00, 0x20, 0x9b, 0xd0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xe7, 0x7f, 0x69, 0xe9,
        0xf0, 0x69, 0xf6, 0xf0, 0x6e, 0x7c, 0xef, 0x7a,
        0x73, 0xf6, 0xfb, 0x77, 0xfa, 0x78, 0x7d, 0xf4,
        0x7a, 0xf4, 0xfb, 0x73, 0xf4, 0xf9, 0x6d, 0xfb,
        0xee, 0x6c, 0xf7, 0xe3, 0x68, 0x74, 0xe4, 0x6c,
        0x6d, 0xeb, 0x72, 0x6e, 0xf1, 0x7b, 0x78, 0xfb,
        0x7a, 0xf8, 0xfa, 0x6e, 0xfe, 0xf1, 0x6e, 0xf8,
        0xef, 0x69, 0xfc, 0xee, 0x6d, 0x7e, 0xed, 0x72,
        0x72, 0xea, 0x7e, 0x6a, 0xf1, 0xf2, 0x6c, 0xfb,
        0xf2, 0x73, 0xfd, 0xf5, 0xfd, 0xfd, 0x7b, 0xfe,
        0xf4, 0x76, 0x6f, 0xef, 0x7c, 0x6d, 0xe7, 0xfd,
        0x6b, 0xe5, 0xf9, 0x6b, 0xeb, 0xfa, 0x6a, 0xee,
        0xee, 0x6a, 0xfc, 0xee, 0x72, 0xfd, 0xfd, 0x77,
        0xf9, 0x76, 0x7f, 0xef, 0x6e, 0x77, 0xeb, 0x77,
        0x6e, 0xf3, 0xf9, 0x6e, 0xf3, 0xee, 0x6b, 0xfb,
        0xee, 0x6d, 0x7a, 0xef, 0x70, 0x74, 0xec, 0x7f,
        0x6f, 0xf1, 0xfc, 0x78, 0xfa, 0x76, 0xfe, 0xf9,
        0x6e, 0xf2, 0xf0, 0x67, 0xf6, 0xeb, 0x69, 0x7c,
        0xeb, 0x6d, 0x79, 0xe7, 0x73, 0x73, 0xeb, 0x72,
        0x73, 0xef, 0x76, 0x7b, 0xf4, 0xfa, 0xfd, 0x7a,
        0x7e, 0xf8, 0x7e, 0x70},

    {   //0x80, 0x00, 0x01, 0xa1, 0x00, 0x20, 0x9c, 0x70,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xfa, 0xf5, 0x6d, 0xfd,
        0xea, 0x6f, 0x74, 0xe7, 0x7e, 0x6c, 0xf2, 0xfb,
        0x6c, 0xf6, 0xef, 0x6b, 0xf7, 0xf1, 0x6c, 0xf6,
        0xf8, 0x71, 0xf8, 0xf7, 0x7d, 0xf9, 0x77, 0x78,
        0xee, 0x73, 0x6e, 0xea, 0x7b, 0x6b, 0xe5, 0xf9,
        0x66, 0xec, 0xf1, 0x69, 0xfb, 0xfb, 0x70, 0xfc,
        0xf5, 0x7a, 0x7b, 0xf6, 0x7c, 0x7e, 0xff, 0x77,
        0xfd, 0x7b, 0x7e, 0xf4, 0x76, 0x75, 0xef, 0xfa,
        0x6d, 0xf6, 0xed, 0x6a, 0x7e, 0xe6, 0x6c, 0x70,
        0xe7, 0x73, 0x6f, 0xef, 0x77, 0x77, 0xf2, 0x7e,
        0x7d, 0xf6, 0x75, 0x7c, 0xf5, 0x6e, 0xfc, 0xf3,
        0x6d, 0xf0, 0xf1, 0x69, 0xf4, 0xea, 0x6a, 0x77,
        0xea, 0x70, 0x6f, 0xed, 0x7e, 0x6d, 0xf5, 0xf5,
        0x78, 0x7e, 0x7d, 0x7e, 0x7d, 0x7e, 0xf8, 0xfc,
        0x7d, 0xfe, 0xf7, 0x7a, 0x73, 0xef, 0x79, 0x76,
        0xe8, 0x77, 0x6d, 0xe8, 0xfb, 0x68, 0xef, 0xf1,
        0x68, 0xf9, 0xed, 0x6e, 0xfc, 0xef, 0x7c, 0xf7,
        0x7a, 0x75, 0xef, 0x76, 0x77, 0xed, 0x74, 0x75,
        0xed, 0x7d, 0x6e, 0xee, 0xfb, 0x6d, 0xed, 0xf7,
        0x69, 0xf8, 0xf1, 0x6d, 0x79, 0xf0, 0x78, 0x73,
        0xf3, 0xf7, 0x77, 0xfc},

    {   //0x80, 0x00, 0x01, 0xa2, 0x00, 0x20, 0x9d, 0x10,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xfb, 0x7f, 0x7d, 0x70,
        0xfb, 0xf7, 0x6e, 0xf2, 0xf4, 0x6c, 0xf9, 0xee,
        0x6d, 0x75, 0xeb, 0x6d, 0x76, 0xe6, 0x6d, 0x70,
        0xe8, 0x77, 0x73, 0xef, 0x7e, 0x7a, 0xfc, 0x7b,
        0xfa, 0x7c, 0x72, 0xf1, 0xf8, 0x69, 0xff, 0xef,
        0x6c, 0xfb, 0xeb, 0x6f, 0x7a, 0xef, 0x78, 0x6f,
        0xf5, 0xfe, 0x6e, 0xef, 0xf2, 0x6e, 0xf5, 0xef,
        0x74, 0x7d, 0xf8, 0x7c, 0xff, 0xfe, 0xfa, 0xf0,
        0x77, 0x7b, 0xe9, 0x7d, 0x6b, 0xea, 0xfc, 0x68,
        0xe9, 0xf8, 0x67, 0xef, 0xfa, 0x6a, 0xf9, 0xfc,
        0x6e, 0xf9, 0xf2, 0x75, 0x7d, 0xf9, 0x7b, 0xfc,
        0x74, 0x7a, 0xf2, 0x72, 0x7e, 0xeb, 0x78, 0x70,
        0xed, 0xf6, 0x6b, 0xfe, 0xee, 0x6f, 0x7e, 0xeb,
        0x6f, 0x78, 0xee, 0x78, 0x75, 0xf8, 0x7a, 0x73,
        0xef, 0xfb, 0x75, 0xf9, 0x79, 0xfc, 0xf9, 0x6d,
        0xf3, 0xf2, 0x6b, 0xf0, 0xea, 0x6a, 0xf9, 0xe4,
        0x6e, 0x70, 0xec, 0x75, 0x72, 0xec, 0x7d, 0x74,
        0xf3, 0x7a, 0x7c, 0x7f, 0x75, 0x7d, 0x7c, 0x7e,
        0xfa, 0x7a, 0x78, 0xf8, 0xf6, 0x6f, 0x7b, 0xee,
        0x6f, 0x73, 0xec, 0x7b, 0x6a, 0xed, 0xf2, 0x6b,
        0xfa, 0xef, 0x6f, 0xfb},

    {   //0x80, 0x00, 0x01, 0xa3, 0x00, 0x20, 0x9d, 0xb0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xee, 0x72, 0xf8, 0xf7,
        0x75, 0xf7, 0x79, 0x74, 0xf3, 0x7c, 0x7a, 0xef,
        0x77, 0x73, 0xea, 0x7c, 0x6a, 0xeb, 0xfb, 0x69,
        0xed, 0xf2, 0x6b, 0xf7, 0xef, 0x74, 0x78, 0xfe,
        0x7d, 0x7a, 0xff, 0xfa, 0xfe, 0x7e, 0x7c, 0xfa,
        0x7c, 0x73, 0xf4, 0xfd, 0x78, 0xf0, 0xfa, 0x6f,
        0xf9, 0xed, 0x6c, 0x76, 0xea, 0x6c, 0x71, 0xe7,
        0x76, 0x70, 0xec, 0xfc, 0x76, 0xfa, 0x77, 0x7e,
        0xf8, 0x7a, 0xf9, 0xf5, 0x71, 0xfa, 0xef, 0x6a,
        0xfa, 0xee, 0x6c, 0xf6, 0xee, 0x6b, 0x7b, 0xec,
        0x73, 0x6e, 0xf2, 0xff, 0x6f, 0xf7, 0xf4, 0x78,
        0x7d, 0xf6, 0xfb, 0x7a, 0x78, 0xfd, 0xfc, 0x79,
        0xfa, 0xf2, 0x7c, 0x78, 0xee, 0x7d, 0x6c, 0xed,
        0x7e, 0x6c, 0xea, 0xfb, 0x6a, 0xef, 0xf0, 0x6b,
        0xfa, 0xee, 0x6f, 0xff, 0xf3, 0x74, 0xfc, 0x7a,
        0x7a, 0xf0, 0x6e, 0x73, 0xed, 0x76, 0x78, 0xeb,
        0x7d, 0x6f, 0xef, 0xf8, 0x6c, 0xfa, 0xf6, 0x6d,
        0xf8, 0xf2, 0x6f, 0x78, 0xf3, 0x7e, 0x71, 0xf8,
        0xf9, 0x72, 0xfc, 0xfb, 0x7e, 0xff, 0x79, 0xf3,
        0xf9, 0x6d, 0xf7, 0xee, 0x6c, 0xfc, 0xec, 0x6c,
        0x79, 0xeb, 0x72, 0x6d},

    {   //0x80, 0x00, 0x01, 0xa4, 0x00, 0x20, 0x9e, 0x50,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xeb, 0x77, 0x6e, 0xe6,
        0x79, 0x6e, 0xeb, 0x7a, 0x74, 0xf7, 0x75, 0x7d,
        0xfc, 0x72, 0xf7, 0xf3, 0x6e, 0xf9, 0xec, 0x6e,
        0x71, 0xee, 0x77, 0x73, 0xed, 0xfc, 0x6f, 0xf7,
        0xef, 0x6f, 0xfd, 0xed, 0x6f, 0xfa, 0xee, 0x6e,
        0x7b, 0xf4, 0x79, 0x7d, 0xfe, 0x7b, 0xf9, 0xfe,
        0x78, 0xed, 0xfc, 0x6c, 0xea, 0xf6, 0x64, 0xf3,
        0xf0, 0x68, 0xef, 0xed, 0x6c, 0xf8, 0xef, 0x74,
        0x7c, 0xf6, 0x76, 0x7c, 0xf5, 0x7d, 0x7f, 0x7f,
        0x7f, 0xf2, 0x78, 0x77, 0xf4, 0x7b, 0x74, 0xf7,
        0xf4, 0x6e, 0x7c, 0xeb, 0x6e, 0x6e, 0xea, 0x79,
        0x6f, 0xea, 0xfd, 0x73, 0xf1, 0xfb, 0x71, 0xf9,
        0xff, 0x71, 0xf5, 0x7d, 0x78, 0xf3, 0x72, 0xfc,
        0xee, 0x6b, 0x7e, 0xec, 0x6a, 0x77, 0xe8, 0x6f,
        0x6e, 0xea, 0x7b, 0x6b, 0xf3, 0xfc, 0x72, 0xf4,
        0xf9, 0x7b, 0xfc, 0x7c, 0xfd, 0x7d, 0x79, 0xfb,
        0xfd, 0xfc, 0xfc, 0xf8, 0x7c, 0x78, 0xef, 0x7a,
        0x6d, 0xef, 0xfd, 0x6a, 0xf0, 0xed, 0x6c, 0xfb,
        0xe9, 0x6d, 0x76, 0xed, 0x72, 0x7e, 0xf1, 0x73,
        0xf8, 0xfc, 0x71, 0xf4, 0x78, 0x6e, 0xf1, 0xff,
        0x70, 0xf0, 0xff, 0x6a},

    {   //0x80, 0x00, 0x01, 0xa5, 0x00, 0x20, 0x9e, 0xf0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xed, 0xf4, 0x68, 0xf5,
        0xf1, 0x6c, 0xfc, 0xee, 0x79, 0x79, 0xf0, 0xfa,
        0x74, 0x7d, 0xfc, 0x7a, 0xff, 0xfa, 0xfc, 0xfa,
        0x7b, 0xf9, 0xfa, 0x6f, 0x7f, 0xf4, 0x75, 0x7e,
        0xed, 0x72, 0x73, 0xe6, 0x77, 0x6c, 0xe7, 0x7a,
        0x6d, 0xec, 0xfd, 0x6f, 0xf5, 0xfb, 0x7b, 0xfe,
        0x75, 0xfe, 0xf5, 0x75, 0xfc, 0xed, 0x6e, 0x7b,
        0xec, 0x6d, 0x77, 0xee, 0x71, 0x79, 0xee, 0x7d,
        0x70, 0xf2, 0xf3, 0x6e, 0xfd, 0xf3, 0x71, 0x79,
        0xf4, 0x7e, 0x78, 0xf9, 0xf7, 0xfe, 0x7b, 0x79,
        0xf8, 0x7e, 0x71, 0xf3, 0xfc, 0x6b, 0xf1, 0xf4,
        0x68, 0xf2, 0xef, 0x6b, 0xf1, 0xed, 0x6c, 0x7e,
        0xec, 0x70, 0x77, 0xf3, 0x74, 0x7e, 0xf9, 0x7d,
        0xf1, 0x7c, 0x7b, 0xec, 0x79, 0x6e, 0xef, 0xfc,
        0x6f, 0xf5, 0xf5, 0x6f, 0x7f, 0xee, 0x72, 0x75,
        0xf1, 0x77, 0x77, 0xf5, 0x7c, 0x6f, 0xfa, 0xf4,
        0x70, 0xfe, 0xf9, 0x77, 0xf9, 0x7e, 0xfc, 0xf7,
        0x70, 0xfb, 0xef, 0x6b, 0x74, 0xea, 0x70, 0x72,
        0xe9, 0x79, 0x6f, 0xeb, 0x7c, 0x6c, 0xef, 0x7b,
        0x6f, 0xf2, 0x7d, 0x78, 0xfd, 0x7d, 0xfa, 0x7b,
        0x77, 0xf8, 0xfa, 0x75},

    {   //0x80, 0x00, 0x01, 0xa6, 0x00, 0x20, 0x9f, 0x90,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xfb, 0xef, 0x71, 0x7a,
        0xec, 0x79, 0x6f, 0xf0, 0xf7, 0x72, 0xf4, 0xee,
        0x75, 0x7e, 0xee, 0x73, 0x76, 0xf0, 0x74, 0xff,
        0xf1, 0x79, 0x7f, 0xfc, 0x7e, 0xf9, 0x7c, 0x74,
        0xf3, 0x7f, 0x6b, 0xef, 0xf5, 0x68, 0xee, 0xed,
        0x69, 0xf8, 0xef, 0x6e, 0xfc, 0xf0, 0x77, 0x7d,
        0xf7, 0x7f, 0x7c, 0x7e, 0x7f, 0xf8, 0xfa, 0xfe,
        0xf4, 0x7a, 0x79, 0xf2, 0x79, 0x71, 0xf9, 0xfb,
        0x72, 0x7c, 0xed, 0x76, 0x71, 0xea, 0x7f, 0x6b,
        0xf1, 0xf9, 0x6e, 0xf5, 0xf8, 0x72, 0xfb, 0xfc,
        0x7a, 0xfa, 0x7b, 0x77, 0xf4, 0x75, 0x79, 0xf0,
        0x6c, 0x7e, 0xec, 0x6b, 0x74, 0xec, 0x71, 0x6f,
        0xeb, 0xfe, 0x6e, 0xf0, 0xf9, 0x6f, 0x7e, 0xfc,
        0x7f, 0x7d, 0xfe, 0xf7, 0x7f, 0x7c, 0xf6, 0xfd,
        0x76, 0x7f, 0xfa, 0x7d, 0x78, 0xf6, 0x7f, 0x6f,
        0xef, 0xf9, 0x6b, 0xf8, 0xf2, 0x6d, 0xff, 0xeb,
        0x6f, 0x75, 0xeb, 0x75, 0x75, 0xf8, 0x7a, 0xf8,
        0xf8, 0x78, 0xf1, 0xfc, 0x73, 0xee, 0x7e, 0x6d,
        0xf4, 0xfb, 0x6e, 0xf4, 0xf3, 0x6b, 0xf4, 0xee,
        0x6c, 0x7c, 0xf3, 0x78, 0x76, 0xf8, 0xf8, 0x78,
        0xfb, 0xf0, 0x7e, 0x78},

    {   //0x80, 0x00, 0x01, 0xa7, 0x00, 0x20, 0xa0, 0x30,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xfc, 0xf8, 0xfc, 0x7f,
        0xf7, 0xf7, 0x74, 0xfe, 0xf3, 0x72, 0x77, 0xee,
        0x7c, 0x76, 0xeb, 0x7b, 0x6d, 0xea, 0xfe, 0x6a,
        0xf0, 0xfe, 0x6f, 0xf8, 0xfc, 0x7c, 0xfc, 0x7b,
        0xfb, 0xfa, 0x6e, 0xfe, 0xf0, 0x6f, 0x7d, 0xed,
        0x70, 0x75, 0xec, 0x74, 0x71, 0xf2, 0xfd, 0x78,
        0xf9, 0xf8, 0x73, 0x7e, 0xf1, 0x74, 0x74, 0xf7,
        0xfe, 0x79, 0x7f, 0xf7, 0x7c, 0x7c, 0xfc, 0xfd,
        0x7a, 0x6f, 0xf3, 0xf6, 0x6d, 0xf1, 0xee, 0x6c,
        0xf7, 0xef, 0x6a, 0x7f, 0xef, 0x6f, 0xfe, 0xee,
        0x75, 0x7b, 0xf3, 0x7c, 0x7b, 0xff, 0x76, 0xf8,
        0x79, 0x75, 0xee, 0x76, 0x6f, 0xef, 0xff, 0x6c,
        0xf7, 0xf2, 0x73, 0xfc, 0xee, 0x7b, 0x77, 0xf2,
        0x7e, 0x71, 0xf9, 0xf9, 0x77, 0xfa, 0xf4, 0x76,
        0x7e, 0xf4, 0x79, 0x7d, 0x7e, 0x7b, 0xf8, 0x75,
        0x7e, 0xef, 0x71, 0x7d, 0xe9, 0x73, 0x6e, 0xec,
        0x7e, 0x6d, 0xee, 0xfc, 0x6e, 0xf1, 0xfa, 0x73,
        0xfa, 0x7e, 0xfd, 0xf6, 0x7e, 0xf6, 0xfd, 0x77,
        0xf6, 0xfc, 0x72, 0xfe, 0xf2, 0x75, 0x78, 0xef,
        0x79, 0x73, 0xef, 0xfa, 0x6e, 0x7b, 0xf2, 0x73,
        0x75, 0xef, 0x79, 0x72},

    {   //0x80, 0x00, 0x01, 0xa8, 0x00, 0x20, 0xa0, 0xd0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xf2, 0x7c, 0x76, 0xfa,
        0x7a, 0xfc, 0xfb, 0x79, 0xfc, 0x7d, 0x77, 0xf5,
        0xfd, 0x6d, 0xf2, 0xf1, 0x6b, 0xf1, 0xec, 0x6a,
        0xfb, 0xec, 0x6f, 0x78, 0xf7, 0x7b, 0x7c, 0xfb,
        0xfd, 0xfc, 0x7c, 0xfe, 0xfb, 0x78, 0x78, 0xf9,
        0x7e, 0x79, 0xf5, 0x7f, 0x76, 0xf3, 0xf8, 0x75,
        0x7c, 0xf0, 0x7d, 0x71, 0xee, 0xfd, 0x6c, 0xf1,
        0xf7, 0x6c, 0x7f, 0xf7, 0x79, 0xfb, 0xf9, 0x7d,
        0xfc, 0x7b, 0x7b, 0xf7, 0x71, 0x7a, 0xee, 0x75,
        0x78, 0xeb, 0x6f, 0x79, 0xeb, 0x75, 0x74, 0xf0,
        0xfa, 0x6a, 0xed, 0xfe, 0x6c, 0xec, 0xf7, 0x71,
        0x75, 0xf5, 0xf9, 0x6f, 0x7b, 0xea, 0x7f, 0x6d,
        0x7b, 0xef, 0x7d, 0x6a, 0xf0, 0xee, 0x6f, 0x7e,
        0xf2, 0x7c, 0x7c, 0xfc, 0x76, 0xfd, 0xf2, 0x6c,
        0xfb, 0xe9, 0x6f, 0x73, 0xed, 0xf3, 0x74, 0x73,
        0xfa, 0xee, 0x71, 0x6f, 0xe9, 0xfa, 0x68, 0xf3,
        0xec, 0x6f, 0x7c, 0xf3, 0x7b, 0xfe, 0x7d, 0x70,
        0xf8, 0xf5, 0x6f, 0x79, 0xf3, 0xfe, 0x6d, 0x7d,
        0xe9, 0x78, 0x6d, 0xee, 0xef, 0x6e, 0x6c, 0xf0,
        0xf4, 0x6b, 0x7c, 0xe9, 0x7d, 0x6e, 0xf0, 0xfc,
        0x6c, 0xfc, 0xfc, 0x79},

    {   //0x80, 0x00, 0x01, 0xa9, 0x00, 0x20, 0xa1, 0x70,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xee, 0x7b, 0x6e, 0xeb,
        0xfb, 0x69, 0xf7, 0xf2, 0x7c, 0x7b, 0xf4, 0xed,
        0x7a, 0x6e, 0xef, 0xed, 0x64, 0x79, 0xe2, 0x76,
        0x6d, 0xec, 0xf6, 0x6d, 0x7c, 0xf9, 0x7b, 0x77,
        0x7f, 0xfb, 0xfe, 0xfe, 0x72, 0x7b, 0xef, 0x79,
        0x70, 0xf4, 0xf1, 0x70, 0x74, 0xee, 0xf8, 0x6d,
        0x7b, 0xe9, 0xfc, 0x65, 0xef, 0xe8, 0x66, 0x76,
        0xe9, 0x75, 0x72, 0xf2, 0x7c, 0x7c, 0xfe, 0x74,
        0xf5, 0xf5, 0x71, 0x7b, 0xef, 0xfe, 0x77, 0x79,
        0xfe, 0xf5, 0x6c, 0x76, 0xe6, 0x79, 0x6b, 0xea,
        0xed, 0x68, 0x6f, 0xea, 0x7e, 0x6c, 0xf5, 0xee,
        0x75, 0x77, 0xf5, 0x7e, 0x72, 0x7a, 0xfe, 0xfa,
        0xfc, 0x6f, 0x7e, 0xf6, 0x72, 0x76, 0x7b, 0xf7,
        0xf6, 0x6f, 0xf5, 0xe7, 0x6c, 0x6e, 0xe4, 0xfc,
        0x62, 0xed, 0xe9, 0x6f, 0xfe, 0xef, 0xf7, 0x7a,
        0x76, 0xf1, 0xf4, 0x6f, 0xfd, 0xef, 0x7a, 0x76,
        0x7c, 0x7b, 0xf8, 0xfd, 0x72, 0xf8, 0xef, 0x6f,
        0x71, 0xeb, 0xfc, 0x69, 0xf7, 0xe7, 0x6f, 0x69,
        0xea, 0xf5, 0x6a, 0xfb, 0xef, 0x7c, 0x79, 0xff,
        0xf2, 0x7f, 0x6d, 0xf9, 0xf2, 0x71, 0xff, 0xf6,
        0x7e, 0xf4, 0x7c, 0x6b},

    {   //0x80, 0x00, 0x01, 0xaa, 0x00, 0x20, 0xa2, 0x10,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xf4, 0xf6, 0x65, 0xf5,
        0xe7, 0x6c, 0x75, 0xea, 0xf9, 0x6b, 0x75, 0xed,
        0xfa, 0x6e, 0xfa, 0xeb, 0x73, 0x6d, 0xf2, 0x7e,
        0x6d, 0x7f, 0xf3, 0xfc, 0x7e, 0x7a, 0x7c, 0xfd,
        0x7d, 0x71, 0x77, 0xee, 0x7c, 0x6d, 0xea, 0xef,
        0x67, 0xfc, 0xe9, 0x75, 0x6c, 0xf4, 0xed, 0x7a,
        0x71, 0xf8, 0xef, 0x6c, 0x74, 0xe8, 0x79, 0x6c,
        0xee, 0xfe, 0x6f, 0xfc, 0x71, 0x7b, 0xf1, 0x72,
        0x72, 0xee, 0xf9, 0x6e, 0xfc, 0xed, 0x7d, 0x70,
        0xf7, 0xec, 0x71, 0x6b, 0xec, 0xf3, 0x6c, 0xfc,
        0xe9, 0xfd, 0x6e, 0xf5, 0xef, 0x6d, 0x72, 0xf1,
        0x7b, 0x78, 0xf9, 0x7e, 0xf7, 0xf2, 0x6f, 0x79,
        0xed, 0x76, 0x6f, 0xec, 0xf4, 0x74, 0x7c, 0xf7,
        0xf2, 0x6a, 0x70, 0xe5, 0xf9, 0x69, 0xec, 0xea,
        0x68, 0x76, 0xee, 0x77, 0x71, 0xfb, 0xf5, 0xfb,
        0x7b, 0x78, 0xf8, 0xf8, 0x79, 0x78, 0x7e, 0xf6,
        0x72, 0x70, 0xef, 0xfe, 0x6d, 0xfe, 0xec, 0x7d,
        0x6b, 0xf6, 0xea, 0x6f, 0x6d, 0xec, 0xf9, 0x68,
        0xf5, 0xeb, 0x6f, 0x7e, 0xf1, 0xfd, 0xf8, 0x78,
        0x79, 0xed, 0xfe, 0x73, 0xf9, 0xfe, 0x7b, 0x77,
        0x76, 0xf0, 0xf9, 0x6d},

    {   //0x80, 0x00, 0x01, 0xab, 0x00, 0x20, 0xa2, 0xb0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xf6, 0xe9, 0x69, 0x6b,
        0xe8, 0x7b, 0x69, 0xf7, 0xed, 0x75, 0x70, 0xf2,
        0xf6, 0x6e, 0x76, 0xf1, 0xfc, 0x7b, 0x7e, 0xf9,
        0xf8, 0x7a, 0x77, 0x7f, 0xfb, 0x7b, 0x7a, 0xf7,
        0xf2, 0x78, 0x6f, 0xee, 0xf4, 0x63, 0x7f, 0xe5,
        0x6f, 0x6f, 0xea, 0xf8, 0x6e, 0x7d, 0xf4, 0xfc,
        0x74, 0x7b, 0xed, 0xfd, 0x6c, 0xfa, 0xf6, 0x74,
        0xfe, 0x7f, 0x7a, 0xf3, 0x78, 0x72, 0xee, 0xfd,
        0x6c, 0xf8, 0xeb, 0x76, 0x6d, 0xec, 0xed, 0x6d,
        0x78, 0xea, 0xfd, 0x6d, 0xf5, 0xee, 0x75, 0x72,
        0xf8, 0xf2, 0x77, 0x71, 0xf0, 0xf2, 0x77, 0xf7,
        0x7f, 0x79, 0xf7, 0x70, 0x79, 0xeb, 0x77, 0x6e,
        0xea, 0xf3, 0x67, 0x7b, 0xed, 0x79, 0x72, 0xff,
        0xef, 0x7c, 0x6b, 0xf2, 0xef, 0x6a, 0x77, 0xea,
        0xff, 0x74, 0xf9, 0xfa, 0x7c, 0x77, 0x76, 0x7d,
        0xfb, 0x79, 0x79, 0xf3, 0xf6, 0x6f, 0x76, 0xeb,
        0x7e, 0x69, 0xf1, 0xeb, 0x74, 0x74, 0xf1, 0xf4,
        0x6e, 0x73, 0xed, 0xf9, 0x6a, 0xf5, 0xea, 0x6d,
        0x73, 0xf3, 0x7a, 0x7e, 0xf8, 0x78, 0xf7, 0xf5,
        0x6e, 0xfc, 0xf0, 0x74, 0x77, 0xf4, 0xef, 0x7a,
        0x6f, 0xf2, 0xf2, 0x6d},

    {   //0x80, 0x00, 0x01, 0xac, 0x00, 0x20, 0xa3, 0x50,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x75, 0xea, 0xfd, 0x6c,
        0xef, 0xed, 0x6e, 0x72, 0xf2, 0xf7, 0x70, 0x7d,
        0xef, 0xfe, 0x7e, 0xfc, 0x79, 0xfb, 0xff, 0x74,
        0xf0, 0xf0, 0x74, 0x7f, 0xee, 0xff, 0x6d, 0x78,
        0xf0, 0xf4, 0x6e, 0xfb, 0xe7, 0x6d, 0x6a, 0xea,
        0x7d, 0x67, 0xf9, 0xef, 0x79, 0x7b, 0xfe, 0xfa,
        0xfd, 0x74, 0x7d, 0xf8, 0x7b, 0x76, 0xff, 0xfb,
        0xfe, 0x74, 0x79, 0xee, 0x7d, 0x6c, 0xf5, 0xee,
        0x72, 0x6e, 0xf4, 0xf7, 0x69, 0x7b, 0xea, 0x75,
        0x6e, 0xee, 0xf1, 0x71, 0x7d, 0xf7, 0xfa, 0x7e,
        0x75, 0xfc, 0xf6, 0x75, 0x7a, 0xf4, 0x7e, 0xff,
        0xfc, 0xfc, 0xed, 0x75, 0x6a, 0xee, 0xf7, 0x68,
        0xfe, 0xea, 0x74, 0x70, 0xef, 0xf7, 0x70, 0x74,
        0xef, 0xf3, 0x6f, 0x7c, 0xec, 0xfd, 0x78, 0xfd,
        0xfc, 0xfb, 0x7d, 0xfa, 0xef, 0xfa, 0x7c, 0x7d,
        0xf7, 0xfe, 0x6b, 0x79, 0xeb, 0xfe, 0x6d, 0xed,
        0xeb, 0x6b, 0x7b, 0xed, 0x76, 0x6e, 0x7f, 0xf3,
        0xfc, 0x6c, 0x7b, 0xef, 0x71, 0x6f, 0xf3, 0x7f,
        0x77, 0xfa, 0x7a, 0xff, 0xfe, 0x6e, 0x7c, 0xee,
        0x71, 0x6e, 0xee, 0xf4, 0x72, 0x77, 0xef, 0xfc,
        0x6d, 0xfb, 0xec, 0x77},

    {   //0x80, 0x00, 0x01, 0xad, 0x00, 0x20, 0xa3, 0xf0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x6d, 0xef, 0xf4, 0x6f,
        0x7a, 0xef, 0xf8, 0x76, 0xfd, 0xf9, 0x7a, 0x77,
        0x7f, 0x7e, 0xfd, 0xfe, 0x71, 0xf3, 0xed, 0x6b,
        0x76, 0xeb, 0x7a, 0x6f, 0xf5, 0xef, 0x79, 0x72,
        0xf5, 0xf1, 0x6e, 0x70, 0xe7, 0xf6, 0x6a, 0xf4,
        0xed, 0x6f, 0x77, 0xfa, 0x7a, 0x7d, 0x7d, 0xfd,
        0xf5, 0xfb, 0x79, 0x7d, 0xf4, 0x7e, 0x6c, 0xfe,
        0xec, 0x79, 0x6f, 0xee, 0xf2, 0x6b, 0x79, 0xef,
        0xff, 0x6d, 0xfa, 0xe8, 0x77, 0x6c, 0xed, 0xf4,
        0x6d, 0xf9, 0xf5, 0x76, 0xfb, 0x7e, 0x7c, 0xf4,
        0x7a, 0x73, 0xf4, 0xf6, 0x70, 0x7b, 0xf6, 0xfa,
        0x77, 0x72, 0xed, 0xf5, 0x69, 0xf8, 0xea, 0x6c,
        0x6c, 0xee, 0xfa, 0x6c, 0x7b, 0xee, 0xfc, 0x75,
        0xfc, 0xf8, 0x77, 0x7b, 0xfc, 0xfd, 0xfb, 0x7d,
        0x7b, 0xf0, 0xfe, 0x6c, 0xfd, 0xf1, 0xfd, 0x76,
        0xfb, 0xed, 0x77, 0x6c, 0xf0, 0xf3, 0x67, 0x7c,
        0xe7, 0x75, 0x6d, 0xef, 0xf9, 0x75, 0x78, 0x7a,
        0xf9, 0x7a, 0x76, 0xf1, 0xf9, 0x77, 0xff, 0x7e,
        0xfb, 0x7e, 0x70, 0xfb, 0xee, 0x77, 0x6f, 0xef,
        0xf8, 0x6c, 0xfe, 0xec, 0x7b, 0x6d, 0xf2, 0xeb,
        0x6f, 0x71, 0xee, 0xfa},

    {   //0x80, 0x00, 0x01, 0xae, 0x00, 0x20, 0xa4, 0x90,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x72, 0xfd, 0xf9, 0x7f,
        0x79, 0x7b, 0xf5, 0xfe, 0x75, 0xfb, 0xf5, 0xfc,
        0x7a, 0x74, 0xfb, 0xf2, 0x6d, 0x73, 0xe7, 0x7b,
        0x6c, 0xeb, 0xf1, 0x6c, 0x78, 0xef, 0xfa, 0x71,
        0x79, 0xee, 0xfc, 0x6f, 0xfa, 0xf7, 0x70, 0x78,
        0xf2, 0xfb, 0x7e, 0xfe, 0x7c, 0xf9, 0x79, 0x6e,
        0x7d, 0xf6, 0xfe, 0x71, 0xf7, 0xec, 0x6e, 0x73,
        0xec, 0x7d, 0x6a, 0xf8, 0xeb, 0x79, 0x74, 0xf6,
        0xf1, 0x7b, 0x71, 0xf4, 0xf5, 0x6e, 0xff, 0xf1,
        0x74, 0x7a, 0x7d, 0x77, 0xf7, 0x78, 0x6e, 0xf4,
        0xf1, 0x6f, 0x7a, 0xef, 0x79, 0x6f, 0xfb, 0xef,
        0x7d, 0x6e, 0xf2, 0xed, 0x6c, 0x74, 0xed, 0xfa,
        0x72, 0xfb, 0xf4, 0x77, 0x6f, 0xfd, 0xf6, 0x79,
        0xfe, 0xf8, 0xf7, 0xf8, 0x76, 0x78, 0xf2, 0xfc,
        0x6d, 0xf9, 0xed, 0x70, 0x75, 0xf3, 0x7f, 0x70,
        0x72, 0xee, 0xf0, 0x6d, 0xfd, 0xe9, 0x76, 0x6d,
        0xf1, 0xfd, 0x6f, 0xf9, 0xf5, 0xf8, 0xf9, 0x7d,
        0xf6, 0xf2, 0x75, 0x74, 0xf6, 0xf6, 0x7e, 0x77,
        0xf4, 0xf2, 0x6e, 0x78, 0xef, 0x7f, 0x6e, 0xf7,
        0xeb, 0x76, 0x6d, 0xf7, 0xf5, 0x6d, 0x76, 0xee,
        0x7d, 0x70, 0xf7, 0xf8},

    {   //0x80, 0x00, 0x01, 0xaf, 0x00, 0x20, 0xa5, 0x30,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x7b, 0x7d, 0x7a, 0xfa,
        0xf7, 0x6f, 0x79, 0xf2, 0x79, 0x72, 0x7c, 0xf8,
        0xfd, 0x74, 0xfd, 0xee, 0x76, 0x6b, 0xed, 0xf4,
        0x68, 0x7c, 0xed, 0x7d, 0x77, 0xf9, 0xf7, 0x78,
        0x74, 0xf9, 0xf6, 0x7a, 0x7c, 0xf7, 0xf6, 0x7b,
        0x73, 0xfd, 0xf5, 0xfe, 0x76, 0xf2, 0xf0, 0x76,
        0x79, 0xf9, 0xfa, 0x6d, 0x74, 0xe9, 0xfe, 0x6b,
        0xef, 0xee, 0x6f, 0x77, 0xf4, 0x7f, 0x79, 0x7b,
        0xf6, 0xf4, 0x74, 0x7a, 0xf3, 0x7d, 0x71, 0x7f,
        0xfd, 0xfa, 0xfc, 0x73, 0xf7, 0xf9, 0x6a, 0x7d,
        0xee, 0x76, 0x6f, 0xef, 0xed, 0x73, 0x74, 0xf1,
        0xf6, 0x70, 0x7b, 0xee, 0x7d, 0x75, 0xf8, 0xf7,
        0x7d, 0x79, 0xfb, 0xf1, 0x7d, 0x72, 0xfc, 0xfe,
        0x7b, 0x78, 0x79, 0xf1, 0x7f, 0x6f, 0xf4, 0xee,
        0x6c, 0x73, 0xeb, 0x7d, 0x6f, 0xfa, 0xf4, 0xfe,
        0x73, 0x7d, 0xf0, 0x76, 0x70, 0xee, 0xf4, 0x74,
        0xfa, 0xf5, 0xff, 0x79, 0x72, 0x7e, 0xf6, 0x7d,
        0x77, 0xf1, 0xf5, 0x71, 0x7a, 0xf4, 0x7f, 0x6c,
        0xfe, 0xeb, 0x79, 0x6f, 0xf2, 0xf6, 0x74, 0x75,
        0xf5, 0xf9, 0x72, 0x7d, 0xf0, 0x7d, 0x6d, 0xfb,
        0xf9, 0x76, 0x78, 0x7e},

    {   //0x80, 0x00, 0x01, 0xb0, 0x00, 0x20, 0xa5, 0xd0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xfc, 0xf2, 0x77, 0x76,
        0xed, 0x78, 0x6e, 0xf5, 0xf6, 0x75, 0x77, 0xf5,
        0xf5, 0x73, 0x71, 0xf1, 0xf4, 0x6f, 0xfb, 0xed,
        0x74, 0x72, 0xf6, 0xf8, 0x79, 0x7e, 0xf4, 0xf3,
        0x7d, 0x79, 0xfc, 0xfb, 0x7e, 0x74, 0xfe, 0xf2,
        0x75, 0x75, 0xf0, 0xfb, 0x6e, 0x7c, 0xef, 0x7e,
        0x72, 0xfd, 0xee, 0xff, 0x6c, 0xf6, 0xf4, 0x6c,
        0x7f, 0xeb, 0xfe, 0x7e, 0xf6, 0xfa, 0xf8, 0x77,
        0x73, 0xf7, 0xf7, 0x76, 0x7f, 0xf0, 0x7e, 0x75,
        0x7b, 0xfb, 0xff, 0x6e, 0xfa, 0xed, 0x71, 0x6f,
        0xf2, 0xf8, 0x6e, 0x76, 0xf2, 0x7f, 0x71, 0xfa,
        0xf1, 0x78, 0x74, 0xfa, 0xfc, 0x7d, 0x78, 0x7f,
        0xf3, 0xfe, 0x72, 0xfc, 0xf5, 0x71, 0x79, 0xfa,
        0xfd, 0x79, 0x6e, 0xf5, 0xf1, 0x6c, 0x79, 0xeb,
        0x7e, 0x6f, 0xf2, 0xf4, 0x72, 0x7a, 0xf9, 0xfa,
        0x7b, 0x78, 0xf4, 0xf3, 0x73, 0x7a, 0xf7, 0xfb,
        0xfe, 0x7c, 0xf4, 0xf2, 0x77, 0x78, 0xf7, 0x7c,
        0x6d, 0x7b, 0xee, 0x7e, 0x6f, 0xf7, 0xec, 0x77,
        0x71, 0xf0, 0x7d, 0x6f, 0xfb, 0xf5, 0xfe, 0x7c,
        0x7b, 0xf2, 0xfd, 0x71, 0xfe, 0xef, 0xff, 0xfc,
        0xfa, 0x7e, 0xf1, 0x67},

    {   //0x80, 0x00, 0x01, 0xb1, 0x00, 0x20, 0xa6, 0x70,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xfe, 0xe8, 0x6c, 0x6d,
        0xe4, 0xef, 0x62, 0xf7, 0xeb, 0x75, 0x6a, 0xf3,
        0xf6, 0x6d, 0x7d, 0xf6, 0x7c, 0x6f, 0xf3, 0xfe,
        0x77, 0xf5, 0xfa, 0x70, 0xf9, 0xed, 0x6e, 0x79,
        0xee, 0xfb, 0x6c, 0xf9, 0xe7, 0x6b, 0x6a, 0xe4,
        0xf6, 0x61, 0xf7, 0xe6, 0x6d, 0x6c, 0xec, 0xf6,
        0x6a, 0x77, 0xeb, 0xfd, 0x6d, 0xef, 0xeb, 0x6e,
        0x76, 0xf0, 0x6f, 0x76, 0xed, 0x7a, 0x7b, 0xec,
        0xfc, 0x70, 0xfb, 0xf3, 0x6d, 0x72, 0xea, 0xfd,
        0x68, 0xf8, 0xe8, 0x6c, 0x6a, 0xe8, 0xfa, 0x67,
        0xf4, 0xe9, 0x71, 0x6c, 0xee, 0xf6, 0x69, 0xff,
        0xee, 0x74, 0xfb, 0xed, 0x75, 0x7b, 0xef, 0x74,
        0x6e, 0xee, 0x7f, 0x6b, 0xf3, 0xee, 0x6f, 0x6c,
        0xef, 0xed, 0x69, 0x7a, 0xe5, 0x73, 0x67, 0xeb,
        0xf0, 0x67, 0xfc, 0xe9, 0x76, 0x74, 0xf1, 0xfb,
        0x76, 0xfa, 0xfb, 0x75, 0xfc, 0xf3, 0x78, 0x7a,
        0xee, 0x78, 0x6d, 0xec, 0xf1, 0x6b, 0x7e, 0xe8,
        0x7d, 0x68, 0xf0, 0xec, 0x66, 0x72, 0xe5, 0x7d,
        0x69, 0xec, 0xed, 0x6b, 0x79, 0xf3, 0x73, 0x7c,
        0xef, 0xfd, 0xfc, 0xf9, 0xfc, 0x77, 0x7b, 0xf3,
        0x71, 0x77, 0xe9, 0xf9},

    {   //0x80, 0x00, 0x01, 0xb2, 0x00, 0x20, 0xa7, 0x10,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x68, 0xf2, 0xe8, 0x69,
        0x6e, 0xe7, 0x7e, 0x6a, 0xef, 0xea, 0x6e, 0x6f,
        0xeb, 0xfc, 0x6a, 0xfc, 0xf2, 0x70, 0xfe, 0xf2,
        0x78, 0x7c, 0xf6, 0x7d, 0x78, 0xf3, 0xfa, 0x71,
        0xfb, 0xed, 0x70, 0x6b, 0xe9, 0xf0, 0x65, 0xf7,
        0xe6, 0x69, 0x6c, 0xe9, 0xff, 0x67, 0xfd, 0xed,
        0x78, 0x6f, 0xf8, 0xf7, 0x71, 0x7e, 0xf4, 0x77,
        0x7b, 0xf1, 0x75, 0x74, 0xef, 0x71, 0x6d, 0xec,
        0xf3, 0x6b, 0xfc, 0xe7, 0x75, 0x6a, 0xec, 0xf3,
        0x65, 0x7d, 0xe7, 0x77, 0x6d, 0xec, 0xf1, 0x6b,
        0x78, 0xf2, 0x7c, 0x77, 0xf4, 0xf6, 0x74, 0x7f,
        0xf5, 0x70, 0x7c, 0xee, 0x6f, 0x7c, 0xe6, 0x79,
        0x69, 0xed, 0xef, 0x67, 0x79, 0xea, 0x7a, 0x6d,
        0xf2, 0xeb, 0x6d, 0x6e, 0xe9, 0xfb, 0x69, 0xf9,
        0xf2, 0x6f, 0x7b, 0xf2, 0x79, 0x78, 0xf5, 0x7e,
        0x76, 0xf1, 0xfa, 0x70, 0xf6, 0xed, 0x6c, 0x6f,
        0xe6, 0xfd, 0x68, 0xef, 0xeb, 0x6c, 0x72, 0xeb,
        0xf6, 0x6c, 0xfb, 0xe8, 0x77, 0x6c, 0xef, 0xfb,
        0x6b, 0xf2, 0xef, 0x74, 0xf6, 0xef, 0x74, 0x7d,
        0xf2, 0x6e, 0x77, 0xec, 0xfb, 0x6e, 0xfb, 0xeb,
        0x74, 0x6b, 0xed, 0xf6},

    {   //0x80, 0x00, 0x01, 0xb3, 0x00, 0x20, 0xa7, 0xb0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x67, 0xfb, 0xe7, 0x71,
        0x6a, 0xee, 0xf7, 0x68, 0x7b, 0xed, 0x79, 0x76,
        0xee, 0xfa, 0x72, 0xf8, 0xf8, 0x6e, 0xf9, 0xf7,
        0x6e, 0xfa, 0xee, 0x73, 0x6e, 0xf2, 0xf4, 0x6b,
        0x7a, 0xeb, 0x7c, 0x69, 0xf3, 0xec, 0x66, 0x72,
        0xe5, 0x7a, 0x6b, 0xee, 0xf7, 0x6d, 0xfe, 0xf8,
        0x75, 0x79, 0xf8, 0xfd, 0x7a, 0xf9, 0xfd, 0x71,
        0xf6, 0xf0, 0x6f, 0x76, 0xe9, 0xfc, 0x69, 0xf3,
        0xf0, 0x6a, 0x71, 0xe9, 0xf8, 0x69, 0xef, 0xe6,
        0x6d, 0x70, 0xec, 0x7a, 0x6e, 0xf0, 0xf5, 0x78,
        0xf8, 0xf7, 0x78, 0x7f, 0xfa, 0x7a, 0x7d, 0xf5,
        0xf8, 0x6d, 0x7d, 0xea, 0x6d, 0x6b, 0xea, 0xfa,
        0x67, 0xf1, 0xe7, 0x6e, 0x72, 0xe9, 0xf8, 0x6b,
        0x7f, 0xee, 0x78, 0x76, 0xef, 0xfe, 0x71, 0xfb,
        0xf8, 0x76, 0xf8, 0xf8, 0x72, 0xfe, 0xf4, 0x73,
        0x6d, 0xf0, 0xef, 0x6b, 0x7e, 0xe8, 0x72, 0x69,
        0xeb, 0xf4, 0x65, 0xfc, 0xe8, 0x73, 0x6f, 0xf3,
        0xf6, 0x73, 0x7c, 0xf2, 0x7e, 0x73, 0xf3, 0xf8,
        0x70, 0xfa, 0xfe, 0x6e, 0xf7, 0xef, 0x6f, 0x7c,
        0xe8, 0x7a, 0x6c, 0xee, 0xf1, 0x6a, 0x77, 0xe8,
        0x7d, 0x6b, 0xee, 0xec},

    {   //0x80, 0x00, 0x01, 0xb4, 0x00, 0x20, 0xa8, 0x50,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x6b, 0x74, 0xee, 0x7b,
        0x73, 0xf4, 0xf2, 0x78, 0x7c, 0xf3, 0x78, 0x72,
        0xf5, 0x7e, 0x78, 0xee, 0xf7, 0x6b, 0xfa, 0xeb,
        0x68, 0x70, 0xea, 0x78, 0x6b, 0xf3, 0xee, 0x6d,
        0x6f, 0xee, 0xfa, 0x69, 0x7c, 0xeb, 0x75, 0x71,
        0xed, 0x7f, 0x6d, 0xf7, 0xf8, 0x72, 0xf9, 0xf7,
        0x75, 0xfc, 0xf3, 0x73, 0x6e, 0xec, 0xf4, 0x6c,
        0xf9, 0xec, 0x6e, 0x6e, 0xea, 0xf7, 0x6a, 0xfa,
        0xeb, 0x73, 0x6c, 0xf4, 0xf3, 0x6d, 0x7e, 0xeb,
        0x78, 0x76, 0xec, 0xfd, 0x71, 0xf2, 0x7e, 0x71,
        0xf0, 0xf6, 0x70, 0xf9, 0xec, 0x75, 0x6f, 0xf2,
        0xf6, 0x6e, 0x7b, 0xe8, 0x7c, 0x69, 0xec, 0xee,
        0x69, 0x78, 0xeb, 0x7a, 0x70, 0xef, 0xf2, 0x72,
        0xfe, 0xf3, 0x73, 0x79, 0xf2, 0x7e, 0x7c, 0xef,
        0x7e, 0x6d, 0xf2, 0xf2, 0x6b, 0x78, 0xee, 0x7b,
        0x6c, 0xfa, 0xed, 0x6b, 0x6d, 0xe9, 0xfb, 0x66,
        0xf5, 0xec, 0x6b, 0x79, 0xeb, 0x76, 0x72, 0xf3,
        0xfc, 0x77, 0xfd, 0xfc, 0x7a, 0xfd, 0xf8, 0x78,
        0x72, 0xef, 0xf9, 0x6b, 0xfc, 0xef, 0x6d, 0x6f,
        0xea, 0xf9, 0x69, 0xf6, 0xe9, 0x6f, 0x6c, 0xee,
        0xfa, 0x6a, 0xfb, 0xed},

    {   //0x80, 0x00, 0x01, 0xb5, 0x00, 0x20, 0xa8, 0xf0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x74, 0x7c, 0xee, 0x7c,
        0x78, 0xf7, 0x7e, 0x7f, 0xf8, 0xf8, 0x78, 0x7d,
        0xef, 0x77, 0x6d, 0xf2, 0xf0, 0x6b, 0xfe, 0xe5,
        0x70, 0x6c, 0xe9, 0xf6, 0x68, 0x7d, 0xed, 0x78,
        0x76, 0xef, 0xf6, 0x72, 0x7e, 0xf3, 0x77, 0x77,
        0xf4, 0x7e, 0x77, 0xf4, 0x7d, 0x6e, 0xf5, 0xf2,
        0x6e, 0x79, 0xec, 0x7e, 0x6c, 0xf0, 0xed, 0x67,
        0x78, 0xe7, 0x79, 0x6b, 0xee, 0xf3, 0x6d, 0x7d,
        0xf1, 0x7d, 0x73, 0xf8, 0xf6, 0x74, 0x76, 0xf9,
        0x78, 0x77, 0xf2, 0x7a, 0x73, 0xeb, 0xf8, 0x6a,
        0xf7, 0xf2, 0x6a, 0x76, 0xea, 0x7e, 0x6e, 0xf3,
        0xed, 0x6f, 0x71, 0xee, 0xfd, 0x70, 0xf6, 0xf2,
        0x74, 0x7c, 0xf5, 0x76, 0x6f, 0xfa, 0x7f, 0x77,
        0xf6, 0xf1, 0x73, 0x7c, 0xec, 0x74, 0x6d, 0xee,
        0xf7, 0x6a, 0xf8, 0xec, 0x6f, 0x75, 0xed, 0xfc,
        0x6d, 0x7d, 0xef, 0x7a, 0x70, 0xee, 0xf6, 0x6c,
        0xfb, 0xef, 0x6f, 0x7b, 0xed, 0x7b, 0x75, 0xf0,
        0x7d, 0x6e, 0xf3, 0xf2, 0x72, 0x7e, 0xee, 0x7c,
        0x6f, 0xef, 0xf4, 0x6d, 0xfc, 0xea, 0x76, 0x6d,
        0xf3, 0xf6, 0x6d, 0x7c, 0xee, 0x7b, 0x73, 0xee,
        0xf6, 0x6c, 0x7e, 0xf3},

    {   //0x80, 0x00, 0x01, 0xb6, 0x00, 0x20, 0xa9, 0x90,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x70, 0xff, 0xee, 0x71,
        0x75, 0xed, 0x7a, 0x6d, 0xf9, 0xf7, 0x75, 0x79,
        0xee, 0xfb, 0x6d, 0xf6, 0xee, 0x6d, 0x6f, 0xec,
        0x7e, 0x6d, 0xed, 0xee, 0x6e, 0x7f, 0xef, 0x74,
        0x70, 0xf6, 0xfb, 0x76, 0xf7, 0xf4, 0x6e, 0x7c,
        0xee, 0x76, 0x74, 0xf0, 0xf8, 0x70, 0xf9, 0xf1,
        0x72, 0x75, 0xee, 0xf7, 0x6b, 0xfc, 0xea, 0x71,
        0x71, 0xec, 0x7b, 0x6c, 0xf2, 0xf4, 0x6e, 0xfe,
        0xf1, 0x78, 0x79, 0xf7, 0x7d, 0x72, 0xfb, 0xf5,
        0x73, 0x79, 0xf0, 0x7a, 0x6c, 0xef, 0xf0, 0x6c,
        0xfb, 0xe9, 0x76, 0x6e, 0xee, 0xf7, 0x6d, 0xf9,
        0xec, 0x74, 0x73, 0xed, 0xfd, 0x6e, 0xf9, 0xf5,
        0x76, 0xfe, 0xf1, 0x77, 0x78, 0xf3, 0x7c, 0x6f,
        0x7d, 0xf3, 0x75, 0x76, 0xea, 0xff, 0x6b, 0xf1,
        0xf1, 0x6a, 0x78, 0xe9, 0x78, 0x75, 0xeb, 0xf7,
        0x6f, 0x7d, 0xf3, 0x79, 0x73, 0xf6, 0xf6, 0x72,
        0xfd, 0xf5, 0x6e, 0x7a, 0xef, 0x77, 0x72, 0xf0,
        0xf9, 0x6f, 0xf7, 0xee, 0x6f, 0x76, 0xec, 0xfc,
        0x6b, 0xf3, 0xec, 0x6e, 0x79, 0xed, 0x79, 0x6f,
        0xf1, 0xf4, 0x71, 0x78, 0xf6, 0x7e, 0x72, 0xf8,
        0xf8, 0x72, 0xf5, 0xee},

    {   //0x80, 0x00, 0x01, 0xb7, 0x00, 0x20, 0xaa, 0x30,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x6f, 0x76, 0xef, 0x71,
        0x6e, 0xec, 0xfa, 0x6d, 0xf1, 0xed, 0x6f, 0x6f,
        0xf0, 0xfa, 0x6e, 0xfa, 0xed, 0x73, 0x72, 0xef,
        0x7e, 0x6c, 0xfa, 0xf2, 0x74, 0x7f, 0xee, 0x7a,
        0x77, 0xf1, 0x7f, 0x6e, 0xfc, 0xf0, 0x6f, 0x7e,
        0xe9, 0x76, 0x6f, 0xee, 0xf9, 0x6c, 0xfe, 0xee,
        0x7a, 0x73, 0xf2, 0xf6, 0x6e, 0x7c, 0xed, 0x79,
        0x6f, 0xed, 0xf7, 0x6e, 0xf5, 0xf5, 0x6d, 0xfd,
        0xf0, 0x74, 0x76, 0xf1, 0x7f, 0x6f, 0xf5, 0xf5,
        0x70, 0x79, 0xee, 0x7e, 0x6b, 0xf6, 0xf1, 0x6c,
        0x77, 0xee, 0x7a, 0x6e, 0xf0, 0xf1, 0x6f, 0x7b,
        0xef, 0x77, 0x6f, 0xef, 0xfb, 0x6f, 0xf1, 0xf3,
        0x6f, 0xfc, 0xf1, 0x76, 0x79, 0xee, 0xfe, 0x70,
        0xfc, 0xf1, 0x71, 0x6e, 0xef, 0xf4, 0x6e, 0xf5,
        0xea, 0x6e, 0x74, 0xec, 0x7b, 0x6c, 0xf6, 0xf5,
        0x71, 0xfe, 0xf1, 0x76, 0x76, 0xf4, 0x7d, 0x6f,
        0xfa, 0xf2, 0x6f, 0x7c, 0xf0, 0x73, 0x73, 0xf0,
        0xf6, 0x6e, 0xfe, 0xeb, 0x79, 0x71, 0xed, 0xf7,
        0x6b, 0xfb, 0xeb, 0x6f, 0x76, 0xeb, 0xff, 0x72,
        0xf3, 0xfb, 0x71, 0xfa, 0xf3, 0x7a, 0x76, 0xf8,
        0xfd, 0x6f, 0xfd, 0xf1},

    {   //0x80, 0x00, 0x01, 0xb8, 0x00, 0x20, 0xaa, 0xd0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x72, 0x7a, 0xe9, 0x7c,
        0x6a, 0xf0, 0xf3, 0x6c, 0x7d, 0xee, 0x78, 0x6f,
        0xee, 0xf7, 0x6c, 0x7d, 0xef, 0x77, 0x77, 0xee,
        0xfc, 0x75, 0xf4, 0xf8, 0x70, 0x7a, 0xf6, 0x76,
        0x76, 0xf0, 0xfa, 0x6f, 0xfd, 0xee, 0x6e, 0x6f,
        0xeb, 0x7e, 0x6d, 0xee, 0xf2, 0x6d, 0x7e, 0xee,
        0x78, 0x6f, 0xf5, 0xf4, 0x76, 0x7c, 0xf3, 0x78,
        0x71, 0xf5, 0xfc, 0x6f, 0xfa, 0xf0, 0x71, 0x7d,
        0xee, 0x76, 0x73, 0xee, 0xfa, 0x6b, 0xfb, 0xec,
        0x71, 0x72, 0xee, 0xff, 0x6c, 0xf9, 0xf1, 0x6f,
        0x77, 0xf0, 0x7f, 0x71, 0xf9, 0xf4, 0x77, 0x7e,
        0xee, 0x7d, 0x71, 0xf0, 0xfa, 0x6d, 0xf4, 0xef,
        0x6f, 0xfc, 0xec, 0x73, 0x6e, 0xef, 0xfa, 0x6e,
        0xfb, 0xef, 0x79, 0x72, 0xef, 0xf5, 0x6d, 0x7d,
        0xed, 0x79, 0x75, 0xee, 0xfc, 0x72, 0xf6, 0xf8,
        0x6f, 0x7b, 0xef, 0x78, 0x78, 0xee, 0x7f, 0x6e,
        0xf7, 0xf0, 0x6c, 0x79, 0xeb, 0x7c, 0x73, 0xef,
        0xf4, 0x6e, 0x77, 0xef, 0x7a, 0x6d, 0xf8, 0xf0,
        0x6e, 0x79, 0xef, 0x71, 0x6f, 0xf1, 0x7d, 0x6e,
        0xf1, 0xf6, 0x6e, 0xfe, 0xf8, 0x71, 0x75, 0xf2,
        0xfc, 0x6d, 0xfd, 0xef},

    {   //0x80, 0x00, 0x01, 0xb9, 0x00, 0x20, 0xab, 0x70,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x72, 0x73, 0xee, 0xfc,
        0x6d, 0xf4, 0xee, 0x6e, 0x74, 0xee, 0x7e, 0x6e,
        0xf4, 0xf3, 0x72, 0xfa, 0xec, 0x79, 0x76, 0xef,
        0x7d, 0x71, 0xf1, 0xf5, 0x72, 0xf9, 0xef, 0x73,
        0x6f, 0xf4, 0xf5, 0x70, 0xfe, 0xeb, 0x78, 0x6e,
        0xed, 0xf9, 0x68, 0xfa, 0xeb, 0x73, 0x7d, 0xec,
        0x7b, 0x74, 0xf4, 0xf7, 0x73, 0xff, 0xef, 0x7a,
        0x7a, 0xf2, 0xff, 0x6e, 0xfb, 0xf3, 0x6e, 0x7c,
        0xec, 0x7d, 0x6f, 0xf2, 0xf3, 0x6c, 0x78, 0xed,
        0x75, 0x6d, 0xed, 0xf4, 0x6d, 0xfb, 0xf2, 0x6f,
        0x7a, 0xf1, 0x7b, 0x72, 0xf8, 0xfc, 0x71, 0x7c,
        0xf6, 0x7d, 0x78, 0xee, 0xf8, 0x6c, 0xfc, 0xef,
        0x6e, 0x71, 0xed, 0x7a, 0x6c, 0xef, 0xf1, 0x6d,
        0x78, 0xee, 0x7a, 0x6f, 0xf3, 0xf7, 0x71, 0x7d,
        0xf3, 0x73, 0x73, 0xf2, 0xfe, 0x77, 0xf3, 0xf2,
        0x77, 0x7f, 0xf0, 0x76, 0x6c, 0xef, 0xf3, 0x6e,
        0xf5, 0xec, 0x6f, 0x71, 0xec, 0x7c, 0x6c, 0xf6,
        0xf0, 0x72, 0x79, 0xf5, 0x7d, 0x73, 0xfb, 0xf4,
        0x6f, 0x79, 0xee, 0x7b, 0x76, 0xee, 0xfc, 0x6f,
        0xf3, 0xf3, 0x6c, 0x7e, 0xec, 0x79, 0x73, 0xf0,
        0xf7, 0x6d, 0xfc, 0xec},

    {   //0x80, 0x00, 0x01, 0xba, 0x00, 0x20, 0xac, 0x10,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x75, 0x70, 0xef, 0xfa,
        0x6e, 0xfd, 0xf0, 0x78, 0x7a, 0xef, 0xfc, 0x71,
        0xf9, 0xf5, 0x70, 0x7b, 0xef, 0x79, 0x74, 0xed,
        0xfe, 0x6c, 0xf4, 0xf3, 0x6e, 0x7e, 0xee, 0x78,
        0x6f, 0xf5, 0xfa, 0x6d, 0x75, 0xf0, 0x7f, 0x6f,
        0xf2, 0xf1, 0x6f, 0xfe, 0xf0, 0x71, 0x72, 0xf0,
        0x7e, 0x72, 0xf3, 0xf4, 0x73, 0xfd, 0xef, 0x72,
        0x73, 0xef, 0xfa, 0x6f, 0xf6, 0xef, 0x6f, 0x77,
        0xef, 0x7e, 0x6e, 0xf9, 0xee, 0x76, 0x77, 0xef,
        0x7d, 0x6d, 0xf6, 0xf4, 0x6d, 0xfe, 0xed, 0x74,
        0x7a, 0xf1, 0x79, 0x74, 0xf2, 0xf7, 0x71, 0xfe,
        0xf1, 0x7a, 0x72, 0xf5, 0xf4, 0x6e, 0xfc, 0xeb,
        0x72, 0x70, 0xee, 0xf9, 0x6c, 0xfc, 0xef, 0x71,
        0x78, 0xee, 0x7c, 0x6d, 0xf8, 0xf9, 0x6f, 0xfe,
        0xf0, 0x77, 0x7a, 0xee, 0x7a, 0x6e, 0xf5, 0xf3,
        0x76, 0x7e, 0xee, 0xfe, 0x6f, 0xf3, 0xf2, 0x6b,
        0x77, 0xeb, 0x76, 0x6e, 0xed, 0xf8, 0x6e, 0xf7,
        0xf2, 0x6e, 0x76, 0xf2, 0x7d, 0x78, 0xf9, 0xf7,
        0x73, 0x7c, 0xf2, 0x74, 0x75, 0xee, 0xf7, 0x70,
        0xf8, 0xef, 0x6f, 0x76, 0xed, 0x7d, 0x6e, 0xf3,
        0xee, 0x6f, 0x76, 0xee},

    {   //0x80, 0x00, 0x01, 0xbb, 0x00, 0x20, 0xac, 0xb0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x75, 0x6f, 0xf1, 0xf9,
        0x6f, 0xfe, 0xf5, 0x76, 0x7b, 0xf6, 0xfe, 0x76,
        0xf6, 0xf6, 0x71, 0x7a, 0xf0, 0x7a, 0x6f, 0xef,
        0xf6, 0x6b, 0xf4, 0xec, 0x6e, 0x73, 0xee, 0x7d,
        0x6c, 0xf8, 0xf3, 0x73, 0x79, 0xf0, 0x7e, 0x6e,
        0xf7, 0xf6, 0x72, 0xfe, 0xf1, 0x7b, 0x78, 0xf0,
        0x7e, 0x6f, 0xf5, 0xf4, 0x74, 0x7f, 0xeb, 0x7c,
        0x6f, 0xee, 0xf7, 0x6b, 0xfc, 0xeb, 0x76, 0x75,
        0xef, 0xfb, 0x6f, 0xfe, 0xf5, 0x74, 0x75, 0xf0,
        0xf8, 0x71, 0xfa, 0xf5, 0x6e, 0x7a, 0xf0, 0x71,
        0x77, 0xec, 0xfc, 0x70, 0xf4, 0xf5, 0x6d, 0x7b,
        0xee, 0x79, 0x6e, 0xf4, 0xf4, 0x6e, 0x78, 0xee,
        0x78, 0x71, 0xef, 0xf6, 0x6e, 0x7d, 0xf4, 0x73,
        0x71, 0xf6, 0xfb, 0x71, 0xf3, 0xf5, 0x6d, 0x7c,
        0xef, 0x74, 0x73, 0xed, 0xfd, 0x6e, 0xf1, 0xf2,
        0x6f, 0x76, 0xf1, 0xfa, 0x71, 0xf8, 0xef, 0x73,
        0x78, 0xef, 0x78, 0x6c, 0xf6, 0xf6, 0x72, 0xfb,
        0xee, 0x77, 0x7b, 0xf0, 0x78, 0x6e, 0xf6, 0xf4,
        0x73, 0xff, 0xed, 0x76, 0x70, 0xf0, 0xfa, 0x70,
        0xf8, 0xec, 0x7b, 0x76, 0xef, 0xfb, 0x6e, 0xfc,
        0xef, 0x72, 0x75, 0xed},

    {   //0x80, 0x00, 0x01, 0xbc, 0x00, 0x20, 0xad, 0x50,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xfc, 0x70, 0xf4, 0xf9,
        0x6e, 0xfb, 0xf3, 0x6f, 0x79, 0xf3, 0x7c, 0x73,
        0xfb, 0xf6, 0x73, 0x7e, 0xee, 0x7d, 0x6e, 0xf6,
        0xf1, 0x6c, 0x7c, 0xeb, 0x78, 0x73, 0xee, 0xfa,
        0x6d, 0xfb, 0xf2, 0x72, 0x76, 0xf3, 0xff, 0x72,
        0xf6, 0xfc, 0x6d, 0x7e, 0xf4, 0x77, 0x79, 0xef,
        0xfc, 0x72, 0xf9, 0xf4, 0x6f, 0x72, 0xed, 0xfd,
        0x6d, 0xf2, 0xee, 0x6e, 0x7a, 0xed, 0x73, 0x6f,
        0xf0, 0xfb, 0x71, 0xfc, 0xf3, 0x78, 0x7a, 0xf6,
        0x7b, 0x73, 0xf8, 0xf1, 0x75, 0x7d, 0xef, 0x76,
        0x70, 0xf1, 0xfa, 0x6e, 0xf8, 0xeb, 0x74, 0x73,
        0xed, 0xfe, 0x6e, 0xf5, 0xf1, 0x6f, 0x78, 0xef,
        0x7a, 0x71, 0xf4, 0xf9, 0x74, 0xfb, 0xf5, 0x76,
        0x76, 0xf3, 0xfc, 0x6f, 0xfa, 0xf4, 0x6e, 0x7b,
        0xec, 0x78, 0x6c, 0xf0, 0xf8, 0x6b, 0xfd, 0xf0,
        0x76, 0x79, 0xee, 0xfc, 0x6e, 0xfe, 0xf1, 0x79,
        0x7a, 0xee, 0xfb, 0x76, 0xf4, 0xf7, 0x6f, 0x7f,
        0xef, 0x76, 0x77, 0xed, 0xfa, 0x6f, 0xf8, 0xf7,
        0x6d, 0x78, 0xec, 0x7d, 0x6f, 0xf1, 0xf5, 0x6f,
        0x7f, 0xef, 0x78, 0x74, 0xf3, 0xf5, 0x73, 0x7c,
        0xf1, 0x77, 0x75, 0xf2},

    {   //0x80, 0x00, 0x01, 0xbd, 0x00, 0x20, 0xad, 0xf0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x7f, 0x6e, 0xf5, 0xef,
        0x6e, 0x7d, 0xf2, 0x71, 0x78, 0xee, 0xfd, 0x6f,
        0xf6, 0xef, 0x71, 0x75, 0xf1, 0xfe, 0x70, 0xf6,
        0xf1, 0x6f, 0x77, 0xef, 0x7a, 0x6f, 0xf7, 0xf8,
        0x71, 0xfe, 0xf2, 0x77, 0x75, 0xf2, 0x7f, 0x6e,
        0xf3, 0xf5, 0x6f, 0xfc, 0xf0, 0x71, 0x6f, 0xf3,
        0xff, 0x6d, 0xfe, 0xf0, 0x7a, 0x73, 0xf3, 0xfc,
        0x6b, 0x7f, 0xee, 0x75, 0x78, 0xed, 0x7f, 0x71,
        0xf2, 0xfc, 0x6f, 0xfa, 0xef, 0x7a, 0x7d, 0xee,
        0xfe, 0x74, 0xf6, 0xf4, 0x71, 0x7f, 0xeb, 0x7e,
        0x71, 0xf3, 0xf5, 0x6f, 0x7c, 0xee, 0x78, 0x72,
        0xef, 0xf3, 0x71, 0xf9, 0xef, 0x74, 0x78, 0xf0,
        0x7b, 0x73, 0xf1, 0xf7, 0x70, 0xfe, 0xf4, 0x74,
        0x7a, 0xf4, 0xff, 0x6f, 0xfd, 0xef, 0x6f, 0x73,
        0xee, 0x7e, 0x6e, 0xf4, 0xf1, 0x6c, 0x78, 0xee,
        0x76, 0x6f, 0xf4, 0xf7, 0x75, 0xf9, 0xf4, 0x73,
        0x75, 0xf2, 0x7e, 0x73, 0xf3, 0xf7, 0x75, 0xfe,
        0xf3, 0x74, 0x71, 0xef, 0xf6, 0x70, 0xf9, 0xed,
        0x76, 0x72, 0xee, 0x7d, 0x6a, 0xf7, 0xf2, 0x6f,
        0x7b, 0xee, 0x7a, 0x73, 0xf8, 0xfd, 0x73, 0x7d,
        0xf1, 0x7c, 0x77, 0xf4},

    {   //0x80, 0x00, 0x01, 0xbe, 0x00, 0x20, 0xae, 0x90,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xfe, 0x70, 0xf8, 0xf4,
        0x6e, 0x7d, 0xeb, 0x77, 0x6f, 0xf3, 0xfc, 0x6d,
        0xff, 0xef, 0x72, 0x73, 0xed, 0xf9, 0x6f, 0xfa,
        0xf2, 0x76, 0x7d, 0xf1, 0x7e, 0x76, 0xf5, 0xf7,
        0x74, 0x7e, 0xf4, 0x7b, 0x7a, 0xf0, 0xf9, 0x6e,
        0xf6, 0xef, 0x6d, 0x78, 0xee, 0x7c, 0x72, 0xef,
        0xf8, 0x6e, 0x7f, 0xf1, 0x7b, 0x77, 0xf0, 0xf2,
        0x75, 0x7e, 0xf6, 0x70, 0x71, 0xf2, 0xfd, 0x73,
        0xf0, 0xf4, 0x71, 0xff, 0xf5, 0x73, 0x72, 0xee,
        0xfb, 0x6e, 0xf7, 0xef, 0x70, 0x75, 0xef, 0x7a,
        0x71, 0xf3, 0xf2, 0x74, 0x77, 0xed, 0x7a, 0x6f,
        0xf2, 0xf5, 0x6e, 0xfb, 0xea, 0x71, 0x78, 0xed,
        0x7b, 0x6e, 0xef, 0xf5, 0x6b, 0xf8, 0xed, 0x72,
        0x6f, 0xf1, 0xfa, 0x6e, 0xfc, 0xee, 0x71, 0x6e,
        0xef, 0xfa, 0x6a, 0xfd, 0xed, 0x6f, 0x75, 0xee,
        0x7c, 0x6d, 0xf4, 0xf3, 0x6d, 0x7f, 0xef, 0x76,
        0x74, 0xee, 0xfe, 0x6d, 0xf2, 0xf0, 0x6f, 0x7d,
        0xeb, 0x7d, 0x6e, 0xf1, 0xf6, 0x6b, 0x78, 0xec,
        0x79, 0x6e, 0xec, 0xf0, 0x6d, 0xfd, 0xef, 0x6e,
        0x73, 0xed, 0x7e, 0x6f, 0xf1, 0xf6, 0x6f, 0x7d,
        0xf1, 0x75, 0x71, 0xed},

    {   //0x80, 0x00, 0x01, 0xbf, 0x00, 0x20, 0xaf, 0x30,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xf8, 0x6d, 0xfd, 0xef,
        0x6e, 0x6f, 0xee, 0x7d, 0x6c, 0xf3, 0xee, 0x6e,
        0x78, 0xed, 0x7a, 0x6d, 0xf2, 0xf6, 0x6d, 0xfb,
        0xed, 0x72, 0x74, 0xef, 0x7b, 0x71, 0xf4, 0xf2,
        0x73, 0x7f, 0xec, 0x7a, 0x71, 0xef, 0xf2, 0x6d,
        0xfb, 0xea, 0x6e, 0x70, 0xea, 0xfb, 0x6b, 0xf5,
        0xee, 0x70, 0x7b, 0xef, 0x7a, 0x6f, 0xf2, 0xf6,
        0x6f, 0xfe, 0xef, 0x7a, 0x74, 0xf0, 0x7f, 0x6d,
        0xf7, 0xf1, 0x6e, 0x7a, 0xec, 0x7a, 0x6d, 0xf1,
        0xf7, 0x68, 0x7a, 0xeb, 0x75, 0x6e, 0xec, 0xfb,
        0x6b, 0xfc, 0xf4, 0x70, 0x7a, 0xed, 0xfa, 0x75,
        0xf3, 0xf2, 0x73, 0x7c, 0xef, 0x78, 0x72, 0xec,
        0xf7, 0x6d, 0xf6, 0xef, 0x6e, 0x77, 0xec, 0x79,
        0x6d, 0xf0, 0xf1, 0x6d, 0x7a, 0xee, 0x79, 0x6e,
        0xf3, 0xf9, 0x6c, 0xff, 0xf0, 0x72, 0x75, 0xf3,
        0xfe, 0x70, 0xf7, 0xf2, 0x6f, 0x7e, 0xed, 0x74,
        0x6f, 0xee, 0xf7, 0x6d, 0xf5, 0xec, 0x6e, 0x76,
        0xec, 0xfe, 0x6f, 0xf5, 0xf0, 0x72, 0x78, 0xf2,
        0x7a, 0x6d, 0xf5, 0xf0, 0x6f, 0xfd, 0xed, 0x75,
        0x74, 0xef, 0x7b, 0x6b, 0xf5, 0xf2, 0x6e, 0x7e,
        0xec, 0x79, 0x6f, 0xf0},

    {   //0x80, 0x00, 0x01, 0xc0, 0x00, 0x20, 0xaf, 0xd0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xfa, 0x6b, 0xfd, 0xec,
        0x76, 0x72, 0xed, 0xf9, 0x6d, 0xfe, 0xf1, 0x74,
        0x79, 0xea, 0xfa, 0x6f, 0xf4, 0xf1, 0x6f, 0x7c,
        0xee, 0x71, 0x73, 0xec, 0x7c, 0x6d, 0xf6, 0xf3,
        0x71, 0x7b, 0xed, 0x7b, 0x6f, 0xf1, 0xf1, 0x6e,
        0x78, 0xec, 0x7b, 0x6f, 0xee, 0xf4, 0x6d, 0xfb,
        0xef, 0x70, 0x74, 0xef, 0xfe, 0x6f, 0xf5, 0xf6,
        0x6e, 0xfd, 0xee, 0x74, 0x72, 0xef, 0xfa, 0x6e,
        0xf7, 0xf4, 0x6e, 0x73, 0xee, 0xfa, 0x6e, 0xf3,
        0xed, 0x70, 0x77, 0xed, 0x78, 0x6b, 0xf2, 0xf9,
        0x6d, 0xfd, 0xf1, 0x71, 0x74, 0xf3, 0x78, 0x6f,
        0xf5, 0xf5, 0x73, 0x7d, 0xee, 0x7b, 0x6f, 0xf3,
        0xf8, 0x6d, 0xfb, 0xea, 0x75, 0x73, 0xec, 0xfa,
        0x6c, 0xfb, 0xf1, 0x6f, 0x78, 0xec, 0x7c, 0x6e,
        0xf3, 0xf7, 0x72, 0xfe, 0xf2, 0x74, 0x7a, 0xee,
        0x7e, 0x6f, 0xf9, 0xf0, 0x73, 0x7a, 0xed, 0x7c,
        0x6d, 0xf0, 0xf3, 0x6b, 0x7e, 0xec, 0x76, 0x71,
        0xee, 0xfb, 0x6e, 0xf6, 0xef, 0x74, 0x79, 0xf0,
        0xfc, 0x73, 0xf8, 0xf9, 0x6f, 0x7d, 0xef, 0x76,
        0x70, 0xee, 0xf8, 0x6f, 0xf8, 0xf2, 0x6f, 0x75,
        0xed, 0xfc, 0x6e, 0xf0},

    {   //0x80, 0x00, 0x01, 0xc1, 0x00, 0x20, 0xb0, 0x70,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xee, 0x6e, 0x7d, 0xee,
        0x78, 0x72, 0xee, 0xf5, 0x70, 0xfc, 0xf2, 0x75,
        0x73, 0xf6, 0x7e, 0x70, 0xf5, 0xf2, 0x6f, 0x7a,
        0xef, 0x72, 0x6e, 0xf0, 0x7f, 0x6c, 0xf6, 0xee,
        0x71, 0x75, 0xee, 0xfe, 0x6e, 0xf8, 0xf1, 0x6f,
        0x78, 0xee, 0x7c, 0x6e, 0xf7, 0xf5, 0x70, 0xfe,
        0xf0, 0x74, 0x75, 0xf1, 0x7f, 0x6d, 0xf9, 0xef,
        0x6f, 0x7c, 0xed, 0x76, 0x6f, 0xef, 0xf8, 0x6c,
        0xfa, 0xec, 0x76, 0x76, 0xf0, 0xfb, 0x70, 0xf8,
        0xef, 0x73, 0x74, 0xef, 0xfd, 0x6e, 0xf6, 0xf7,
        0x6e, 0x7f, 0xef, 0x73, 0x76, 0xee, 0xfc, 0x6e,
        0xf9, 0xf7, 0x6d, 0x7a, 0xed, 0x7f, 0x70, 0xef,
        0xf2, 0x6f, 0x7b, 0xef, 0x7b, 0x75, 0xed, 0xf3,
        0x72, 0xfe, 0xf1, 0x75, 0x71, 0xf1, 0xfe, 0x71,
        0xf1, 0xf4, 0x6f, 0x7e, 0xf0, 0x74, 0x75, 0xef,
        0xff, 0x71, 0xf8, 0xf0, 0x73, 0x73, 0xef, 0xfe,
        0x6f, 0xf6, 0xf0, 0x70, 0x78, 0xee, 0x77, 0x6e,
        0xf1, 0xf5, 0x70, 0xfd, 0xf1, 0x75, 0x7b, 0xef,
        0xff, 0x71, 0xf5, 0xf3, 0x71, 0x7e, 0xf1, 0x75,
        0x70, 0xf3, 0xf8, 0x6f, 0xfb, 0xed, 0x75, 0x6f,
        0xf3, 0xff, 0x6b, 0xfc},

    {   //0x80, 0x00, 0x01, 0xc2, 0x00, 0x20, 0xb1, 0x10,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xef, 0x71, 0x79, 0xee,
        0x7d, 0x70, 0xf4, 0xfc, 0x6e, 0xfd, 0xf3, 0x73,
        0x77, 0xf2, 0x7e, 0x6f, 0xfc, 0xf5, 0x72, 0x7b,
        0xeb, 0xfe, 0x6f, 0xf1, 0xf5, 0x6d, 0x7c, 0xef,
        0x76, 0x74, 0xed, 0xf6, 0x71, 0xfa, 0xf2, 0x74,
        0x79, 0xf0, 0x7d, 0x74, 0xf3, 0xf8, 0x70, 0x7d,
        0xf1, 0x78, 0x77, 0xf2, 0xfd, 0x6f, 0xfb, 0xef,
        0x6f, 0x73, 0xed, 0xfe, 0x6e, 0xf2, 0xf2, 0x6e,
        0x7e, 0xef, 0x77, 0x71, 0xf4, 0xf8, 0x73, 0xfc,
        0xf6, 0x77, 0x79, 0xf2, 0xfd, 0x70, 0xf4, 0xf0,
        0x6f, 0x7c, 0xf2, 0x72, 0x70, 0xf2, 0xfa, 0x6e,
        0xf9, 0xee, 0x73, 0x74, 0xf1, 0x7f, 0x6f, 0xf2,
        0xef, 0x74, 0x7f, 0xef, 0x7c, 0x70, 0xf9, 0xfb,
        0x73, 0xfb, 0xef, 0x7b, 0x75, 0xef, 0xfd, 0x6f,
        0xf7, 0xf5, 0x6f, 0x7c, 0xec, 0x79, 0x6f, 0xf0,
        0xfa, 0x6f, 0xfb, 0xef, 0x77, 0x78, 0xef, 0xfb,
        0x6f, 0xfe, 0xf2, 0x76, 0x7a, 0xef, 0xff, 0x74,
        0xf5, 0xf7, 0x6e, 0x7f, 0xf1, 0x78, 0x76, 0xf3,
        0xff, 0x6e, 0xf8, 0xf3, 0x6e, 0x79, 0xee, 0x7a,
        0x6f, 0xf8, 0xf9, 0x6f, 0x7f, 0xf2, 0x7c, 0x73,
        0xf1, 0xf3, 0x6a, 0xed},

    {   //0x80, 0x00, 0x01, 0xc3, 0x00, 0x20, 0xb1, 0xb0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xf8, 0x64, 0xfb, 0xed,
        0xfc, 0x6b, 0x7b, 0xef, 0x7e, 0xf7, 0x7c, 0x6a,
        0x7a, 0xe9, 0xf2, 0x6e, 0x7b, 0xf2, 0x7b, 0x7f,
        0xf4, 0x70, 0x6e, 0xec, 0xec, 0x73, 0x71, 0xf7,
        0xfd, 0x7b, 0xef, 0x77, 0x6b, 0xf2, 0xec, 0x79,
        0x70, 0xff, 0xff, 0xfe, 0xf2, 0xff, 0x6a, 0xfd,
        0xeb, 0x7d, 0x73, 0xfe, 0xff, 0x79, 0xf3, 0xf4,
        0x6c, 0x73, 0xea, 0xf0, 0x75, 0x7a, 0x7f, 0x79,
        0xf8, 0xed, 0x76, 0x6b, 0xf3, 0xee, 0x77, 0x77,
        0xfe, 0x78, 0xfb, 0xea, 0xfd, 0x69, 0xf8, 0xed,
        0x75, 0xfc, 0xf6, 0x77, 0xfd, 0xed, 0xfb, 0x69,
        0x72, 0xee, 0xf9, 0x7a, 0xfc, 0x79, 0x71, 0xfa,
        0xf1, 0x6e, 0x6b, 0xee, 0xf6, 0x77, 0xf7, 0x75,
        0x6e, 0xf3, 0xed, 0x73, 0x70, 0xee, 0xf2, 0x79,
        0xfc, 0x78, 0x6e, 0xfd, 0xec, 0xff, 0x6b, 0xfd,
        0xee, 0x7a, 0x79, 0xfa, 0x76, 0x7a, 0xec, 0xf8,
        0x68, 0x77, 0xed, 0x7d, 0x7e, 0xf9, 0x6f, 0x7b,
        0xed, 0xfa, 0x6d, 0x75, 0xf0, 0xf9, 0x7d, 0x7d,
        0x74, 0x73, 0xf6, 0xec, 0x74, 0x6c, 0xee, 0xf3,
        0x6f, 0x7e, 0x7e, 0x71, 0xf4, 0xea, 0x77, 0x69,
        0xfb, 0xf3, 0x77, 0xfa},

    {   //0x80, 0x00, 0x01, 0xc4, 0x00, 0x20, 0xb2, 0x50,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xff, 0x74, 0xf6, 0xef,
        0x79, 0x69, 0x7c, 0xee, 0xf9, 0xfb, 0xfc, 0x76,
        0x79, 0xf1, 0xf4, 0x6a, 0x78, 0xe7, 0xf8, 0x7a,
        0xf6, 0x77, 0x74, 0xee, 0xef, 0x6f, 0x77, 0xee,
        0xf9, 0x7c, 0x7d, 0x70, 0x79, 0xf5, 0xf3, 0x75,
        0x6c, 0xf4, 0xf0, 0x79, 0x7e, 0x7f, 0x75, 0xfe,
        0xec, 0x79, 0x68, 0xf5, 0xed, 0x78, 0x7d, 0x7d,
        0x73, 0xfe, 0xee, 0xfd, 0x6c, 0xfc, 0xee, 0xfe,
        0x79, 0x75, 0x74, 0xfd, 0xf1, 0xf3, 0x6f, 0x71,
        0xed, 0xff, 0x73, 0xfc, 0x7c, 0x76, 0xef, 0xee,
        0x68, 0x6c, 0xee, 0xfb, 0x78, 0xf8, 0xff, 0x7c,
        0xf5, 0xf3, 0x73, 0x6c, 0xf7, 0xed, 0xfd, 0x7d,
        0x7d, 0x7a, 0x7e, 0xf1, 0x7c, 0x6b, 0xf6, 0xeb,
        0x7d, 0x7a, 0x7e, 0x73, 0x7b, 0xec, 0xf9, 0x6b,
        0xfb, 0xeb, 0xff, 0x7a, 0x7d, 0x78, 0xfe, 0xf0,
        0xf7, 0x6d, 0x6f, 0xf1, 0xf6, 0x75, 0x79, 0x7f,
        0x7a, 0xfc, 0xf2, 0x6d, 0x6d, 0xea, 0xf2, 0x77,
        0xfc, 0x79, 0x73, 0xf6, 0xee, 0x73, 0x6f, 0xef,
        0xf1, 0x7a, 0x73, 0x78, 0x7b, 0xfe, 0xef, 0xfb,
        0x6b, 0x7c, 0xed, 0x7a, 0x74, 0xfe, 0xfe, 0xfb,
        0xee, 0xfc, 0x68, 0x7a},

    {   //0x80, 0x00, 0x01, 0xc5, 0x00, 0x20, 0xb2, 0xf0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xec, 0x7d, 0x7d, 0xf9,
        0x7b, 0x7d, 0xf0, 0xfc, 0x6a, 0x79, 0xed, 0xf3,
        0x7b, 0x78, 0xfe, 0x7d, 0xfe, 0xf2, 0x73, 0x6c,
        0xed, 0xee, 0x74, 0x78, 0xfe, 0x7a, 0xf8, 0xed,
        0x73, 0x6b, 0xf2, 0xf2, 0x7b, 0x7d, 0x7c, 0x7b,
        0xfd, 0xf0, 0x7a, 0x6b, 0xfb, 0xec, 0xf9, 0x78,
        0xfd, 0xfa, 0x7b, 0xf6, 0xf7, 0x6d, 0x7b, 0xe9,
        0xf8, 0x73, 0x7b, 0x7b, 0x7b, 0xf2, 0xf5, 0x6c,
        0x79, 0xee, 0x7e, 0x76, 0x74, 0x79, 0xfd, 0xf9,
        0xf2, 0x72, 0x6c, 0xf6, 0xf0, 0x74, 0x74, 0xf5,
        0x7f, 0xff, 0xef, 0x76, 0x69, 0xf4, 0xec, 0x7b,
        0x7b, 0xfd, 0xfe, 0xfe, 0xf9, 0x74, 0x6b, 0xfa,
        0xee, 0xfe, 0x77, 0x79, 0x7f, 0x7c, 0xf7, 0xf8,
        0x6d, 0x7a, 0xec, 0xfb, 0x6e, 0x79, 0xf7, 0x7c,
        0xf5, 0xef, 0x6d, 0x70, 0xed, 0xf6, 0x77, 0x7d,
        0xfe, 0xfe, 0xfa, 0xfd, 0x71, 0x71, 0xf0, 0xef,
        0xfd, 0x76, 0xfe, 0xfc, 0x79, 0xf3, 0x7d, 0x6d,
        0xf0, 0xe8, 0x7b, 0x6f, 0xff, 0x7e, 0x7d, 0xee,
        0xfb, 0x6e, 0xfb, 0xed, 0xfe, 0x73, 0x7a, 0xfa,
        0xf8, 0xf5, 0xfc, 0x6d, 0x77, 0xf2, 0xfc, 0x73,
        0x76, 0xf6, 0xff, 0x7e},

    {   //0x80, 0x00, 0x01, 0xc6, 0x00, 0x20, 0xb3, 0x90,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xf6, 0x6c, 0x6e, 0xec,
        0xf0, 0x74, 0x79, 0xf9, 0x7c, 0xfa, 0xf2, 0x71,
        0x72, 0xf0, 0xef, 0x7c, 0x70, 0x7c, 0xf6, 0xfe,
        0xfa, 0x7c, 0x6d, 0xfd, 0xee, 0x7d, 0x6f, 0x7d,
        0xf8, 0x7d, 0xf2, 0xfc, 0x6b, 0xfb, 0xea, 0xf9,
        0x74, 0xfe, 0xfc, 0x79, 0xfb, 0x7f, 0x6e, 0x7b,
        0xed, 0xf0, 0x77, 0x73, 0xfc, 0xfc, 0x7d, 0xfe,
        0x74, 0x74, 0xef, 0xf1, 0x70, 0x70, 0xf9, 0x7d,
        0xfd, 0xed, 0x72, 0x6f, 0xef, 0xf3, 0x72, 0x70,
        0xfc, 0xf8, 0xfa, 0xf4, 0x7a, 0x6e, 0xfd, 0xef,
        0xfc, 0x6f, 0x7c, 0xf4, 0x7e, 0xfc, 0xfe, 0x6d,
        0x7e, 0xeb, 0xf7, 0x71, 0x7a, 0xf9, 0x79, 0xf9,
        0xfd, 0x6a, 0x79, 0xed, 0xfc, 0x6f, 0x72, 0xfa,
        0xfa, 0xfb, 0xf8, 0x76, 0x75, 0xf6, 0xee, 0x79,
        0x72, 0xf1, 0xf2, 0x7e, 0xf9, 0x75, 0x6e, 0xef,
        0xed, 0x7e, 0x77, 0xfb, 0xfe, 0x79, 0xfa, 0x73,
        0x6f, 0xf3, 0xed, 0xfc, 0x6f, 0x7a, 0xf8, 0x7d,
        0xfa, 0xfd, 0x74, 0xfe, 0xed, 0xf5, 0x6d, 0x72,
        0xf0, 0xf8, 0xf9, 0xf3, 0x70, 0x78, 0xed, 0xf8,
        0x71, 0x79, 0xf6, 0xfb, 0xf9, 0xf9, 0x6f, 0x77,
        0xf1, 0xf1, 0x7b, 0x70},

    {   //0x80, 0x00, 0x01, 0xc7, 0x00, 0x20, 0xb4, 0x30,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xf9, 0xf7, 0x79, 0xfe,
        0x7a, 0x70, 0xf7, 0xeb, 0x7a, 0x6c, 0xfd, 0xf8,
        0xff, 0xf8, 0x7a, 0x6f, 0xf9, 0xf1, 0x7b, 0x6f,
        0x7b, 0xf6, 0xfb, 0xf9, 0x79, 0x6d, 0x7b, 0xf5,
        0xfb, 0x70, 0x75, 0xf2, 0xfd, 0x77, 0x7c, 0x70,
        0x72, 0xf1, 0xf0, 0x72, 0x74, 0xf5, 0x7f, 0x7a,
        0xfc, 0x72, 0x79, 0xed, 0xf1, 0x7a, 0x70, 0x7e,
        0xf8, 0xfe, 0xfb, 0x78, 0x75, 0xfb, 0xef, 0x7b,
        0x6b, 0xfd, 0xf1, 0xfe, 0xf8, 0xfd, 0x71, 0xfd,
        0xec, 0xfc, 0x6e, 0x7f, 0xf1, 0xfd, 0xfd, 0x7d,
        0x72, 0xf7, 0xee, 0xf7, 0x78, 0x77, 0xf7, 0xfd,
        0x7e, 0xfe, 0x79, 0x7e, 0xef, 0xee, 0x71, 0x6f,
        0xee, 0xf7, 0x7c, 0xf7, 0x77, 0x73, 0xf4, 0xf3,
        0x73, 0x6f, 0xf8, 0xf2, 0xfa, 0xff, 0x72, 0x75,
        0xfa, 0xf3, 0xfd, 0x72, 0xfc, 0xf0, 0x7d, 0x7c,
        0x79, 0x73, 0xf9, 0xec, 0xf6, 0x6e, 0x7c, 0xf3,
        0x78, 0x7c, 0xfe, 0x71, 0x7e, 0xed, 0xfa, 0x6e,
        0x73, 0xf8, 0xfb, 0xfe, 0x7c, 0x75, 0xfe, 0xf4,
        0xf4, 0x77, 0x6f, 0xf2, 0xf2, 0x7e, 0xfd, 0x76,
        0x73, 0xf5, 0xf0, 0x72, 0x6c, 0xf9, 0xfa, 0x77,
        0x7e, 0x74, 0x75, 0xf7},

    {   //0x80, 0x00, 0x01, 0xc8, 0x00, 0x20, 0xb4, 0xd0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xf2, 0x7b, 0x6d, 0x7c,
        0xf3, 0xfe, 0x7c, 0x7b, 0x7a, 0xf8, 0xf1, 0xf9,
        0x6e, 0x75, 0xee, 0xf9, 0xfd, 0xfd, 0x74, 0x7a,
        0xf2, 0xf9, 0x6d, 0x7a, 0xee, 0xfa, 0x7f, 0x7a,
        0x70, 0x78, 0xf5, 0xf4, 0x78, 0x73, 0xf5, 0xf2,
        0x79, 0x77, 0x78, 0x7b, 0xf5, 0xee, 0x7b, 0x6d,
        0xfa, 0xf4, 0x7a, 0xfe, 0x7c, 0x79, 0xf1, 0xee,
        0x7e, 0x6d, 0xfa, 0xef, 0xfb, 0xfb, 0x7b, 0x78,
        0xfd, 0xf1, 0xf9, 0x6f, 0x79, 0xef, 0xf8, 0x78,
        0x79, 0x7a, 0x7d, 0xf4, 0xf3, 0x70, 0x74, 0xf1,
        0xfe, 0x7a, 0xff, 0x74, 0xfe, 0xed, 0xf2, 0x70,
        0x6f, 0xf8, 0xf9, 0x7e, 0x7c, 0x7a, 0xff, 0xfb,
        0xf6, 0x77, 0x6a, 0xfa, 0xee, 0x7d, 0x7a, 0x7b,
        0x75, 0x7e, 0xf4, 0x7f, 0x6f, 0xf9, 0xee, 0x7e,
        0x7b, 0x73, 0x72, 0xfc, 0xf2, 0xfd, 0x6f, 0x7a,
        0xf3, 0xfc, 0x7a, 0x79, 0x7c, 0xfb, 0xf2, 0xf1,
        0x70, 0x6f, 0xf0, 0xf6, 0x75, 0x7a, 0x7c, 0x7c,
        0xf2, 0xf1, 0x73, 0x73, 0xf3, 0xf8, 0x7f, 0x7a,
        0x73, 0x7d, 0xf7, 0xf4, 0x7a, 0x70, 0xf8, 0xef,
        0xfe, 0x77, 0x7b, 0xff, 0xfa, 0xef, 0xfc, 0x6d,
        0x7b, 0xf2, 0x7c, 0x74},

    {   //0x80, 0x00, 0x01, 0xc9, 0x00, 0x20, 0xb5, 0x70,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x7a, 0x7b, 0xfa, 0xf0,
        0xfc, 0x6e, 0x79, 0xf3, 0xfc, 0xff, 0x7a, 0x7d,
        0xf7, 0xf6, 0xf8, 0x72, 0x74, 0xee, 0xee, 0x7b,
        0x78, 0x7c, 0x78, 0xfc, 0xf2, 0x77, 0x70, 0xef,
        0xf1, 0x7b, 0x7a, 0x77, 0x7e, 0xf8, 0xf4, 0x7c,
        0x70, 0xfb, 0xf6, 0x7c, 0x74, 0x77, 0xfa, 0xf5,
        0xf2, 0xfc, 0x6e, 0x7e, 0xee, 0xfb, 0x76, 0x7c,
        0x7d, 0x7a, 0xf5, 0xfa, 0x6c, 0x79, 0xf0, 0xfe,
        0x77, 0x74, 0x75, 0x7e, 0xf6, 0xfa, 0x74, 0x75,
        0xf4, 0xf6, 0x79, 0x70, 0x78, 0xf8, 0xf6, 0xf1,
        0x76, 0x6d, 0xf7, 0xf5, 0x7a, 0x7a, 0xfe, 0xfb,
        0xf3, 0xef, 0x78, 0x6f, 0xf6, 0xee, 0xf6, 0x7d,
        0x7b, 0xfd, 0xfb, 0xf9, 0x7e, 0x70, 0x7f, 0xf0,
        0xf9, 0x76, 0x72, 0x7d, 0x7e, 0xf8, 0xf8, 0x6e,
        0x79, 0xee, 0xfd, 0x74, 0x79, 0x7a, 0x7e, 0xf4,
        0xf7, 0x6e, 0x74, 0xf3, 0xf8, 0x7b, 0x74, 0x7a,
        0xfa, 0xfc, 0xfc, 0x78, 0x6f, 0xf8, 0xef, 0xfe,
        0x76, 0x7c, 0xfa, 0xf8, 0xf1, 0x7c, 0x6e, 0xf6,
        0xf1, 0x7b, 0x76, 0x77, 0x7d, 0xfb, 0xf1, 0xfe,
        0x70, 0xfe, 0xf2, 0xfd, 0x71, 0x72, 0xfe, 0xf7,
        0xf6, 0xf8, 0x71, 0x72},

    {   //0x80, 0x00, 0x01, 0xca, 0x00, 0x20, 0xb6, 0x10,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xf2, 0xf8, 0x74, 0x75,
        0xfb, 0xfc, 0xf9, 0xf8, 0x6e, 0x74, 0xef, 0xf6,
        0x7c, 0x7a, 0x7c, 0xfc, 0xfa, 0xf6, 0x79, 0x75,
        0xf4, 0xee, 0xfc, 0x70, 0x78, 0xfb, 0xfe, 0xf7,
        0xfe, 0x70, 0xf8, 0xee, 0x7d, 0x73, 0x79, 0xfb,
        0xf9, 0xf6, 0x7a, 0x6d, 0xfe, 0xf0, 0xfb, 0x7b,
        0x77, 0xfd, 0xf6, 0xf5, 0xfa, 0x73, 0x7b, 0xf0,
        0xf3, 0x75, 0x70, 0xfd, 0x7f, 0xfc, 0xf2, 0x78,
        0x76, 0xef, 0xf7, 0x72, 0x6f, 0x77, 0xfc, 0xf4,
        0xf5, 0x76, 0x74, 0xf8, 0xf8, 0x7c, 0x71, 0x7b,
        0xf5, 0xf9, 0xf7, 0x78, 0x6e, 0x7d, 0xf1, 0xfd,
        0x70, 0x7a, 0xfa, 0xfe, 0xf9, 0xfe, 0x6e, 0xff,
        0xed, 0xfa, 0x78, 0x79, 0xfe, 0xf8, 0xf6, 0xfd,
        0x71, 0x7d, 0xf4, 0xfa, 0x77, 0x6f, 0xfe, 0xf7,
        0xfa, 0xf7, 0x75, 0x71, 0xf4, 0xf0, 0x77, 0x71,
        0xfa, 0xfa, 0xfc, 0xf8, 0x75, 0x71, 0xf4, 0xf2,
        0xfe, 0x78, 0x77, 0xf8, 0xf7, 0xf9, 0x7a, 0x74,
        0xf9, 0xf2, 0xfb, 0x6f, 0x70, 0xfe, 0xfe, 0xf8,
        0xfa, 0x75, 0xfe, 0xed, 0xf9, 0x71, 0x78, 0xf7,
        0xf7, 0xf3, 0xf7, 0x73, 0x7c, 0xf1, 0xf9, 0x7b,
        0x74, 0xfd, 0xf3, 0xfb},

    {   //0x80, 0x00, 0x01, 0xcb, 0x00, 0x20, 0xb6, 0xb0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x7d, 0x71, 0x70, 0xfc,
        0xf2, 0x7b, 0x70, 0xfd, 0xf8, 0x7f, 0xf7, 0x76,
        0x6f, 0xf6, 0xf1, 0x7b, 0x6f, 0x7a, 0xfa, 0xfb,
        0xf8, 0x7c, 0x75, 0xfa, 0xf2, 0xfd, 0x71, 0x71,
        0xf8, 0xf6, 0xfb, 0x7e, 0x70, 0x7b, 0xf0, 0xf5,
        0x77, 0x77, 0xf5, 0xfc, 0xfd, 0xfb, 0x6f, 0x7a,
        0xee, 0xf6, 0x75, 0x73, 0x7d, 0xfc, 0xfc, 0xff,
        0x78, 0x7c, 0xf5, 0xf1, 0x7f, 0x6e, 0x79, 0xf5,
        0xfa, 0xf8, 0x7e, 0x75, 0xf7, 0xf5, 0x76, 0x6d,
        0x7d, 0xf6, 0xfa, 0xf2, 0x78, 0x6d, 0x7e, 0xf5,
        0xfe, 0x71, 0x7a, 0xf5, 0xf7, 0xfb, 0x7c, 0x74,
        0x7b, 0xf4, 0xf4, 0x78, 0x73, 0xf5, 0xf6, 0xff,
        0xfb, 0x76, 0x7d, 0xef, 0xf5, 0x73, 0x73, 0xfb,
        0xfb, 0xf8, 0xf8, 0x74, 0x77, 0xf7, 0xf7, 0x79,
        0x6f, 0x7d, 0xf4, 0xfd, 0x7d, 0x79, 0x73, 0xfd,
        0xee, 0xfa, 0x70, 0x7d, 0xf2, 0x7f, 0xfe, 0x7b,
        0x71, 0xf9, 0xef, 0xfb, 0x72, 0x79, 0xf8, 0xfc,
        0xf9, 0x7e, 0x76, 0xfa, 0xf1, 0xfb, 0x6f, 0x6e,
        0xfa, 0xf5, 0xfa, 0xfc, 0x79, 0x76, 0xf9, 0xf7,
        0x72, 0x6f, 0xf6, 0xf5, 0xfa, 0xf9, 0x74, 0x78,
        0xf3, 0xf6, 0x7a, 0x73},

    {   //0x80, 0x00, 0x01, 0xcc, 0x00, 0x20, 0xb7, 0x50,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xfd, 0xf5, 0xfa, 0x7f,
        0x74, 0x78, 0xfc, 0xf3, 0xfd, 0x6d, 0x79, 0xf6,
        0xfe, 0xff, 0x7e, 0x77, 0xfe, 0xf0, 0xfb, 0x6f,
        0x78, 0xf2, 0xf5, 0xf7, 0x7e, 0x72, 0xfd, 0xf4,
        0xfe, 0x77, 0x77, 0xf7, 0xf5, 0xfd, 0x7a, 0x74,
        0x7c, 0xf7, 0xf0, 0x7a, 0x6e, 0xf5, 0xf5, 0x7c,
        0xfe, 0x78, 0x79, 0xf3, 0xef, 0x7a, 0x6f, 0x7c,
        0xf9, 0xfc, 0xfc, 0x76, 0x79, 0xf5, 0xf7, 0x7b,
        0x6e, 0x77, 0xf6, 0xf9, 0xfb, 0x7c, 0x76, 0x7f,
        0xf3, 0xf9, 0x6f, 0x77, 0xf0, 0xfa, 0x7e, 0x7e,
        0x74, 0x7d, 0xf2, 0xf8, 0x74, 0x74, 0xfb, 0xf8,
        0xfb, 0x78, 0x72, 0xfd, 0xf6, 0xf7, 0x7b, 0x6f,
        0xfc, 0xf1, 0xfd, 0xfe, 0x7b, 0x76, 0xfc, 0xf4,
        0x77, 0x6d, 0xfa, 0xf2, 0xfb, 0xfc, 0x78, 0x75,
        0xfa, 0xf4, 0x7f, 0x72, 0x7d, 0xf1, 0xf7, 0x7d,
        0x75, 0x77, 0xfb, 0xf2, 0xf6, 0x77, 0x76, 0xf5,
        0xfb, 0x7a, 0x7b, 0x78, 0xfd, 0xef, 0xf3, 0x6f,
        0x73, 0xf4, 0xf7, 0xfa, 0xfc, 0x7a, 0xfe, 0xf6,
        0xfc, 0x75, 0x6f, 0xfc, 0xef, 0xf8, 0x7e, 0x7c,
        0x7d, 0xfa, 0xf3, 0x7f, 0x6e, 0xfe, 0xf2, 0x7d,
        0xff, 0x7b, 0x77, 0xf8},

    {   //0x80, 0x00, 0x01, 0xcd, 0x00, 0x20, 0xb7, 0xf0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xef, 0xfe, 0x6e, 0x7b,
        0xf6, 0xfe, 0x7d, 0x79, 0x79, 0xfd, 0xf9, 0x7e,
        0x6f, 0x6f, 0xf8, 0xf5, 0x7e, 0x7a, 0x77, 0x7c,
        0xfb, 0xfb, 0x6f, 0x70, 0xf1, 0xf7, 0x7f, 0x7d,
        0x73, 0x7d, 0xf2, 0xf4, 0x78, 0x72, 0xf8, 0xf5,
        0xfd, 0x77, 0x76, 0xfe, 0xf7, 0xf1, 0xfd, 0x6e,
        0x7b, 0xf1, 0xff, 0x7b, 0x7b, 0xfe, 0xf9, 0xf5,
        0xff, 0x6d, 0x7b, 0xf3, 0xf9, 0xfc, 0x7c, 0x79,
        0xf8, 0xf3, 0xfc, 0x70, 0x78, 0xf2, 0xf5, 0xff,
        0x78, 0x7b, 0xfc, 0xf9, 0xf4, 0x78, 0x70, 0xf4,
        0xf4, 0x7a, 0x7a, 0x7a, 0xff, 0xf2, 0xf2, 0x75,
        0x6f, 0xfa, 0xf8, 0xf9, 0xfe, 0x77, 0xfd, 0xf8,
        0xf8, 0x78, 0x6e, 0x7e, 0xef, 0xf5, 0x7d, 0x7d,
        0x7d, 0x7e, 0xf9, 0x7e, 0x6e, 0x7c, 0xef, 0xfb,
        0x77, 0x76, 0x75, 0xff, 0xf4, 0xfb, 0x74, 0x7b,
        0xf7, 0xff, 0x7c, 0x73, 0x76, 0xf9, 0xf4, 0xf4,
        0x76, 0x6f, 0xf9, 0xf5, 0x7b, 0x77, 0x7e, 0x7d,
        0xfd, 0xf6, 0x74, 0x6d, 0xf8, 0xf1, 0xfd, 0x7e,
        0x78, 0xfe, 0xf6, 0xf7, 0x7b, 0x71, 0xfe, 0xf5,
        0xf8, 0x7c, 0x76, 0xfe, 0xf8, 0xf3, 0xf8, 0x6f,
        0x79, 0xf2, 0xfa, 0x7a},

    {   //0x80, 0x00, 0x01, 0xce, 0x00, 0x20, 0xb8, 0x90,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x7a, 0xfd, 0xfd, 0xf6,
        0xf9, 0x6d, 0x73, 0xf2, 0xf6, 0xfc, 0x7c, 0x7b,
        0xfb, 0xf8, 0xfe, 0x73, 0x74, 0xf5, 0xf0, 0xfb,
        0x79, 0x7a, 0xfd, 0xfe, 0xf7, 0x79, 0x6f, 0xf6,
        0xf0, 0xff, 0x77, 0x7b, 0x7b, 0xfa, 0xf1, 0x7c,
        0x71, 0xfe, 0xf5, 0xfd, 0x79, 0x76, 0xff, 0xfa,
        0xfa, 0x7d, 0x6e, 0x77, 0xf6, 0xf8, 0x7a, 0x75,
        0x7e, 0xfe, 0xf9, 0xf9, 0x6f, 0x75, 0xf0, 0xf6,
        0x7b, 0x79, 0x7b, 0xfc, 0xf3, 0xf5, 0x76, 0x78,
        0xf5, 0xf3, 0xfe, 0x77, 0x79, 0xf9, 0xf8, 0xf8,
        0x7c, 0x6e, 0x7e, 0xf5, 0xff, 0x77, 0x7b, 0xfb,
        0xf9, 0xf2, 0x7b, 0x6d, 0xff, 0xf3, 0xf9, 0x7d,
        0x7b, 0xfe, 0xfb, 0xf9, 0x7c, 0x72, 0x7c, 0xf1,
        0xf1, 0x7d, 0x72, 0x7d, 0xfc, 0xfe, 0xf9, 0x78,
        0x76, 0xf3, 0xf5, 0x78, 0x76, 0x7d, 0xfd, 0xf6,
        0xf5, 0x72, 0x73, 0xf5, 0xf8, 0x7e, 0x76, 0x7a,
        0xf9, 0xf8, 0xfa, 0x79, 0x72, 0xfe, 0xef, 0xfb,
        0x72, 0x77, 0xfc, 0xfd, 0xfb, 0x7e, 0x6f, 0xfc,
        0xee, 0xfd, 0x75, 0x75, 0x7b, 0xfd, 0xf7, 0x7f,
        0x6f, 0x7b, 0xf6, 0xfd, 0x7a, 0x77, 0xfd, 0xf5,
        0xf6, 0xf9, 0x75, 0x74},

    {   //0x80, 0x00, 0x01, 0xcf, 0x00, 0x20, 0xb9, 0x30,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xf7, 0xf3, 0x7c, 0x76,
        0xf9, 0xf6, 0xfb, 0xf7, 0x74, 0x72, 0xf2, 0xf1,
        0xfe, 0x77, 0x78, 0x7e, 0xfb, 0xfc, 0x76, 0x76,
        0xf8, 0xf0, 0xf8, 0x77, 0x78, 0xfa, 0xfe, 0xfa,
        0x7d, 0x70, 0x7b, 0xf2, 0xf9, 0x71, 0x78, 0xfb,
        0xfc, 0xfa, 0x7c, 0x6e, 0x7a, 0xf2, 0xf8, 0x7b,
        0x78, 0xff, 0xf7, 0xf7, 0x7f, 0x75, 0x7b, 0xf5,
        0xf3, 0xff, 0x71, 0x7e, 0xf9, 0xfe, 0xf9, 0x79,
        0x76, 0xf4, 0xf1, 0x79, 0x71, 0x7b, 0xfa, 0xf8,
        0xf8, 0x79, 0x71, 0xfe, 0xfa, 0xff, 0x76, 0x77,
        0xf5, 0xf3, 0xf8, 0x7b, 0x70, 0x7c, 0xf5, 0xf7,
        0x78, 0x7b, 0xf3, 0xfa, 0xfc, 0xfd, 0x70, 0x78,
        0xef, 0xf3, 0x7a, 0x76, 0x7f, 0xfd, 0xfc, 0x7d,
        0x72, 0x7c, 0xf3, 0xf8, 0x7d, 0x73, 0x7b, 0xf5,
        0xfc, 0xfd, 0x76, 0x71, 0xfc, 0xf3, 0x7e, 0x74,
        0xfb, 0xf8, 0xfa, 0xf5, 0x7a, 0x71, 0xf8, 0xf0,
        0xfb, 0x76, 0x77, 0xfa, 0xfb, 0xff, 0x79, 0x77,
        0xfc, 0xf4, 0xf8, 0x75, 0x6f, 0xfa, 0xfa, 0xfe,
        0xfe, 0x73, 0x7b, 0xf3, 0xf8, 0x73, 0x72, 0xfb,
        0xfb, 0xfb, 0xfb, 0x73, 0x77, 0xf5, 0xf6, 0x7b,
        0x77, 0xfb, 0xf6, 0xf5},

    {   //0x80, 0x00, 0x01, 0xd0, 0x00, 0x20, 0xb9, 0xd0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x7f, 0x74, 0x73, 0xfe,
        0xf0, 0xfb, 0x6f, 0x7a, 0xf2, 0xfa, 0xfe, 0x74,
        0x6f, 0xf7, 0xee, 0x7f, 0x6e, 0x77, 0xf8, 0xfd,
        0xfd, 0x76, 0x71, 0xfa, 0xf3, 0xf7, 0x73, 0x74,
        0xf5, 0xf2, 0xfb, 0x7e, 0x7a, 0x7e, 0xf2, 0xf0,
        0x7d, 0x71, 0xf7, 0xf2, 0xfb, 0xfc, 0x6f, 0x73,
        0xf3, 0xf2, 0x7c, 0x74, 0x7d, 0xfb, 0xf9, 0xfd,
        0x73, 0x77, 0xf6, 0xf1, 0xfc, 0x6e, 0x72, 0xf4,
        0xf5, 0xfb, 0x7e, 0x77, 0x7e, 0xf0, 0xfb, 0x70,
        0x77, 0xf8, 0xf8, 0xf5, 0x7b, 0x6d, 0x7d, 0xf6,
        0xfb, 0x75, 0x76, 0xfb, 0xfa, 0xfa, 0x78, 0x6e,
        0x77, 0xf6, 0xf2, 0x7d, 0x71, 0xfe, 0xf4, 0xfc,
        0xfb, 0x78, 0x78, 0xf5, 0xf0, 0x7e, 0x6f, 0x7d,
        0xf9, 0xf8, 0xf6, 0x74, 0x74, 0xfa, 0xf7, 0xfb,
        0x72, 0x79, 0xf5, 0xf7, 0xfb, 0x78, 0x71, 0x7a,
        0xf2, 0xf4, 0x71, 0x76, 0xf6, 0xfd, 0xff, 0x7a,
        0x70, 0x7c, 0xf1, 0xf4, 0x78, 0x72, 0xfc, 0xf9,
        0xf6, 0x7f, 0x72, 0x7e, 0xf6, 0xf4, 0x7b, 0x6e,
        0x7d, 0xf7, 0xfa, 0xfc, 0x7b, 0x76, 0xfd, 0xf1,
        0x7d, 0x6e, 0xfc, 0xf1, 0xf9, 0xfb, 0x77, 0x71,
        0xfb, 0xf1, 0xf8, 0x79},

    {   //0x80, 0x00, 0x01, 0xd1, 0x00, 0x20, 0xba, 0x70,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x7b, 0xf6, 0xf5, 0xfb,
        0x77, 0x76, 0xfc, 0xf4, 0xf4, 0x75, 0x75, 0xf5,
        0xf8, 0xfb, 0x7e, 0x77, 0x7b, 0xf4, 0xf4, 0x70,
        0x71, 0xf7, 0xf8, 0xfd, 0xfe, 0x73, 0x77, 0xf7,
        0xf8, 0x7c, 0x74, 0xfd, 0xf4, 0xfa, 0x7a, 0x70,
        0x79, 0xfb, 0xf2, 0xfc, 0x6f, 0x7f, 0xf5, 0xff,
        0xfe, 0x7a, 0x71, 0xfa, 0xed, 0xfa, 0x6f, 0x76,
        0xfc, 0xfe, 0xfc, 0x79, 0x77, 0xfb, 0xf7, 0xfa,
        0x71, 0x70, 0xfc, 0xf6, 0xfb, 0x7b, 0x77, 0x7c,
        0xf9, 0xf5, 0x77, 0x72, 0xf5, 0xf8, 0xfe, 0x7d,
        0x72, 0x76, 0xf5, 0xef, 0x7b, 0x76, 0xf9, 0xf8,
        0xf9, 0x7d, 0x74, 0x7c, 0xfa, 0xf4, 0xfc, 0x6f,
        0x7a, 0xf2, 0xf7, 0xfc, 0x7a, 0x79, 0xfc, 0xf4,
        0xfe, 0x6e, 0x7e, 0xf3, 0xfa, 0xfa, 0x79, 0x71,
        0x7f, 0xf2, 0xf6, 0x76, 0x7a, 0xf4, 0xf4, 0xfe,
        0x76, 0x74, 0x7d, 0xf7, 0xf0, 0x7c, 0x70, 0xf9,
        0xf3, 0xfd, 0x7c, 0x77, 0x7c, 0xf4, 0xf0, 0x76,
        0x6f, 0xfc, 0xfb, 0xf9, 0xfb, 0x77, 0x7c, 0xf8,
        0xf3, 0x7d, 0x6f, 0x7a, 0xf5, 0xf7, 0x7b, 0x76,
        0x7b, 0x7b, 0xf8, 0xf8, 0x6f, 0x78, 0xf3, 0xf9,
        0xfe, 0x79, 0x74, 0xfd},

    {   //0x80, 0x00, 0x01, 0xd2, 0x00, 0x20, 0xbb, 0x10,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xf3, 0xf8, 0x74, 0x78,
        0xf8, 0xfb, 0xfb, 0x79, 0x77, 0xfd, 0xf5, 0xf5,
        0x76, 0x6f, 0x7f, 0xf5, 0xfb, 0x7a, 0x7b, 0x7d,
        0xfd, 0xf5, 0x79, 0x6e, 0xfc, 0xf2, 0xfc, 0x7d,
        0x70, 0x73, 0xfd, 0xf3, 0x7e, 0x73, 0xfb, 0xf9,
        0xf9, 0x7b, 0x71, 0x7b, 0xfc, 0xf3, 0xf7, 0x71,
        0x77, 0xf4, 0xfa, 0x79, 0x77, 0x7d, 0xfe, 0xf1,
        0xf5, 0x71, 0x75, 0xf7, 0xf8, 0xfc, 0x7c, 0x78,
        0xfd, 0xf7, 0xfb, 0x76, 0x76, 0xf9, 0xf5, 0xfa,
        0x77, 0x75, 0x7d, 0xf9, 0xef, 0xfc, 0x71, 0xf9,
        0xf0, 0x7d, 0x7a, 0x7a, 0x79, 0xfb, 0xf0, 0xfe,
        0x6f, 0x7e, 0xf9, 0xfa, 0xfe, 0x76, 0x7e, 0xfa,
        0xfa, 0x7f, 0x71, 0x7c, 0xf2, 0xf1, 0xf8, 0x7c,
        0x7e, 0x7e, 0xf7, 0xf7, 0x71, 0x76, 0xf1, 0xf3,
        0x7d, 0x79, 0x76, 0x7c, 0xf4, 0xf3, 0x7a, 0x77,
        0xf9, 0xf8, 0xfe, 0x76, 0x72, 0xfe, 0xf8, 0xf7,
        0x7f, 0x71, 0x7c, 0xf7, 0xfd, 0x78, 0x7a, 0x7e,
        0xfe, 0xf2, 0x7f, 0x6e, 0xff, 0xf4, 0xfc, 0x7e,
        0x78, 0x79, 0x7f, 0xf9, 0xff, 0x74, 0x7a, 0xf9,
        0xf4, 0x7e, 0x72, 0x7b, 0xfd, 0xf8, 0xf5, 0x78,
        0x76, 0xf7, 0xf7, 0x7d},

    {   //0x80, 0x00, 0x01, 0xd3, 0x00, 0x20, 0xbb, 0xb0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x76, 0x7c, 0x7f, 0xf7,
        0xf3, 0x72, 0x70, 0xfa, 0xf8, 0xfe, 0x7c, 0x79,
        0x7f, 0xfa, 0xfc, 0x79, 0x71, 0x7e, 0xf3, 0xf6,
        0x78, 0x73, 0x7c, 0x7c, 0xfa, 0xfc, 0x71, 0xff,
        0xf0, 0xfb, 0x79, 0x77, 0x78, 0xfe, 0xef, 0xf7,
        0x75, 0x7e, 0xf5, 0xf8, 0x7e, 0x77, 0x7e, 0xf8,
        0xf6, 0xf8, 0x78, 0x74, 0xf8, 0xf0, 0xfb, 0x76,
        0x7d, 0xfb, 0xfc, 0xf8, 0x77, 0x73, 0xf4, 0xf2,
        0xfc, 0x7a, 0x77, 0x7c, 0xfa, 0xf4, 0x7a, 0x74,
        0xfd, 0xf7, 0xfa, 0x78, 0x77, 0xfb, 0xfa, 0xf7,
        0xf9, 0x74, 0x79, 0xf4, 0xf7, 0x78, 0x77, 0xfc,
        0xfa, 0xf5, 0xfd, 0x6f, 0x79, 0xf5, 0xf8, 0xfe,
        0x78, 0x7a, 0x7f, 0xfb, 0x7d, 0x71, 0x75, 0xfa,
        0xf3, 0xff, 0x72, 0x79, 0xfc, 0xfd, 0xf7, 0x7d,
        0x74, 0xfb, 0xf2, 0xff, 0x71, 0x77, 0x7d, 0xfa,
        0xf3, 0x7a, 0x74, 0xf8, 0xf8, 0xfc, 0x7c, 0x78,
        0xfd, 0xf9, 0xf9, 0x7b, 0x6f, 0x7d, 0xf4, 0xf6,
        0x79, 0x77, 0xfb, 0xfe, 0xfc, 0xfc, 0x73, 0x78,
        0xf2, 0xf7, 0x7a, 0x76, 0x79, 0xfd, 0xf6, 0xfa,
        0x73, 0x7a, 0xfa, 0xfa, 0x7f, 0x73, 0x7a, 0xfb,
        0xf6, 0xf8, 0x79, 0x74},

    {   //0x80, 0x00, 0x01, 0xd4, 0x00, 0x20, 0xbc, 0x50,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xfd, 0xf0, 0xfd, 0x75,
        0xfe, 0xf8, 0xfe, 0xf8, 0x7c, 0x6f, 0xfd, 0xf1,
        0xf6, 0x7d, 0x79, 0x7f, 0xfa, 0xf8, 0x7d, 0x76,
        0xfa, 0xf5, 0xf5, 0x7b, 0x71, 0xff, 0xfe, 0xf7,
        0xf7, 0x79, 0x7b, 0xf3, 0xf5, 0x77, 0x75, 0xfc,
        0xfb, 0xfa, 0xf8, 0x73, 0x76, 0xfb, 0xf8, 0xfd,
        0x77, 0x79, 0xfd, 0xfc, 0x7f, 0x77, 0x78, 0xf9,
        0xf3, 0xf8, 0x74, 0x77, 0xfc, 0x7e, 0xf8, 0xfd,
        0x74, 0xfc, 0xf2, 0xfd, 0x6f, 0x76, 0x7f, 0x7d,
        0xf8, 0x7e, 0x73, 0x7d, 0xf9, 0xfa, 0x7a, 0x74,
        0xfb, 0xf2, 0xf8, 0xfc, 0x77, 0x7a, 0xf9, 0xf5,
        0xfe, 0x75, 0xfe, 0xff, 0xfe, 0xfa, 0x71, 0x74,
        0xf6, 0xf4, 0x7d, 0x75, 0x7d, 0xff, 0xfd, 0xfb,
        0x78, 0x79, 0xfa, 0xf5, 0xf9, 0x75, 0x76, 0xf9,
        0xf8, 0xfc, 0x7d, 0x77, 0x7e, 0xf5, 0xfa, 0x72,
        0x78, 0xf9, 0xfc, 0xf6, 0xfe, 0x73, 0x7e, 0xf6,
        0xfb, 0x77, 0x76, 0xfe, 0xfc, 0xfc, 0x7c, 0x75,
        0x7c, 0xf9, 0xf5, 0x7d, 0x6e, 0x7a, 0xfb, 0xfe,
        0xfb, 0x7d, 0x7d, 0xf6, 0xf4, 0x7d, 0x73, 0xfe,
        0xfb, 0xf8, 0xf4, 0x7c, 0x7a, 0xf8, 0xf8, 0x7e,
        0x76, 0x7d, 0xf6, 0xf6},

    {   //0x80, 0x00, 0x01, 0xd5, 0x00, 0x20, 0xbc, 0xf0,
        //0xfd, 0xfc, 0xa2, 0x72,
        0xfc, 0x79, 0x77, 0x7b,
        0xf8, 0xf6, 0x75, 0x76, 0xf8, 0xfd, 0xff, 0xfe,
        0x79, 0xfd, 0xf1, 0xf4, 0x7a, 0x79, 0xfd, 0xfd,
        0xf8, 0x7e, 0x73, 0x7e, 0xf9, 0xf8, 0x7d, 0x73,
        0x7f, 0xf7, 0xfb, 0xfe, 0x79, 0x77, 0xfe, 0xf5,
        0xfe, 0x73, 0x7e, 0xf9, 0x7d, 0x7f, 0x79, 0x75,
        0xfb, 0xf6, 0xfd, 0x75, 0x77, 0x7e, 0xfd, 0xfd,
        0x76, 0x7a, 0xf9, 0xf4, 0xf7, 0x75, 0x73, 0xfb,
        0xfb, 0xff, 0xff, 0x79, 0x7a, 0xf8, 0xfa, 0x74,
        0x75, 0xfa, 0xfa, 0xf9, 0xfb, 0x75, 0x7d, 0xfb,
        0xfa, 0x7c, 0x75, 0xfc, 0xf9, 0xf9, 0x7e, 0x76,
        0x7c, 0xfb, 0xf5, 0xfc, 0x72, 0x7c, 0xf9, 0x7e,
        0xfe, 0x7f, 0x79, 0xfd, 0xf2, 0xfc, 0x74, 0x7a,
        0xfc, 0xfd, 0xfa, 0x7c, 0x7a, 0xf8, 0xf7, 0xfb,
        0x78, 0x78, 0xfa, 0xf6, 0xfa, 0x7d, 0x7a, 0x7e,
        0xf8, 0xf4, 0x79, 0x76, 0xf6, 0xfb, 0x7d, 0x7e,
        0x77, 0x7a, 0xf6, 0xf3, 0x7b, 0x74, 0x7f, 0xfb,
        0xfc, 0x7c, 0x77, 0xfd, 0xf9, 0xf9, 0xfe, 0x71,
        0x78, 0xf8, 0xf7, 0xfc, 0x7c, 0x7c, 0xfe, 0xf4,
        0xfd, 0x71, 0x7f, 0xf5, 0xf9, 0xfa, 0x7e, 0x77,
        0x7f, 0xf7, 0xfb, 0x78},

    {   //0x80, 0x00, 0x01, 0xd6, 0x00, 0x20, 0xbd, 0x90,
        //0xfd, 0xfc, 0xa2, 0x72,
        0x77, 0xfd, 0xfc, 0xfe,
        0x76, 0x77, 0xff, 0xfe, 0xf9, 0x78, 0x71, 0xff,
        0xfc, 0x7d, 0x7d, 0x7b, 0x7f, 0xf7, 0xf5, 0x78,
        0x72, 0xfb, 0xfa, 0xfb, 0xfd, 0x7a, 0x7a, 0xfc,
        0xf7, 0x7f, 0x77, 0x7e, 0xf5, 0xf8, 0x7e, 0x78,
        0x7d, 0xfd, 0xfb, 0xf8, 0x75, 0x77, 0xfa, 0xff,
        0x7c, 0x77, 0x7a, 0xfc, 0xf1, 0xf9, 0x73, 0x79,
        0xfc, 0xfd, 0xfc, 0x7f, 0x7c, 0xfb, 0xf6, 0xf9,
        0x79, 0x72, 0x7f, 0xf6, 0xfc, 0x7b, 0x7c, 0xfe,
        0xff, 0xfc, 0x7c, 0x74, 0xfb, 0xf7, 0xfc, 0xff,
        0x78, 0x79, 0xfa, 0xf4, 0x7b, 0x75, 0xfe, 0xfb,
        0xfc, 0xfd, 0x7b, 0x7f, 0xfa, 0xf6, 0xf6, 0x77,
        0x79, 0xf7, 0xf7, 0x7e, 0x7b, 0xfe, 0xfb, 0xf6,
        0xf8, 0x77, 0x79, 0xf7, 0xfb, 0xf9, 0x7c, 0x76,
        0xfe, 0xf9, 0xfd, 0x77, 0x79, 0xfa, 0xf9, 0xfe,
        0x78, 0x77, 0x7d, 0xfd, 0xf6, 0xff, 0x71, 0x7e,
        0xf8, 0x7c, 0x78, 0x7c, 0xfd, 0xfb, 0xf7, 0x7b,
        0x6f, 0x79, 0xfe, 0xfc, 0xfd, 0x7a, 0x7d, 0xfb,
        0xf9, 0x7f, 0x75, 0x7d, 0xf7, 0xf7, 0x7e, 0x79,
        0x7d, 0x7c, 0xfc, 0xf8, 0x77, 0x77, 0xf5, 0xf8,
        0x7d, 0x7b, 0x7a, 0x7e},
};

/* IEEE 1451.0 (2007) -> Cap. 5.10
 -> TIM-Initiated Message
 Sampling modes: Continuous sampling mode
 Data transmission mode: Streaming when a buffer is full mode
 Streaming operation = Continuous sampling + Streaming when a buffer is full

 -> Polling
 Sampling modes: Continuous sampling mode
 Data transmission mode: Only when commanded mode */

